@@ -31,8 +31,8 @@
 #include "voicesignature.h"
 
 #include "input.h"
-#include "windows.h"
-
+#include "windows_handler.h"
+#include "triggers/gestures.h"
 
 class KConfig;
 class QKeySequence;
@@ -69,6 +69,12 @@ class KDE_EXPORT Trigger
         static Trigger* create_cfg_read( KConfigGroup& cfg_P, ActionData* data_P );
         virtual void activate( bool activate_P ) = 0;
 
+        //! Disable the trigger
+        virtual void disable();
+
+        //! Enable the trigger
+        virtual void enable();
+
         /**
          * The trigger will be erased permanently
          */
@@ -94,7 +100,7 @@ class KDE_EXPORT Trigger_list
     public:
         Trigger_list( const QString& comment_P ); // CHECKME nebo i data ?
         Trigger_list( KConfigGroup& cfg_P, ActionData* data_P );
-        ~Trigger_list();
+        virtual ~Trigger_list();
         void activate( bool activate_P );
         void cfg_write( KConfigGroup& cfg_P ) const;
         //! Some convenience typedef
@@ -103,6 +109,12 @@ class KDE_EXPORT Trigger_list
         const QString comment() const;
         Trigger_list* copy( ActionData* data_P ) const;
 
+        //! Disable the trigger
+        virtual void disable();
+
+        //! Enable the trigger
+        virtual void enable();
+
         /**
          * @reimp
          */
@@ -119,12 +131,12 @@ class KDE_EXPORT ShortcutTrigger
 
     typedef Trigger base;
     public:
-        ShortcutTrigger( 
+        ShortcutTrigger(
             ActionData* data_P,
             const KShortcut& shortcut_P,
             const QUuid &uuid = QUuid::createUuid() );
 
-        ShortcutTrigger( 
+        ShortcutTrigger(
             KConfigGroup& cfg_P,
             ActionData* data_P );
 
@@ -144,6 +156,12 @@ class KDE_EXPORT ShortcutTrigger
          */
         void aboutToBeErased();
 
+        //! Disable the trigger
+        virtual void disable();
+
+        //! Enable the trigger
+        virtual void enable();
+
     Q_SIGNALS:
 
         //! Emitted when the global shortcut is changed from somewhere else
@@ -158,68 +176,132 @@ class KDE_EXPORT ShortcutTrigger
 
         //! A persistent identifier for this shortcut
         QUuid _uuid;
+
+        //! Are the conditions met?
+        bool _conditions_met;
+
+        /**
+         * The Key Sequence associated with this Trigger. This is needed
+         * because a inactive trigger doesn't register it's shortcut with
+         * kde's global shortcuts registry so we have to remember the shortcut
+         * ourselves. Beware of synchronizing saved state, global shortcuts
+         * registry state and state of this var :-) .
+         */
+        KShortcut    _shortcut;
+
+
+
     };
 
-class KDE_EXPORT WindowTrigger
-    : public QObject, public Trigger
+
+class KDE_EXPORT WindowTrigger : public QObject, public Trigger
     {
     Q_OBJECT
+
+    Q_FLAGS(WindowEvents)
+
     typedef Trigger base;
+
     public:
+
         enum window_action_t
             {
-            WINDOW_APPEARS         = ( 1 << 0 ),
-            WINDOW_DISAPPEARS      = ( 1 << 1 ),
-            WINDOW_ACTIVATES       = ( 1 << 2 ),
-            WINDOW_DEACTIVATES     = ( 1 << 3 )
+            NONE                   = 0,
+            WINDOW_APPEARS         = ( 1 << 0 ),        //!< The window is opened
+            WINDOW_DISAPPEARS      = ( 1 << 1 ),        //!< The window is closed
+            WINDOW_ACTIVATES       = ( 1 << 2 ),        //!< The window gets the focus
+            WINDOW_DEACTIVATES     = ( 1 << 3 )         //!< The window loses the focus
             };
-        WindowTrigger( ActionData* data_P, Windowdef_list* windows_P, int window_actions_P );
+
+        Q_DECLARE_FLAGS(WindowEvents, window_action_t)
+
+        WindowTrigger(
+                ActionData* data_P,
+                Windowdef_list* windowslist = NULL,
+                WindowEvents window_actions = 0 );
+
         WindowTrigger( KConfigGroup& cfg_P, ActionData* data_P );
+
+        void setOnWindowEvents(WindowEvents events);
+
         virtual ~WindowTrigger();
         virtual void cfg_write( KConfigGroup& cfg_P ) const;
         virtual WindowTrigger* copy( ActionData* data_P ) const;
         virtual const QString description() const;
         const Windowdef_list* windows() const;
+        Windowdef_list* windows();
         bool triggers_on( window_action_t w_action_P ) const;
         virtual void activate( bool activate_P );
 
         virtual TriggerType type() const { return WindowTriggerType; }
-    protected: // CHECKME neco private ?
-        Windowdef_list* _windows;
-        int window_actions;
-        void init();
-        typedef QMap< WId, bool > Windows_map;
-        Windows_map existing_windows;
-        WId last_active_window;
+
+
     protected Q_SLOTS:
         void window_added( WId window_P );
         void window_removed( WId window_P );
         void active_window_changed( WId window_P );
         void window_changed( WId window_P, unsigned int dirty_P );
-    protected:
+
+    private:
+
+        //! Useful code for all constructors
+        void init();
+
+        Windowdef_list* _windows;
+
+        WindowEvents window_actions;
+
+        typedef QMap< WId, bool > Windows_map;
+
+        //! Internal cache. Remembers if a window is a match or not,
+        Windows_map existing_windows;
+
+        //! The last active window
+        WId last_active_window;
+
+        //! Is the trigger active?
         bool active;
     };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(WindowTrigger::WindowEvents)
+
+/**
+ * This class handles the storage of gesture data; it also matches gestures
+ * and links the gesture to an action.
+ * One object equals one configured gesture.
+ */
+
 class KDE_EXPORT GestureTrigger
     : public QObject, public Trigger
     {
     Q_OBJECT
     typedef Trigger base;
     public:
-        GestureTrigger( ActionData* data_P, const QString& gesture_P );
+        GestureTrigger( ActionData* data_P, const StrokePoints& pointdata_P = StrokePoints() );
         GestureTrigger( KConfigGroup& cfg_P, ActionData* data_P );
         virtual ~GestureTrigger();
         virtual void cfg_write( KConfigGroup& cfg_P ) const;
         virtual Trigger* copy( ActionData* data_P ) const;
         virtual const QString description() const;
-        const QString& gesturecode() const;
+        const StrokePoints& pointData() const;
+
+        //! Set the point data of the gesture
+        void setPointData(const StrokePoints &data);
+
         virtual void activate( bool activate_P );
 
         virtual TriggerType type() const { return GestureTriggerType; }
     protected Q_SLOTS:
-        void handle_gesture( const QString& gesture_P, WId window_P );
+        void handle_gesture( const StrokePoints& gesture_P );
+    Q_SIGNALS:
+        void gotScore( ActionData* const data, const qreal score );
     private:
-        QString _gesturecode;
+        void importKde3Gesture(KConfigGroup& cfg_P);
+
+        qreal comparePointData(const StrokePoints &a, const StrokePoints &b) const;
+        inline qreal angleSquareDifference(qreal alpha, qreal beta) const;
+
+        StrokePoints _pointdata;
     };
 
 
@@ -240,7 +322,7 @@ class KDE_EXPORT Voice_trigger
         const QString& voicecode() const;
         virtual void activate( bool activate_P );
         VoiceSignature voicesignature( int ech ) const;
-        
+
         virtual TriggerType type() const { return SoundTrigger; }
     public slots:
         void handle_Voice(  );
@@ -264,7 +346,8 @@ VoiceSignature Voice_trigger::voicesignature(int ech) const
     {
     return _voicesignature[ech-1];
     }
-#endif 
+#endif
 } // namespace KHotKeys
 
+
 #endif
