@@ -75,12 +75,11 @@
 
 #define PASSDLG_HIDE_TIMEOUT 10000
 
-
 //===========================================================================
 //
 // Simple dialog for entering a password.
 //
-PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin)
+PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, const QString &text)
     : KDialog(parent, Qt::X11BypassWindowManagerHint),
       mPlugin( plugin ),
       mCapsLocked(-1),
@@ -92,9 +91,13 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin)
     pixLabel->setPixmap(DesktopIcon("system-lock-screen"));
 
     KUser user; QString fullName=user.property(KUser::FullName).toString();
-    QLabel *greetLabel = new QLabel( fullName.isEmpty() ?
+    QString greetString = text;
+    if (text.isEmpty()) {
+        greetString = fullName.isEmpty() ?
             i18n("<nobr><b>The session is locked</b></nobr><br />") :
-            i18n("<nobr><b>The session was locked by %1</b></nobr><br />", fullName ), w );
+            i18n("<nobr><b>The session was locked by %1</b></nobr><br />", fullName );
+    }
+    QLabel *greetLabel = new QLabel(greetString, w);
 
     mStatusLabel = new QLabel( "<b> </b>", w );
     mStatusLabel->setAlignment( Qt::AlignCenter );
@@ -148,7 +151,7 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin)
     connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
     connect(mNewSessButton, SIGNAL(clicked()), SLOT(slotSwitchUser()));
 
-    if (!KDisplayManager().isSwitchable() || !KAuthorized::authorizeKAction("switch_user"))
+    if (!text.isEmpty() || !KDisplayManager().isSwitchable() || !KAuthorized::authorizeKAction("switch_user"))
         mNewSessButton->hide();
 
     installEventFilter(this);
@@ -158,7 +161,7 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin)
     connect(qApp, SIGNAL(activity()), SLOT(slotActivity()) );
 
     greet->start();
-    
+
     capsLocked();
 }
 
@@ -199,7 +202,7 @@ void PasswordDlg::timerEvent(QTimerEvent *ev)
 {
     if (ev->timerId() == mTimeoutTimerId)
     {
-        reject();
+        done(TIMEOUT_CODE);
     }
     else if (ev->timerId() == mFailedTimerId)
     {
@@ -300,11 +303,11 @@ bool PasswordDlg::GRecvArr (char **ret)
         return false;
     *ret = buf;
     if (GRead (buf, len)) {
-	return true;
+        return true;
     } else {
-	::free(buf);
-	*ret = 0;
-	return false;
+        ::free(buf);
+        *ret = 0;
+        return false;
     }
 }
 
@@ -416,7 +419,7 @@ void PasswordDlg::cantCheck()
     greet->failed();
     static_cast< LockProcess* >(parent())->msgBox( this, QMessageBox::Critical,
         i18n("Cannot unlock the session because the authentication system failed to work;\n"
-             "you must kill krunner_lock (pid %1) manually.", getpid()) );
+             "you must kill kscreenlocker (pid %1) manually.", getpid()) );
     greet->revive();
 }
 
@@ -485,7 +488,7 @@ void PasswordDlg::slotOK()
 void PasswordDlg::setVisible( bool visible )
 {
     QDialog::setVisible( visible );
-  
+
     if ( visible )
         QApplication::flush();
 }
@@ -544,12 +547,12 @@ void PasswordDlg::slotStartNewSession()
 class LockListViewItem : public QTreeWidgetItem {
 public:
     LockListViewItem( QTreeWidget *parent,
-		      const QString &sess, const QString &loc, int _vt )
-	: QTreeWidgetItem( parent )
-	, vt( _vt )
+                      const QString &sess, const QString &loc, int _vt )
+        : QTreeWidgetItem( parent )
+        , vt( _vt )
     {
-	setText( 0, sess );
-	setText( 1, loc );
+        setText( 0, sess );
+        setText( 1, loc );
     }
 
     int vt;
@@ -587,7 +590,7 @@ void PasswordDlg::slotSwitchUser()
         QTreeWidgetItem *itm = 0;
         QString user, loc;
         int ns = 0;
-        for (SessList::ConstIterator it = sess.begin(); it != sess.end(); ++it) {
+        for (SessList::ConstIterator it = sess.constBegin(); it != sess.constEnd(); ++it) {
             KDisplayManager::sess2Str2( *it, user, loc );
             itm = new LockListViewItem( lv, user, loc, (*it).vt );
             if (!(*it).vt)
