@@ -48,6 +48,7 @@ Workspace::Workspace( QWidget* parent)
                                "to create a new tab (Menu File->New) before "
                                "you can drag sensors here." ) );
   setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
+  connect(&mDirWatch, SIGNAL(deleted(const QString&)), this, SLOT(removeWorkSheet(const QString &)));
 }
 
 Workspace::~Workspace()
@@ -62,6 +63,7 @@ void Workspace::saveProperties( KConfigGroup& cfg )
       list.append( mSheetList.at(i)->fileName() );
 
   cfg.writePathEntry( "SelectedSheets", list );
+  cfg.writeEntry( "currentSheet", currentIndex() );
 }
 
 void Workspace::readProperties( const KConfigGroup& cfg )
@@ -74,7 +76,7 @@ void Workspace::readProperties( const KConfigGroup& cfg )
     * probably the first time the user has started KSysGuard. We
     * then "restore" a special default configuration. */
     selectedSheets << "ProcessTable.sgrd";
-    selectedSheets << "SystemLoad.sgrd";
+    selectedSheets << "SystemLoad2.sgrd";
   } else if(selectedSheets[0] != "ProcessTable.sgrd") {
     //We need to make sure that this is really is the process table on the first tab. No GUI way of changing this, but should make sure anyway.
     //Plus this migrates users from the kde3 setup
@@ -82,6 +84,11 @@ void Workspace::readProperties( const KConfigGroup& cfg )
     selectedSheets.prepend( "ProcessTable.sgrd");
   }
 
+  int oldSystemLoad = selectedSheets.indexOf("SystemLoad.sgrd");
+  if(oldSystemLoad != -1) {
+    selectedSheets.replace(oldSystemLoad, "SystemLoad2.sgrd");
+  }
+
   KStandardDirs* kstd = KGlobal::dirs();
   QString filename;
   for ( QStringList::Iterator it = selectedSheets.begin(); it != selectedSheets.end(); ++it ) {
@@ -90,9 +97,13 @@ void Workspace::readProperties( const KConfigGroup& cfg )
       restoreWorkSheet( filename, false);
     }
   }
-  //We know that the first tab is the process table
-  setCurrentIndex(0);
-  emit currentChanged(0);
+
+  int idx = cfg.readEntry( "currentSheet", 0 );
+  if (idx < 0 || idx > count() - 1) {
+    idx = 0;
+  }
+  setCurrentIndex(idx);
+  emit currentChanged(idx);
 }
 
 QString Workspace::makeNameForNewSheet() const
@@ -266,9 +277,10 @@ void Workspace::removeAllWorkSheets()
 
 void Workspace::removeWorkSheet( const QString &fileName )
 {
+  QString baseName = fileName.right( fileName.length() - fileName.lastIndexOf( '/' ) - 1 );
   for(int i = 0; i < mSheetList.size(); i++) {
     WorkSheet *sheet = mSheetList.at(i);
-    if ( sheet->fileName() == fileName ) {
+    if ( sheet->fileName() == baseName ) {
       removeTab(indexOf( sheet ));
       mSheetList.removeAt( i );
       delete sheet;
@@ -277,17 +289,28 @@ void Workspace::removeWorkSheet( const QString &fileName )
   }
 }
 
+void Workspace::uploadHotNewWorksheet()
+{
+  WorkSheet *currentWorksheet = (WorkSheet*)currentWidget();
+
+  KMessageBox::information(this, i18n("<qt>To propose the current custom tab as a new System Monitor tab, email <br><a href=\"file:%1\">%2</a><br> to <a href=\"mailto:john.tapsell@kde.org?subject='System Monitor Tab'&attach='file://%2'\">john.tapsell@kde.org</a></qt>", currentWorksheet->fullFileName().section('/',0,-2), currentWorksheet->fullFileName()), i18n("Upload custom System Monitor tab"), QString::null, KMessageBox::AllowLink);
+}
 void Workspace::getHotNewWorksheet()
 {
-  kDebug() << "Need to handle get new stuff";
-  
   KNS::Engine engine(this);
   if(engine.init("ksysguard.knsrc"))
   {
-     KNS::Entry::List entries = engine.downloadDialogModal(this);
-     //TODO: inspect entries here
-     //Don't qDeleteAll entry it's already done when engine is deleted otherwise double delete
-     //qDeleteAll(entries);
+    KNS::Entry::List entries = engine.downloadDialogModal(this);
+    foreach(KNS::Entry *entry, entries) {
+      if(entry->status() == KNS::Entry::Installed && !entry->installedFiles().isEmpty()) {
+        QString filename = entry->installedFiles().first();
+        restoreWorkSheet(filename, true);
+      }
+    }
+
+    //TODO: inspect entries here
+    //Don't qDeleteAll entry it's already done when engine is deleted otherwise double delete
+    //qDeleteAll(entries);
   }
 }
 
@@ -316,6 +339,9 @@ bool Workspace::restoreWorkSheet( const QString &fileName, bool switchToTab)
   if(switchToTab)
    setCurrentIndex(indexOf(sheet));
 
+  //Watch the file incase it is deleted
+  mDirWatch.addFile(fileName);
+
   return true;
 }
 
