@@ -58,6 +58,7 @@ namespace KWin
 class Workspace;
 class Client;
 class Bridge;
+class PaintRedirector;
 
 class Client
     : public Toplevel
@@ -103,6 +104,7 @@ class Client
         QSize maxSize() const;
         virtual QPoint clientPos() const; // Inside of geometry()
         virtual QSize clientSize() const;
+        virtual QRect visibleRect() const;
 
         bool windowEvent( XEvent* e );
         virtual bool eventFilter( QObject* o, QEvent* e );
@@ -201,7 +203,7 @@ class Client
 
         void updateDecoration( bool check_workspace_pos, bool force = false );
         bool checkBorderSizes( bool also_resize );
-        void repaintDecoration();
+        void triggerDecorationRepaint();
 
         void updateShape();
 
@@ -280,7 +282,14 @@ class Client
         void cancelShadeHoverTimer();
         void cancelAutoRaise();
         void checkActiveModal();
+        StrutRect strutRect( StrutArea area ) const;
+        StrutRects strutRects() const;
         bool hasStrut() const;
+        /**
+         * Whether or not the window has a strut that expands through the invisible area of
+         * an xinerama setup where the monitors are not the same resolution.
+         */
+        bool hasOffscreenXineramaStrut() const;
 
         bool isMove() const 
             {
@@ -292,10 +301,25 @@ class Client
             }
 
         // Decorations <-> Effects
-        QList<QRect> shadowQuads( ShadowType type ) const;
-        double shadowOpacity( ShadowType type ) const;
-        double shadowBrightness( ShadowType type ) const;
-        double shadowSaturation( ShadowType type ) const;
+        const QPixmap *topDecoPixmap() const { return &decorationPixmapTop; } 
+        const QPixmap *leftDecoPixmap() const { return &decorationPixmapLeft; } 
+        const QPixmap *bottomDecoPixmap() const { return &decorationPixmapBottom; } 
+        const QPixmap *rightDecoPixmap() const { return &decorationPixmapRight; } 
+
+        bool decorationPixmapRequiresRepaint();
+        void ensureDecorationPixmapsPainted();
+
+        QRect decorationRect() const {
+            return (decoration && decoration->widget()) ?
+                        decoration->widget()->rect().translated(-padding_left, -padding_top) :
+                        QRect(0, 0, width(), height());
+        }
+
+        enum CoordinateMode {
+            DecorationRelative, // Relative to the top left corner of the decoration
+            WindowRelative      // Relative to the top left corner of the window
+        };
+        void layoutDecorationRects(QRect &left, QRect &top, QRect &right, QRect &bottom, CoordinateMode mode) const;
 
     private slots:
         void autoRaise();
@@ -351,6 +375,7 @@ class Client
         void demandAttentionKNotify();
         void syncTimeout();
         void delayedSetShortcut();
+        void repaintDecorationPending();
 
     private:
         void exportMappingState( int s ); // ICCCM 4.1.3.1, 4.1.4, NETWM 2.5.1
@@ -373,9 +398,7 @@ class Client
         void finishWindowRules();
         void setShortcutInternal( const KShortcut& cut );
 
-        void updateWorkareaDiffs();
         void checkDirection( int new_diff, int old_diff, QRect& rect, const QRect& area );
-        static int computeWorkareaDiff( int left, int right, int a_left, int a_right );
         void configureRequest( int value_mask, int rx, int ry, int rw, int rh, int gravity, bool from_tool );
         NETExtendedStrut strut() const;
         int checkShadeGeometry( int w, int h );
@@ -416,6 +439,8 @@ class Client
         void updateHiddenPreview();
 
         void updateInputShape();
+        void repaintDecorationPixmap( QPixmap& pix, const QRect& r, const QPixmap& src, QRegion reg );
+        void resizeDecorationPixmaps();
 
         Time readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStartupInfoData* asn_data,
             bool session ) const;
@@ -505,7 +530,6 @@ class Client
         QRect geom_restore;
         QRect geom_fs_restore;
         MaximizeMode maxmode_restore;
-        int workarea_diff_x, workarea_diff_y;
         QTimer* autoRaiseTimer;
         QTimer* shadeHoverTimer;
         QTimer* delayedMoveResizeTimer;
@@ -529,6 +553,7 @@ class Client
             };
         PendingGeometry_t pending_geometry_update;
         QRect geom_before_block;
+        QRect deco_rect_before_block;
         bool shade_geometry_change;
 #ifdef HAVE_XSYNC
         XSyncCounter sync_counter;
@@ -538,6 +563,7 @@ class Client
         QTimer* sync_timeout;
         bool sync_resize_pending;
         int border_left, border_right, border_top, border_bottom;
+        int padding_left, padding_right, padding_top, padding_bottom;
         QRegion _mask;
         static bool check_active_modal; ///< \see Client::checkActiveModal()
         KShortcut _shortcut;
@@ -547,6 +573,8 @@ class Client
         friend struct ResetupRulesProcedure;
         friend class GeometryUpdatesBlocker;
         QTimer* demandAttentionKNotifyTimer;
+        QPixmap decorationPixmapLeft, decorationPixmapRight, decorationPixmapTop, decorationPixmapBottom;
+        PaintRedirector* paintRedirector;
 
         friend bool performTransiencyCheck();
     };
@@ -758,6 +786,11 @@ inline QSize Client::clientSize() const
     return client_size;
     }
 
+inline QRect Client::visibleRect() const
+   {
+   return geometry().adjusted( -padding_left, -padding_top, padding_right, padding_bottom );
+   }
+
 inline void Client::setGeometry( const QRect& r, ForceGeometry_t force )
     {
     setGeometry( r.x(), r.y(), r.width(), r.height(), force );
@@ -815,34 +848,6 @@ inline bool Client::hiddenPreview() const
     return mapping_state == Kept;
     }
 
-inline QList<QRect> Client::shadowQuads( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowQuads( type );
-    return QList<QRect>();
-    }
-
-inline double Client::shadowOpacity( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowOpacity( type );
-    return 1.0;
-    }
-
-inline double Client::shadowBrightness( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowBrightness( type );
-    return 1.0;
-    }
-
-inline double Client::shadowSaturation( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowSaturation( type );
-    return 1.0;
-    }
-
 KWIN_COMPARE_PREDICATE( WrapperIdMatchPredicate, Client, Window, cl->wrapperId() == value );
 
 } // namespace
