@@ -36,10 +36,8 @@ namespace Oxygen
 // OxygenFactory /////////////////////////////////////////////////////////////
 
 static const int OXYGEN_BUTTONSIZE      = 22;
+static const qreal SHADOW_WIDTH         = 25.5;
 #define TFRAMESIZE 3
-#define BFRAMESIZE 5
-#define LFRAMESIZE 5
-#define RFRAMESIZE 5
 
 enum ButtonType {
     ButtonHelp=0,
@@ -63,15 +61,12 @@ public:
     virtual KDecoration *createDecoration(KDecorationBridge *b);
     virtual bool reset(unsigned long changed);
     virtual bool supports( Ability ability ) const;
-
-    virtual QList< QList<QImage> > shadowTextures();
-    virtual int shadowTextureList( ShadowType type ) const;
-    virtual QList<QRect> shadowQuads( ShadowType type, QSize size ) const;
-    virtual double shadowOpacity( ShadowType type ) const;
+    QList< BorderSize > borderSizes() const;
 
     static bool initialized();
     static Qt::Alignment titleAlignment();
     static bool showStripes();
+    static int borderSize();
     static bool blendTitlebarColors();
 
 private:
@@ -81,6 +76,7 @@ private:
     static bool initialized_;
     static Qt::Alignment titleAlignment_;
     static bool showStripes_;
+    static int borderSize_;
     static bool blendTitlebarColors_;
 };
 
@@ -96,6 +92,9 @@ inline bool OxygenFactory::blendTitlebarColors()
 inline bool OxygenFactory::showStripes()
     { return showStripes_; }
 
+inline int OxygenFactory::borderSize()
+    { return borderSize_; }
+
 } //namespace Oxygen
 } //namespace Ozone
 
