@@ -108,14 +108,22 @@ void Workspace::setupCompositing()
         switch( c )
             {
             case 'O':
+                kDebug( 1212 ) << "Compositing forced to OpenGL mode by environment variable";
                 type = OpenGLCompositing;
                 break;
             case 'X':
+                kDebug( 1212 ) << "Compositing forced to XRender mode by environment variable";
                 type = XRenderCompositing;
                 break;
+            case 'N':
+                if( getenv( "KDE_FAILSAFE" ))
+                    kDebug( 1212 ) << "Compositing disabled forcefully by KDE failsafe mode";
+                else
+                    kDebug( 1212 ) << "Compositing disabled forcefully by environment variable";
+                return; // Return not break
             default:
-                kDebug( 1212 ) << "No compositing";
-                return;
+                kDebug( 1212 ) << "Unknown KWIN_COMPOSE mode set, ignoring";
+                break;
             }
         }
 
@@ -175,10 +183,8 @@ void Workspace::setupCompositing()
         { // autoconfigure refresh rate based on XRandR info
         if( Extensions::randrAvailable() )
             {
-            XRRScreenConfiguration *config;
-
-            config = XRRGetScreenInfo( display(), rootWindow() );
-            rate = XRRConfigCurrentRate( config );
+            XRRScreenConfiguration *config = XRRGetScreenInfo( display(), rootWindow() );
+            rate = xrrRefreshRate = XRRConfigCurrentRate( config );
             XRRFreeScreenConfigInfo( config );
             }
         }
@@ -277,7 +283,27 @@ void Workspace::slotToggleCompositing()
     {
     suspendCompositing( !compositingSuspended );
     }
-    
+
+// for the dbus call
+void Workspace::toggleCompositing()
+    {
+    slotToggleCompositing();
+    if( compositingSuspended )
+        {
+        // when disabled show a shortcut how the user can get back compositing
+        QString shortcut, message;
+        if( KAction* action = qobject_cast<KAction*>( keys->action("Suspend Compositing")))
+            shortcut = action->globalShortcut().primary().toString(QKeySequence::NativeText);
+        if( !shortcut.isEmpty() && options->useCompositing )
+            {
+            // display notification only if there is the shortcut
+            message = i18n( "Compositing has been suspended by another application.<br/>"
+                "You can resume using the '%1' shortcut.", shortcut );
+            Notify::raise( Notify::CompositingSuspendedDbus, message );
+            }
+        }
+    }
+
 void Workspace::suspendCompositing()
     {
     suspendCompositing( true );
@@ -288,6 +314,16 @@ void Workspace::suspendCompositing( bool suspend )
     compositingSuspended = suspend;
     finishCompositing();
     setupCompositing(); // will do nothing if suspended
+    emit compositingToggled( !compositingSuspended );
+    }
+
+void Workspace::resetCompositing()
+    {
+    if( compositing())
+        {
+        finishCompositing();
+        QTimer::singleShot( 0, this, SLOT( setupCompositing()));
+        }
     }
 
 void Workspace::addRepaint( int x, int y, int w, int h )
@@ -465,6 +501,13 @@ void Workspace::checkCompositePaintTime( int msec )
     {
     if( options->disableCompositingChecks )
         return;
+    // Sanity check. QTime uses the system clock so if the user changes the time or
+    // timezone our timer will return undefined results. Ideally we would use a system
+    // clock independent timer but I am uncertain if Qt provides a nice wrapper for
+    // one or not. As it's unlikely for a single paint to take 15 seconds it seems
+    // like a good upper bound.
+    if( msec < 0 || msec > 15000 )
+        return;
     composite_paint_times.prepend( msec );
     bool tooslow = false;
     // If last 3 paints were way too slow, disable and warn.
@@ -892,6 +935,7 @@ void Client::finishCompositing()
     {
     Toplevel::finishCompositing();
     updateVisibility();
+    triggerDecorationRepaint();
     }
 
 bool Client::shouldUnredirect() const
