@@ -52,6 +52,7 @@ EffectsHandlerImpl::EffectsHandlerImpl(CompositingType type)
     , keyboard_grab_effect( NULL )
     , fullscreen_effect( 0 )
     , next_window_quad_type( EFFECT_QUAD_TYPE_START )
+    , mouse_poll_ref_count( 0 )
     {
     reconfigure();
     }
@@ -191,16 +192,6 @@ bool EffectsHandlerImpl::hasDecorationShadows() const
     return Workspace::self()->hasDecorationShadows();
     }
 
-QList< QList<QImage> > EffectsHandlerImpl::shadowTextures()
-    {
-    return Workspace::self()->decorationShadowTextures();
-    }
-
-int EffectsHandlerImpl::shadowTextureList( ShadowType type ) const
-    {
-    return Workspace::self()->decorationShadowTextureList( type );
-    }
-
 // start another painting pass
 void EffectsHandlerImpl::startPaint()
     {
@@ -354,6 +345,33 @@ void EffectsHandlerImpl::grabbedKeyboardEvent( QKeyEvent* e )
         keyboard_grab_effect->grabbedKeyboardEvent( e );
     }
 
+const void* EffectsHandlerImpl::getProxy( QString name )
+    {
+    // All effects start with "kwin4_effect_", prepend it to the name
+    name.prepend( "kwin4_effect_" );
+
+    for( QVector< EffectPair >::iterator it = loaded_effects.begin(); it != loaded_effects.end(); ++it)
+        if ( (*it).first == name )
+            return (*it).second->proxy();
+
+    return NULL;
+    }
+
+void EffectsHandlerImpl::startMousePolling()
+    {
+    if( !mouse_poll_ref_count ) // Start timer if required
+        Workspace::self()->startMousePolling();
+    mouse_poll_ref_count++;
+    }
+
+void EffectsHandlerImpl::stopMousePolling()
+    {
+    assert( mouse_poll_ref_count );
+    mouse_poll_ref_count--;
+    if( !mouse_poll_ref_count ) // Stop timer if required
+        Workspace::self()->stopMousePolling();
+    }
+
 bool EffectsHandlerImpl::hasKeyboardGrab() const
     {
     return keyboard_grab_effect != NULL;
@@ -418,24 +436,49 @@ void EffectsHandlerImpl::setCurrentDesktop( int desktop )
     Workspace::self()->setCurrentDesktop( desktop );
     }
 
-QString EffectsHandlerImpl::desktopName( int desktop ) const
+QSize EffectsHandlerImpl::desktopGridSize() const
     {
-    return Workspace::self()->desktopName( desktop );
+    return Workspace::self()->desktopGridSize();
     }
 
-void EffectsHandlerImpl::calcDesktopLayout(int* x, int* y, Qt::Orientation* orientation) const
+int EffectsHandlerImpl::desktopGridWidth() const
     {
-    Workspace::self()->calcDesktopLayout( x, y, orientation );
+    return Workspace::self()->desktopGridWidth();
     }
 
-bool EffectsHandlerImpl::optionRollOverDesktops() const
+int EffectsHandlerImpl::desktopGridHeight() const
     {
-    return options->rollOverDesktops;
+    return Workspace::self()->desktopGridHeight();
     }
 
-int EffectsHandlerImpl::desktopToLeft( int desktop, bool wrap ) const
+int EffectsHandlerImpl::workspaceWidth() const
     {
-    return Workspace::self()->desktopToLeft( desktop, wrap );
+    return Workspace::self()->workspaceWidth();
+    }
+
+int EffectsHandlerImpl::workspaceHeight() const
+    {
+    return Workspace::self()->workspaceHeight();
+    }
+
+int EffectsHandlerImpl::desktopAtCoords( QPoint coords ) const
+    {
+    return Workspace::self()->desktopAtCoords( coords );
+    }
+
+QPoint EffectsHandlerImpl::desktopGridCoords( int id ) const
+    {
+    return Workspace::self()->desktopGridCoords( id );
+    }
+
+QPoint EffectsHandlerImpl::desktopCoords( int id ) const
+    {
+    return Workspace::self()->desktopCoords( id );
+    }
+
+int EffectsHandlerImpl::desktopAbove( int desktop, bool wrap ) const
+    {
+    return Workspace::self()->desktopAbove( desktop, wrap );
     }
 
 int EffectsHandlerImpl::desktopToRight( int desktop, bool wrap ) const
@@ -443,14 +486,39 @@ int EffectsHandlerImpl::desktopToRight( int desktop, bool wrap ) const
     return Workspace::self()->desktopToRight( desktop, wrap );
     }
 
-int EffectsHandlerImpl::desktopUp( int desktop, bool wrap ) const
+int EffectsHandlerImpl::desktopBelow( int desktop, bool wrap ) const
     {
-    return Workspace::self()->desktopUp( desktop, wrap );
+    return Workspace::self()->desktopBelow( desktop, wrap );
     }
 
-int EffectsHandlerImpl::desktopDown( int desktop, bool wrap ) const
+int EffectsHandlerImpl::desktopToLeft( int desktop, bool wrap ) const
     {
-    return Workspace::self()->desktopDown( desktop, wrap );
+    return Workspace::self()->desktopToLeft( desktop, wrap );
+    }
+
+bool EffectsHandlerImpl::isDesktopLayoutDynamic() const
+    {
+    return Workspace::self()->isDesktopLayoutDynamic();
+    }
+
+int EffectsHandlerImpl::addDesktop( QPoint coords )
+    {
+    return Workspace::self()->addDesktop( coords );
+    }
+
+void EffectsHandlerImpl::deleteDesktop( int id )
+    {
+    Workspace::self()->deleteDesktop( id );
+    }
+
+QString EffectsHandlerImpl::desktopName( int desktop ) const
+    {
+    return Workspace::self()->desktopName( desktop );
+    }
+
+bool EffectsHandlerImpl::optionRollOverDesktops() const
+    {
+    return options->rollOverDesktops;
     }
 
 double EffectsHandlerImpl::animationTimeFactor() const
@@ -637,6 +705,11 @@ Window EffectsHandlerImpl::createInputWindow( Effect* e, int x, int y, int w, in
     XDefineCursor( display(), win, cursor.handle());
     XMapWindow( display(), win );
     input_windows.append( qMakePair( e, win ));
+
+    // Raise electric border windows above the input windows
+    // so they can still be triggered.
+    Workspace::self()->raiseElectricBorderWindows();
+
     return win;
     }
 
@@ -712,6 +785,9 @@ void EffectsHandlerImpl::checkInputWindowStacking()
     XRaiseWindow( display(), wins[ 0 ] );
     XRestackWindows( display(), wins, pos );
     delete[] wins;
+    // Raise electric border windows above the input windows
+    // so they can still be triggered. TODO: Do both at once.
+    Workspace::self()->raiseElectricBorderWindows();
     }
 
 QPoint EffectsHandlerImpl::cursorPos() const
@@ -1058,6 +1134,12 @@ double EffectWindowImpl::opacity() const
     return toplevel->opacity();
     }
 
+bool EffectWindowImpl::hasAlpha() const
+    {
+    return toplevel->hasAlpha();
+    }
+
+
 bool EffectWindowImpl::isDeleted() const
     {
     return (dynamic_cast<Deleted*>( toplevel ) != 0);
@@ -1307,6 +1389,22 @@ bool EffectWindowImpl::isManaged() const
     return dynamic_cast< const Client* >( toplevel ) != NULL;
     }
 
+bool EffectWindowImpl::acceptsFocus() const
+    {
+    const Client* client = dynamic_cast< const Client* >( toplevel );
+    if( !client )
+        return true; // We don't actually know...
+    return client->wantsInput();
+    }
+
+bool EffectWindowImpl::keepAbove() const
+    {
+    const Client* client = dynamic_cast< const Client* >( toplevel );
+    if( !client )
+        return true;
+    return client->keepAbove();
+    }
+
 bool EffectWindowImpl::isModal() const
     {
     if( Client* c = dynamic_cast< Client* >( toplevel ))
@@ -1337,50 +1435,6 @@ EffectWindowList EffectWindowImpl::mainWindows() const
     return EffectWindowList();
     }
 
-QList<QRect> EffectWindowImpl::shadowQuads( ShadowType type ) const
-    {
-    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
-        {
-        if( Client* c = dynamic_cast< Client* >( toplevel ))
-            return c->shadowQuads( type );
-        return QList<QRect>();
-        }
-    return toplevel->workspace()->decorationShadowQuads( type, toplevel->size() );
-    }
-
-double EffectWindowImpl::shadowOpacity( ShadowType type ) const
-    {
-    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
-        {
-        if( Client* c = dynamic_cast< Client* >( toplevel ))
-            return c->shadowOpacity( type );
-        return 1.0;
-        }
-    return toplevel->workspace()->decorationShadowOpacity( type );
-    }
-
-double EffectWindowImpl::shadowBrightness( ShadowType type ) const
-    {
-    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
-        {
-        if( Client* c = dynamic_cast< Client* >( toplevel ))
-            return c->shadowBrightness( type );
-        return 1.0;
-        }
-    return toplevel->workspace()->decorationShadowBrightness( type );
-    }
-
-double EffectWindowImpl::shadowSaturation( ShadowType type ) const
-    {
-    if( type == ShadowBorderedActive ||  type == ShadowBorderedInactive )
-        {
-        if( Client* c = dynamic_cast< Client* >( toplevel ))
-            return c->shadowSaturation( type );
-        return 1.0;
-        }
-    return toplevel->workspace()->decorationShadowSaturation( type );
-    }
-
 WindowQuadList EffectWindowImpl::buildQuads( bool force ) const
     {
     return sceneWindow()->buildQuads( force );
