@@ -59,12 +59,32 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual int currentDesktop() const;
         virtual int numberOfDesktops() const;
         virtual void setCurrentDesktop( int desktop );
+        virtual QSize desktopGridSize() const;
+        virtual int desktopGridWidth() const;
+        virtual int desktopGridHeight() const;
+        virtual int workspaceWidth() const;
+        virtual int workspaceHeight() const;
+        virtual int desktopAtCoords( QPoint coords ) const;
+        virtual QPoint desktopGridCoords( int id ) const;
+        virtual QPoint desktopCoords( int id ) const;
+        virtual int desktopAbove( int desktop = 0, bool wrap = true ) const;
+        virtual int desktopToRight( int desktop = 0, bool wrap = true ) const;
+        virtual int desktopBelow( int desktop = 0, bool wrap = true ) const;
+        virtual int desktopToLeft( int desktop = 0, bool wrap = true ) const;
+        virtual bool isDesktopLayoutDynamic() const;
+        virtual int addDesktop( QPoint coords );
+        virtual void deleteDesktop( int id );
         virtual QString desktopName( int desktop ) const;
+        virtual bool optionRollOverDesktops() const;
+
         virtual int displayWidth() const;
         virtual int displayHeight() const;
         virtual QPoint cursorPos() const;
         virtual bool grabKeyboard( Effect* effect );
         virtual void ungrabKeyboard();
+        virtual const void* getProxy( QString name );
+        virtual void startMousePolling();
+        virtual void stopMousePolling();
         virtual EffectWindow* findWindow( WId id ) const;
         virtual EffectWindowList stackingOrder() const;
         virtual void setElevatedWindow( EffectWindow* w, bool set );
@@ -95,12 +115,6 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual QRect clientArea( clientAreaOption, int screen, int desktop ) const;
         virtual QRect clientArea( clientAreaOption, const EffectWindow* c ) const;
         virtual QRect clientArea( clientAreaOption, const QPoint& p, int desktop ) const;
-        virtual void calcDesktopLayout(int* x, int* y, Qt::Orientation* orientation) const;
-        virtual bool optionRollOverDesktops() const;
-        virtual int desktopToLeft( int desktop, bool wrap ) const;
-        virtual int desktopToRight( int desktop, bool wrap ) const;
-        virtual int desktopUp( int desktop, bool wrap ) const;
-        virtual int desktopDown( int desktop, bool wrap ) const;
         virtual double animationTimeFactor() const;
         virtual WindowQuadType newWindowQuadType();
 
@@ -120,8 +134,6 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual void registerPropertyType( long atom, bool reg );
 
         virtual bool hasDecorationShadows() const;
-        virtual QList< QList<QImage> > shadowTextures();
-        virtual int shadowTextureList( ShadowType type ) const;
 
         // internal (used by kwin core or compositing code)
         void startPaint();
@@ -168,6 +180,7 @@ class EffectsHandlerImpl : public EffectsHandler
         QMultiMap< int, EffectPair > effect_order;
         QHash< long, int > registered_atoms;
         int next_window_quad_type;
+        int mouse_poll_ref_count;
 };
 
 class EffectWindowImpl : public EffectWindow
@@ -191,6 +204,7 @@ class EffectWindowImpl : public EffectWindow
         virtual int desktop() const; // prefer isOnXXX()
         virtual bool isMinimized() const;
         virtual double opacity() const;
+        virtual bool hasAlpha() const;
         virtual QString caption() const;
         virtual QPixmap icon() const;
         virtual QString windowClass() const;
@@ -233,16 +247,13 @@ class EffectWindowImpl : public EffectWindow
         virtual bool isComboBox() const;
         virtual bool isDNDIcon() const;
         virtual bool isManaged() const; // managed or override-redirect
+        virtual bool acceptsFocus() const;
+        virtual bool keepAbove() const;
 
         virtual bool isModal() const;
         virtual EffectWindow* findModal();
         virtual EffectWindowList mainWindows() const;
 
-        virtual QList<QRect> shadowQuads( ShadowType type ) const;
-        virtual double shadowOpacity( ShadowType type ) const;
-        virtual double shadowBrightness( ShadowType type ) const;
-        virtual double shadowSaturation( ShadowType type ) const;
-
         virtual WindowQuadList buildQuads( bool force = false ) const;
 
         const Toplevel* window() const;
