@@ -501,15 +501,18 @@ bool Workspace::workspaceEvent( XEvent * e )
                 {
 #ifdef HAVE_XRANDR
                 XRRUpdateConfiguration( e );
-#endif
                 if( compositing() )
                     {
                     // desktopResized() should take care of when the size or
                     // shape of the desktop has changed, but we also want to
                     // catch refresh rate changes
-                    finishCompositing();
-                    QTimer::singleShot( 0, this, SLOT( setupCompositing() ) );
+                    XRRScreenConfiguration *config = XRRGetScreenInfo( display(), rootWindow() );
+                    bool changed = ( xrrRefreshRate != XRRConfigCurrentRate( config ));
+                    XRRFreeScreenConfigInfo( config );
+                    if( changed )
+                        compositeResetTimer.start( 0 );
                     }
+#endif
                 }
             else if( e->type == Extensions::syncAlarmNotifyEvent() && Extensions::syncAvailable())
                 {
@@ -708,11 +711,11 @@ bool Client::windowEvent( XEvent* e )
             break;
         case ColormapChangeMask:
             if( e->xany.window == window())
-            {
-            cmap = e->xcolormap.colormap;
-            if ( isActive() )
-                workspace()->updateColormap();
-            }
+                {
+                cmap = e->xcolormap.colormap;
+                if ( isActive() )
+                    workspace()->updateColormap();
+                }
             break;
         default:
             if( e->xany.window == window())
@@ -744,7 +747,7 @@ bool Client::mapRequestEvent( XMapRequestEvent* e )
         {
         // Special support for the save-set feature, which is a bit broken.
         // If there's a window from one client embedded in another one,
-        // e.g. using XEMBED, and the embedder suddenly looses its X connection,
+        // e.g. using XEMBED, and the embedder suddenly loses its X connection,
         // save-set will reparent the embedded window to its closest ancestor
         // that will remains. Unfortunately, with reparenting window managers,
         // this is not the root window, but the frame (or in KWin's case,
@@ -1159,7 +1162,7 @@ bool Client::eventFilter( QObject* o, QEvent* e )
         // This will ensure that decoration->width() etc. and decoration->widget()->width() will be in sync.
         // These events only seem to be delayed events from initial resizing before show() was called
         // on the decoration widget.
-        if( ev->size() != size())
+        if( ev->size() != (size() + QSize( padding_left + padding_right, padding_top + padding_bottom ) ) )
             return true;
         // HACK: Avoid decoration redraw delays. On resize Qt sets WA_WStateConfigPending
         // which delays all painting until a matching ConfigureNotify event comes.
@@ -1227,7 +1230,7 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
             }
         else
             { // inactive inner window
-            if( !isActive() && w == wrapperId() && button < 4 )
+            if( !isActive() && w == wrapperId() && button < 6 )
                 {
                 was_action = true;
                 perform_handled = true;
@@ -1242,8 +1245,12 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
                     case Button3:
                         com = options->commandWindow3();
                         break;
+                    case Button4:
+                    case Button5:
+                        com = options->commandWindowWheel();
+                        break;
                     }
-                }
+             }
             // active inner window
             if( isActive() && w == wrapperId()
                 && options->clickRaise && button < 4 ) // exclude wheel
@@ -1300,7 +1307,7 @@ void Client::processDecorationButtonPress( int button, int /*state*/, int x, int
         {
         mode = mousePosition( QPoint( x, y ));
         buttonDown = true;
-        moveOffset = QPoint( x, y );
+        moveOffset = QPoint( x - padding_left, y - padding_top );
         invertedMoveOffset = rect().bottomRight() - moveOffset;
         unrestrictedMoveResize = false;
         startDelayedMoveResize();
