@@ -26,6 +26,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <ksharedconfig.h>
 #include <ktemporaryfile.h>
 
+#include "kwin_interface.h"
+
 #include "ui_main.h"
 #include "compositingprefs.h"
 #include "ktimerdialog.h"
@@ -65,22 +67,16 @@ class KWinCompositingConfig : public KCModule
         void loadGeneralTab();
         void loadEffectsTab();
         void loadAdvancedTab();
-        void loadElectricBorders();
         void saveGeneralTab();
         void saveEffectsTab();
         bool saveAdvancedTab();
-        void electricBorderSelectionChanged(int edge, int index);
 
         void configChanged(bool reinitCompositing);
         void initEffectSelector();
+        void setupCompositingState( bool active, bool enabled = true );
 
     private:
         bool effectEnabled( const QString& effect, const KConfigGroup& cfg ) const;
-        void setupElectricBorders();
-        void addItemToEdgesMonitor(const QString& item);
-        void changeElectricBorder( ElectricBorder border, int index );
-        ElectricBorder checkEffectHasElectricBorder( int index );
-        void saveElectricBorders();
 
         KSharedConfigPtr mKWinConfig;
         Ui::KWinCompositingConfig ui;
@@ -91,16 +87,7 @@ class KWinCompositingConfig : public KCModule
         KSharedConfigPtr mTmpConfig;
         bool m_showConfirmDialog;
 
-        enum ElectricBorderEffects
-            {
-            NoEffect,
-            PresentWindowsAll,
-            PresentWindowsCurrent,
-            DesktopGrid,
-            Cube,
-            Cylinder,
-            Sphere
-            };
+        OrgKdeKWinInterface* kwinInterface;
     };
 
 } // namespace
