@@ -82,7 +82,6 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 
 	QVBoxLayout* layout = new QVBoxLayout(this);
     layout->setMargin(0);
-    layout->setSpacing(KDialog::spacingHint());
 
 // Save this for later...
 //	cbUseMiniWindows = new QCheckBox( i18n( "Render mini &titlebars for all windows"), checkGroup );
@@ -95,8 +94,6 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 	QWidget *pluginPage = new QWidget( tabWidget );
 
 	QVBoxLayout* pluginLayout = new QVBoxLayout(pluginPage);
-	pluginLayout->setMargin(KDialog::marginHint());
-	pluginLayout->setSpacing(KDialog::spacingHint());
 
 	// decoration chooser
 	decorationList = new KComboBox( pluginPage );
@@ -106,10 +103,9 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 	pluginLayout->addWidget(decorationList);
 
 	pluginSettingsGrp = new QGroupBox( i18n("Decoration Options"), pluginPage );
-    QGridLayout *pluginSettingsLayout = new QGridLayout();
+    QVBoxLayout *pluginSettingsLayout = new QVBoxLayout();
 	pluginSettingsGrp->setFlat( true );
 	pluginSettingsLayout->setMargin( 0 );
-	pluginSettingsLayout->setSpacing( KDialog::spacingHint() );
 	pluginSettingsGrp->setLayout( pluginSettingsLayout );
 	pluginLayout->addWidget( pluginSettingsGrp );
 
@@ -123,19 +119,17 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 	lBorder->hide();
 	cBorder->hide();
 	QHBoxLayout *borderSizeLayout = new QHBoxLayout();
-    pluginSettingsGrp->layout()->addItem( borderSizeLayout );
+    pluginSettingsLayout->addLayout( borderSizeLayout );
 	borderSizeLayout->addWidget(lBorder);
 	borderSizeLayout->addWidget(cBorder);
 	borderSizeLayout->addStretch();
 
 	pluginConfigWidget = new KVBox(pluginSettingsGrp);
-	pluginSettingsGrp->layout()->addWidget( pluginConfigWidget );
+	pluginSettingsLayout->addWidget( pluginConfigWidget );
 
 	// Page 2 (Button Selector)
 	QWidget* buttonPage = new QWidget( tabWidget );
 	QVBoxLayout* buttonLayout = new QVBoxLayout(buttonPage);
-	buttonLayout->setMargin(KDialog::marginHint());
-	buttonLayout->setSpacing(KDialog::spacingHint());
 
 	cbShowToolTips = new QCheckBox(
 			i18n("&Show window button tooltips"), buttonPage );
@@ -163,7 +157,6 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 
 	// preview
 	QVBoxLayout* previewLayout = new QVBoxLayout();
-    previewLayout->setSpacing( KDialog::spacingHint() );
     layout->addLayout( previewLayout );
 	previewLayout->setMargin( KDialog::marginHint() );
 
@@ -217,10 +210,10 @@ KWinDecorationModule::~KWinDecorationModule()
 // And insert these into a DecorationInfo structure
 void KWinDecorationModule::findDecorations()
 {
-	QStringList dirList = KGlobal::dirs()->findDirs("data", "kwin");
+	const QStringList dirList = KGlobal::dirs()->findDirs("data", "kwin");
 	QStringList::ConstIterator it;
 
-	for (it = dirList.begin(); it != dirList.end(); ++it)
+	for (it = dirList.constBegin(); it != dirList.constEnd(); ++it)
 	{
 		QDir d(*it);
 		if (d.exists())
@@ -253,7 +246,7 @@ void KWinDecorationModule::createDecorationList()
 	// Sync with kwin hardcoded KDE2 style which has no desktop item
     QStringList decorationNames;
 //	decorationNames.append( i18n("KDE 2") );
-	for (it = decorations.begin(); it != decorations.end(); ++it)
+	for (it = decorations.constBegin(); it != decorations.constEnd(); ++it)
 	{
 		decorationNames.append((*it).name);
 	}
@@ -293,8 +286,8 @@ static const char* const border_names[ KDecorationDefines::BordersCount ] =
 int KWinDecorationModule::borderSizeToIndex( BorderSize size, QList< BorderSize > sizes )
 {
         int pos = 0;
-        for( QList< BorderSize >::ConstIterator it = sizes.begin();
-             it != sizes.end();
+        for( QList< BorderSize >::ConstIterator it = sizes.constBegin();
+             it != sizes.constEnd();
              ++it, ++pos )
             if( size <= *it )
                 break;
@@ -304,9 +297,9 @@ int KWinDecorationModule::borderSizeToIndex( BorderSize size, QList< BorderSize
 KDecorationDefines::BorderSize KWinDecorationModule::indexToBorderSize( int index,
     QList< BorderSize > sizes )
 {
-        QList< BorderSize >::ConstIterator it = sizes.begin();
+        QList< BorderSize >::ConstIterator it = sizes.constBegin();
         for(;
-             it != sizes.end();
+             it != sizes.constEnd();
              ++it, --index )
             if( index == 0 )
                 break;
@@ -555,12 +548,14 @@ void KWinDecorationModule::defaults()
 // Don't set default for now
 //	decorationList->setSelected(
 //		decorationList->findItem( i18n("KDE 2") ), true );  // KDE classic client
+	decorationList->setCurrentItem( i18n("Ozone"), true );  // KDE classic client
+	slotChangeDecoration(i18n("Ozone"));
 
 	buttonPositionWidget->setButtonsLeft(KDecorationOptions::defaultTitleButtonsLeft());
 	buttonPositionWidget->setButtonsRight(KDecorationOptions::defaultTitleButtonsRight());
 
-        border_size = BorderNormal;
-        checkSupportedBorderSizes();
+	border_size = BorderNormal;
+	checkSupportedBorderSizes();
 
 	// Set plugin defaults
 	emit pluginDefaults();
@@ -576,7 +571,7 @@ void KWinDecorationModule::checkSupportedBorderSizes()
 		cBorder->hide();
 	} else {
 		cBorder->clear();
-		for (QList<BorderSize>::const_iterator it = sizes.begin(); it != sizes.end(); ++it) {
+		for (QList<BorderSize>::const_iterator it = sizes.constBegin(); it != sizes.constEnd(); ++it) {
 			BorderSize size = *it;
 			cBorder->addItem(i18n(border_names[size]), borderSizeToIndex(size,sizes) );
 		}
