@@ -68,7 +68,7 @@ Atoms* atoms;
 
 int screen_number = -1;
 
-static bool initting = false;
+bool initting = false;
 
 /**
  * Whether to run Xlib in synchronous mode and print backtraces for X errors.
@@ -176,11 +176,13 @@ static int x11ErrorHandler( Display* d, XErrorEvent* e )
     if( ignore_badwindow && ( e->error_code == BadWindow || e->error_code == BadColor ))
         return 0;
 
+#ifndef NDEBUG
     //fprintf( stderr, "kwin: X Error (%s)\n", KXErrorHandler::errorMessage( *e, d ).data());
-    fprintf( stderr, "kwin: X Error (%s)\n", errorMessage( *e, d ).data() );
+    kWarning( 1212 ) << "kwin: X Error (" << errorMessage( *e, d ) << ")";
+#endif
 
     if( kwin_sync )
-    fprintf( stderr, "%s\n", kBacktrace().toLocal8Bit().data() );
+        fprintf( stderr, "%s\n", kBacktrace().toLocal8Bit().data() );
 
     if( initting )
         {
@@ -246,6 +248,7 @@ Application::Application()
         XSynchronize( display(), True );
         kDebug( 1212 ) << "Running KWin in sync mode";
         }
+    setQuitOnLastWindowClosed( false );
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
     KSharedConfig::Ptr config = KGlobal::config();
     if( !config->isImmutable() && args->isSet( "lock" ))
@@ -311,7 +314,7 @@ Application::Application()
 
     atoms = new Atoms;
 
-    initting = false; // TODO
+//    initting = false; // TODO
 
     // This tries to detect compositing options and can use GLX. GLX problems
     // (X errors) shouldn't cause kwin to abort, so this is out of the
@@ -379,7 +382,8 @@ void Application::crashHandler( int signal )
 
     fprintf( stderr, "Application::crashHandler() called with signal %d; recent crashes: %d\n", signal, crashes );
     char cmd[1024];
-    sprintf( cmd, "kwin --crashes %d &", crashes );
+    sprintf( cmd, "%s --crashes %d &",
+            QFile::encodeName(QCoreApplication::applicationFilePath()).constData(), crashes );
 
     sleep( 1 );
     system( cmd );
@@ -408,6 +412,10 @@ KDE_EXPORT int kdemain( int argc, char * argv[] )
             }
         }
 
+    // KWin only works properly with Qt's native X11 backend; override any compile-time
+    // or command line settings to raster or OpenGL.
+    QApplication::setGraphicsSystem("native");
+
     if( !restored )
         { // We only do the multihead fork if we are not restored by the session
           // manager, since the session manager will register multiple kwins,
