@@ -77,6 +77,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "deleted.h"
 #include "effects.h"
 
+#include <kephal/screens.h>
+
 namespace KWin
 {
 
@@ -87,8 +89,9 @@ namespace KWin
 Scene* scene;
 
 Scene::Scene( Workspace* ws )
-    : wspace( ws ),
-    has_waitSync( false )
+    : wspace( ws )
+    , has_waitSync( false )
+    , selfCheckDone( false )
     {
     }
     
@@ -116,7 +119,7 @@ void Scene::paintScreen( int* mask, QRegion* region )
         *mask &= ~PAINT_SCREEN_REGION;
         *region = infiniteRegion();
         }
-    else if(  *mask & PAINT_SCREEN_REGION )
+    else if( *mask & PAINT_SCREEN_REGION )
         { // make sure not to go outside visible screen
         *region &= QRegion( 0, 0, displayWidth(), displayHeight());
         }
@@ -198,6 +201,9 @@ void Scene::paintGenericScreen( int orig_mask, ScreenPaintData )
         if( !w->isPaintingEnabled())
             continue;
         phase2.append( Phase2Data( w, infiniteRegion(), data.clip, data.mask, data.quads ));
+        // transformations require window pixmap
+        w->suspendUnredirect( data.mask
+            & ( PAINT_WINDOW_TRANSLUCENT | PAINT_SCREEN_TRANSFORMED | PAINT_WINDOW_TRANSFORMED )); 
         }
 
     foreach( const Phase2Data &d, phase2 )
@@ -212,6 +218,7 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
     // TODO PAINT_WINDOW_* flags don't belong here, that's why it's in the assert,
     // perhaps the two enums should be separated
     assert(( orig_mask & ( PAINT_WINDOW_TRANSFORMED | PAINT_SCREEN_TRANSFORMED
+        | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS
         | PAINT_WINDOW_TRANSLUCENT | PAINT_WINDOW_OPAQUE )) == 0 );
     QHash< Window*, Phase2Data > phase2data;
     // Draw each opaque window top to bottom, subtracting the bounding rect of
@@ -225,7 +232,8 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
         data.mask = orig_mask | ( w->isOpaque() ? PAINT_WINDOW_OPAQUE : PAINT_WINDOW_TRANSLUCENT );
         w->resetPaintingEnabled();
         data.paint = region;
-        data.clip = w->isOpaque() ? w->shape().translated( w->x(), w->y()) : QRegion();
+        // Clip out the decoration for opaque windows; the decoration is drawn in the second pass
+        data.clip = w->isOpaque() ? w->clientShape().translated( w->x(), w->y() ) : QRegion();
         data.quads = w->buildQuads();
         // preparation step
         effects->prePaintWindow( effectWindow( w ), data, time_diff );
@@ -233,13 +241,20 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
         foreach( const WindowQuad &q, data.quads )
             if( q.isTransformed())
                 kFatal( 1212 ) << "Pre-paint calls are not allowed to transform quads!" ;
+        if( data.mask & PAINT_WINDOW_TRANSFORMED )
+            kFatal( 1212 ) << "PAINT_WINDOW_TRANSFORMED without PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS!";
 #endif
         if( !w->isPaintingEnabled())
+            {
+            w->suspendUnredirect( true );
             continue;
+            }
         if( data.paint != region ) // prepaint added area to draw
             painted_region |= data.paint; // make sure it makes it to the screen
         // Schedule the window for painting
         phase2data[w] = Phase2Data( w, data.paint, data.clip, data.mask, data.quads );
+        // no transformations, but translucency requires window pixmap
+        w->suspendUnredirect( data.mask & PAINT_WINDOW_TRANSLUCENT );
         }
     // Do the actual painting
     // First opaque windows, top to bottom
@@ -266,6 +281,9 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
             {
             // Paint the opaque window
             paintWindow( d.window, d.mask, d.region, d.quads );
+            // Clip out the client area, so we only draw the decoration in the next pass
+            phase2data[w].region = d.region - d.clip;
+            phase2data[w].mask |= PAINT_DECORATION_ONLY;
             }
         }
     // Fill any areas of the root window not covered by windows
@@ -278,8 +296,7 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
         if( !phase2data.contains( w ))
             continue;
         Phase2Data d = phase2data[w];
-        if( d.mask & PAINT_WINDOW_TRANSLUCENT )
-            paintWindow( d.window, d.mask, d.region, d.quads );
+        paintWindow( d.window, d.mask, d.region, d.quads );
         }
     }
 
@@ -307,6 +324,31 @@ void Scene::finalDrawWindow( EffectWindowImpl* w, int mask, QRegion region, Wind
     w->sceneWindow()->performPaint( mask, region, data );
     }
 
+QList< QPoint > Scene::selfCheckPoints() const
+    {
+    QList< QPoint > ret;
+    // Use Kephal directly, we're interested in "real" screens, not depending on our config.
+    // TODO: Does Kephal allow fake screens as well? We cannot use QDesktopWidget as it will cause a crash if
+    //       the number of screens is different to what Kephal returns.
+    for( int screen = 0;
+         screen < Kephal::ScreenUtils::numScreens();
+         ++screen )
+        { // test top-left and bottom-right of every screen
+        ret.append( Kephal::ScreenUtils::screenGeometry( screen ).topLeft());
+        ret.append( Kephal::ScreenUtils::screenGeometry( screen ).bottomRight() + QPoint( -3 + 1, -2 + 1 )
+            + QPoint( -1, 0 )); // intentionally moved one up, since the source windows will be one down
+        }
+    return ret;
+    }
+
+QRegion Scene::selfCheckRegion() const
+    {
+    QRegion reg;
+    foreach( const QPoint& p, selfCheckPoints())
+        reg |= QRect( p, QSize( selfCheckWidth(), selfCheckHeight()));
+    return reg;
+    }
+
 //****************************************
 // Scene::Window
 //****************************************
@@ -368,6 +410,16 @@ QRegion Scene::Window::shape() const
     return shape_region;
     }
 
+QRegion Scene::Window::clientShape() const
+    {
+    Client *c = dynamic_cast< Client* >( toplevel );
+    if ( c && c->isShade() )
+        return QRegion();
+
+    const QRegion r = shape() & QRect( toplevel->clientPos(), toplevel->clientSize() );
+    return r.isEmpty() ? QRegion() : r;
+    }
+
 bool Scene::Window::isVisible() const
     {
     if( dynamic_cast< Deleted* >( toplevel ) != NULL )
@@ -419,20 +471,23 @@ void Scene::Window::disablePainting( int reason )
     disable_painting |= reason;
     }
 
-WindowQuadList Scene::Window::buildQuads() const
+WindowQuadList Scene::Window::buildQuads( bool force ) const
     {
-    if( cached_quad_list != NULL )
+    if( cached_quad_list != NULL && !force )
         return *cached_quad_list;
     WindowQuadList ret;
     if( toplevel->clientPos() == QPoint( 0, 0 ) && toplevel->clientSize() == toplevel->size())
         ret = makeQuads( WindowQuadContents, shape()); // has no decoration
     else
         {
-        QRegion contents = shape() & QRect( toplevel->clientPos(), toplevel->clientSize());
-        QRegion decoration = shape() - contents;
+        Client *client = dynamic_cast<Client*>( toplevel );
+        QRegion contents = clientShape();
+        QRegion decoration = (client && Workspace::self()->decorationHasAlpha() ?
+                              QRegion(client->decorationRect()) : shape()) - contents;
         ret = makeQuads( WindowQuadContents, contents );
         ret += makeQuads( WindowQuadDecoration, decoration );
         }
+    effects->buildQuads( static_cast<Client*>( toplevel )->effectWindow(), ret );
     cached_quad_list = new WindowQuadList( ret );
     return ret;
     }
