@@ -84,6 +84,10 @@ Sources and other compositing managers:
 
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
 
+#include <X11/extensions/Xcomposite.h>
+
+#include <qpainter.h>
+
 namespace KWin
 {
 
@@ -159,15 +163,36 @@ SceneOpenGL::SceneOpenGL( Workspace* ws )
     // OpenGL scene setup
     glMatrixMode( GL_PROJECTION );
     glLoadIdentity();
+    float fovy = 60.0f;
+    float aspect = 1.0f;
+    float zNear = 0.1f;
+    float zFar = 100.0f;
+    float ymax = zNear * tan( fovy  * M_PI / 360.0f );
+    float ymin = -ymax;
+    float xmin =  ymin * aspect;
+    float xmax = ymax * aspect;
     // swap top and bottom to have OpenGL coordinate system match X system
-    glOrtho( 0, displayWidth(), displayHeight(), 0, 0, 65535 );
+    glFrustum( xmin, xmax, ymin, ymax, zNear, zFar );
     glMatrixMode( GL_MODELVIEW );
     glLoadIdentity();
+    float scaleFactor = 1.1 * tan( fovy * M_PI / 360.0f )/ymax;
+    glTranslatef( xmin*scaleFactor, ymax*scaleFactor, -1.1 );
+    glScalef( (xmax-xmin)*scaleFactor/displayWidth(), -(ymax-ymin)*scaleFactor/displayHeight(), 0.001 );
     if( checkGLError( "Init" ))
         {
         kError( 1212 ) << "OpenGL compositing setup failed";
         return; // error
         }
+    // Do self-check immediatelly during compositing setup only when it's not KWin startup
+    // at the same time (in other words, only when activating compositing using the kcm).
+    // Currently selfcheck causes bad flicker (due to X mapping the overlay window
+    // for too long?) which looks bad during KDE startup.
+    if( !initting )
+        {
+        if( !selfCheck())
+            return;
+        selfCheckDone = true;
+        }
     kDebug( 1212 ) << "DB:" << db << ", TFP:" << tfp_mode << ", SHM:" << shm_mode
         << ", Direct:" << bool( glXIsDirect( display(), ctxbuffer )) << endl;
     init_ok = true;
@@ -604,6 +629,105 @@ bool SceneOpenGL::initDrawableConfigs()
     return true;
     }
 
+// Test if compositing actually _really_ works, by creating a texture from a testing
+// window, drawing it on the screen, reading the contents back and comparing. This
+// should test whether compositing really works.
+// This function does the whole selfcheck, it can be done also in two parts
+// during actual drawing (to avoid flicker, see selfCheck() call from the ctor).
+bool SceneOpenGL::selfCheck()
+    {
+    QRegion reg = selfCheckRegion();
+    if( wspace->overlayWindow())
+        { // avoid covering the whole screen too soon
+        wspace->setOverlayShape( reg );
+        wspace->showOverlay();
+        }
+    selfCheckSetup();
+    flushBuffer( PAINT_SCREEN_REGION, reg );
+    bool ok = selfCheckFinish();
+    if( wspace->overlayWindow())
+        wspace->hideOverlay();
+    return ok;
+    }
+
+void SceneOpenGL::selfCheckSetup()
+    {
+    KXErrorHandler err;
+    QImage img( selfCheckWidth(), selfCheckHeight(), QImage::Format_RGB32 );
+    img.setPixel( 0, 0, QColor( Qt::red ).rgb());
+    img.setPixel( 1, 0, QColor( Qt::green ).rgb());
+    img.setPixel( 2, 0, QColor( Qt::blue ).rgb());
+    img.setPixel( 0, 1, QColor( Qt::white ).rgb());
+    img.setPixel( 1, 1, QColor( Qt::black ).rgb());
+    img.setPixel( 2, 1, QColor( Qt::white ).rgb());
+    QPixmap pix = QPixmap::fromImage( img );
+    foreach( const QPoint& p, selfCheckPoints())
+        {
+        XSetWindowAttributes wa;
+        wa.override_redirect = True;
+        ::Window window = XCreateWindow( display(), rootWindow(), 0, 0, selfCheckWidth(), selfCheckHeight(),
+            0, QX11Info::appDepth(), CopyFromParent, CopyFromParent, CWOverrideRedirect, &wa );
+        XSetWindowBackgroundPixmap( display(), window, pix.handle());
+        XClearWindow( display(), window );
+        XMapWindow( display(), window );
+        // move the window one down to where the result will be rendered too, just in case
+        // the render would fail completely and eventual check would try to read this window's contents
+        XMoveWindow( display(), window, p.x() + 1, p.y());
+        XCompositeRedirectWindow( display(), window, CompositeRedirectAutomatic );
+        Pixmap wpix = XCompositeNameWindowPixmap( display(), window );
+        glXWaitX();
+        Texture texture;
+        texture.load( wpix, QSize( selfCheckWidth(), selfCheckHeight()), QX11Info::appDepth());
+        texture.bind();
+        QRect rect( p.x(), p.y(), selfCheckWidth(), selfCheckHeight());
+        texture.render( infiniteRegion(), rect );
+        texture.unbind();
+        glXWaitGL();
+        XFreePixmap( display(), wpix );
+        XDestroyWindow( display(), window );
+        }
+    err.error( true ); // just sync and discard
+    }
+
+bool SceneOpenGL::selfCheckFinish()
+    {
+    glXWaitGL();
+    KXErrorHandler err;
+    bool ok = true;
+    foreach( const QPoint& p, selfCheckPoints())
+        {
+        QPixmap pix = QPixmap::grabWindow( rootWindow(), p.x(), p.y(), selfCheckWidth(), selfCheckHeight());
+        QImage img = pix.toImage();
+//        kDebug(1212) << "P:" << QColor( img.pixel( 0, 0 )).name();
+//        kDebug(1212) << "P:" << QColor( img.pixel( 1, 0 )).name();
+//        kDebug(1212) << "P:" << QColor( img.pixel( 2, 0 )).name();
+//        kDebug(1212) << "P:" << QColor( img.pixel( 0, 1 )).name();
+//        kDebug(1212) << "P:" << QColor( img.pixel( 1, 1 )).name();
+//        kDebug(1212) << "P:" << QColor( img.pixel( 2, 1 )).name();
+        if( img.pixel( 0, 0 ) != QColor( Qt::red ).rgb()
+            || img.pixel( 1, 0 ) != QColor( Qt::green ).rgb()
+            || img.pixel( 2, 0 ) != QColor( Qt::blue ).rgb()
+            || img.pixel( 0, 1 ) != QColor( Qt::white ).rgb()
+            || img.pixel( 1, 1 ) != QColor( Qt::black ).rgb()
+            || img.pixel( 2, 1 ) != QColor( Qt::white ).rgb())
+            {
+            kError( 1212 ) << "Compositing self-check failed, disabling compositing.";
+            ok = false;
+            break;
+            }
+        }
+    if( err.error( true ))
+        ok = false;
+    if( ok )
+        kDebug( 1212 ) << "Compositing self-check passed.";
+    if( !ok && options->disableCompositingChecks )
+        {
+        kWarning( 1212 ) << "Compositing checks disabled, proceeding regardless of self-check failure.";
+        return true;
+        }
+    return ok;
+    }
+
 // the entry function for painting
 void SceneOpenGL::paint( QRegion damage, ToplevelList toplevels )
     {
@@ -625,7 +749,20 @@ void SceneOpenGL::paint( QRegion damage, ToplevelList toplevels )
 #endif
     glPopMatrix();
     ungrabXServer(); // ungrab before flushBuffer(), it may wait for vsync
+    if( wspace->overlayWindow()) // show the window only after the first pass, since
+        wspace->showOverlay();   // that pass may take long
+    if( !selfCheckDone )
+        {
+        selfCheckSetup();
+        damage |= selfCheckRegion();
+        }
     flushBuffer( mask, damage );
+    if( !selfCheckDone )
+        {
+        if( !selfCheckFinish())
+            QTimer::singleShot( 0, Workspace::self(), SLOT( finishCompositing()));
+        selfCheckDone = true;
+        }
     // do cleanup
     stacking_order.clear();
     checkGLError( "PostPaint" );
@@ -647,8 +784,6 @@ void SceneOpenGL::waitSync()
 // actually paint to the screen (double-buffer swap or copy from pixmap buffer)
 void SceneOpenGL::flushBuffer( int mask, QRegion damage )
     {
-    if( wspace->overlayWindow()) // show the window only after the first pass, since
-        wspace->showOverlay();   // that pass may take long
     if( db )
         {
         if( mask & PAINT_SCREEN_REGION )
@@ -715,8 +850,30 @@ void SceneOpenGL::paintGenericScreen( int mask, ScreenPaintData data )
     if( mask & PAINT_SCREEN_TRANSFORMED )
         { // apply screen transformations
         glPushMatrix();
-        glTranslatef( data.xTranslate, data.yTranslate, 0 );
-        glScalef( data.xScale, data.yScale, 1 );
+        glTranslatef( data.xTranslate, data.yTranslate, data.zTranslate );
+        if( data.rotation )
+            {
+            // translate to rotation point, rotate, translate back
+            glTranslatef( data.rotation->xRotationPoint, data.rotation->yRotationPoint, data.rotation->zRotationPoint );
+            float xAxis = 0.0;
+            float yAxis = 0.0;
+            float zAxis = 0.0;
+            switch( data.rotation->axis )
+                {
+                case RotationData::XAxis:
+                    xAxis = 1.0;
+                    break;
+                case RotationData::YAxis:
+                    yAxis = 1.0;
+                    break;
+                case RotationData::ZAxis:
+                    zAxis = 1.0;
+                    break;
+                }
+            glRotatef( data.rotation->angle, xAxis, yAxis, zAxis );
+            glTranslatef( -data.rotation->xRotationPoint, -data.rotation->yRotationPoint, -data.rotation->zRotationPoint );
+            }
+        glScalef( data.xScale, data.yScale, data.zScale );
         }
     Scene::paintGenericScreen( mask, data );
     if( mask & PAINT_SCREEN_TRANSFORMED )
@@ -847,7 +1004,7 @@ void SceneOpenGL::Texture::release()
         {
         if( !options->glStrictBinding )
             glXReleaseTexImageEXT( display(), bound_glxpixmap, GLX_FRONT_LEFT_EXT );
-        glXDestroyGLXPixmap( display(), bound_glxpixmap );
+        glXDestroyPixmap( display(), bound_glxpixmap );
         bound_glxpixmap = None;
         }
     }
@@ -882,7 +1039,7 @@ void SceneOpenGL::Texture::findTarget()
             mScale.setHeight( 1.0f );
           break;
         default:
-            assert( false );
+            abort();
         }
     }
 
@@ -1133,6 +1290,10 @@ void SceneOpenGL::Texture::unbind()
 SceneOpenGL::Window::Window( Toplevel* c )
     : Scene::Window( c )
     , texture()
+    , topTexture()
+    , leftTexture()
+    , rightTexture()
+    , bottomTexture()
     {
     }
 
@@ -1157,7 +1318,7 @@ bool SceneOpenGL::Window::bindTexture()
     bool success = texture.load( pix, toplevel->size(), toplevel->depth(),
         toplevel->damage());
     if( success )
-        toplevel->resetDamage( toplevel->rect());
+        toplevel->resetDamage( QRect( toplevel->clientPos(), toplevel->clientSize() ) );
     else
         kDebug( 1212 ) << "Failed to bind window";
     return success;
@@ -1166,6 +1327,10 @@ bool SceneOpenGL::Window::bindTexture()
 void SceneOpenGL::Window::discardTexture()
     {
     texture.discard();
+    topTexture.discard();
+    leftTexture.discard();
+    rightTexture.discard();
+    bottomTexture.discard();
     }
 
 // This call is used in SceneOpenGL::windowGeometryShapeChanged(),
@@ -1192,23 +1357,17 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
     // check if there is something to paint (e.g. don't paint if the window
     // is only opaque and only PAINT_WINDOW_TRANSLUCENT is requested)
     bool opaque = isOpaque() && data.opacity == 1.0;
-    if( mask & ( PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSLUCENT ))
-        {}
-    else if( mask & PAINT_WINDOW_OPAQUE )
-        {
-        if( !opaque )
-            return;
-        }
-    else if( mask & PAINT_WINDOW_TRANSLUCENT )
-        {
-        if( opaque )
-            return;
-        }
+    /* HACK: It seems this causes painting glitches, disable temporarily
+    if(( mask & PAINT_WINDOW_OPAQUE ) ^ ( mask & PAINT_WINDOW_TRANSLUCENT ))
+        { // We are only painting either opaque OR translucent windows, not both
+        if( mask & PAINT_WINDOW_OPAQUE && !opaque )
+            return; // Only painting opaque and window is translucent
+        if( mask & PAINT_WINDOW_TRANSLUCENT && opaque )
+            return; // Only painting translucent and window is opaque
+        }*/
     // paint only requested areas
     if( region != infiniteRegion()) // avoid integer overflow
         region.translate( -x(), -y());
-    if(( mask & ( PAINT_SCREEN_TRANSFORMED | PAINT_WINDOW_TRANSFORMED )) == 0 )
-        region &= shape();
     if( region.isEmpty())
         return;
     if( !bindTexture())
@@ -1244,42 +1403,213 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
     // do required transformations
     int x = toplevel->x();
     int y = toplevel->y();
+    double z = 0.0;
     if( mask & PAINT_WINDOW_TRANSFORMED )
         {
         x += data.xTranslate;
         y += data.yTranslate;
+        z += data.zTranslate;
+        }
+    glTranslatef( x, y, z );
+    if(( mask & PAINT_WINDOW_TRANSFORMED ) && ( data.xScale != 1 || data.yScale != 1 || data.zScale != 1 ))
+        glScalef( data.xScale, data.yScale, data.zScale );
+    if(( mask & PAINT_WINDOW_TRANSFORMED ) && data.rotation )
+        {
+        glTranslatef( data.rotation->xRotationPoint, data.rotation->yRotationPoint, data.rotation->zRotationPoint );
+        float xAxis = 0.0;
+        float yAxis = 0.0;
+        float zAxis = 0.0;
+        switch( data.rotation->axis )
+            {
+            case RotationData::XAxis:
+                xAxis = 1.0;
+                break;
+            case RotationData::YAxis:
+                yAxis = 1.0;
+                break;
+            case RotationData::ZAxis:
+                zAxis = 1.0;
+                break;
+            }
+        glRotatef( data.rotation->angle, xAxis, yAxis, zAxis );
+        glTranslatef( -data.rotation->xRotationPoint, -data.rotation->yRotationPoint, -data.rotation->zRotationPoint );
         }
-    glTranslatef( x, y, 0 );
-    if(( mask & PAINT_WINDOW_TRANSFORMED ) && ( data.xScale != 1 || data.yScale != 1 ))
-        glScalef( data.xScale, data.yScale, 1 );
     region.translate( toplevel->x(), toplevel->y() );  // Back to screen coords
 
     texture.bind();
     texture.enableUnnormalizedTexCoords();
 
-    WindowQuadList decoration = data.quads.select( WindowQuadDecoration );    
-    if( data.contents_opacity != data.decoration_opacity && !decoration.isEmpty())
+    WindowQuadList decoration = data.quads.select( WindowQuadDecoration );
+    // paint the content
+    if ( !(mask & PAINT_DECORATION_ONLY) )
         {
-        prepareStates( data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
+        prepareStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
         renderQuads( mask, region, data.quads.select( WindowQuadContents ));
-        restoreStates( data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
-        prepareStates( data.opacity * data.decoration_opacity, data.brightness, data.saturation, data.shader );
-        renderQuads( mask, region, decoration );
-        restoreStates( data.opacity * data.decoration_opacity, data.brightness, data.saturation, data.shader );
-        }
-    else
-        {
-        prepareStates( data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
-        renderQuads( mask, region, data.quads.select( WindowQuadContents ));
-        renderQuads( mask, region, data.quads.select( WindowQuadDecoration ));
-        restoreStates( data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
+        restoreStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
         }
 
     texture.disableUnnormalizedTexCoords();
     texture.unbind();
+
+    // decorations
+    if( Client *client = dynamic_cast<Client*>(toplevel) )
+        {
+        if (!client->noBorder())
+            {
+            bool updateDeco = client->decorationPixmapRequiresRepaint();
+            client->ensureDecorationPixmapsPainted();
+
+            const QPixmap *left   = client->leftDecoPixmap();
+            const QPixmap *top    = client->topDecoPixmap();
+            const QPixmap *right  = client->rightDecoPixmap();
+            const QPixmap *bottom = client->bottomDecoPixmap();
+
+            WindowQuadList topList, leftList, rightList, bottomList;
+            QRect topRect, leftRect, rightRect, bottomRect;
+            client->layoutDecorationRects(leftRect, topRect, rightRect, bottomRect, Client::WindowRelative);
+
+            foreach( WindowQuad quad, decoration )
+                {
+                if( topRect.contains( QPoint( quad.originalLeft(), quad.originalTop() ) ) )
+                    {
+                    topList.append( quad );
+                    continue;
+                    }
+                if( bottomRect.contains( QPoint( quad.originalLeft(), quad.originalTop() ) ) )
+                    {
+                    bottomList.append( quad );
+                    continue;
+                    }
+                if( leftRect.contains( QPoint( quad.originalLeft(), quad.originalTop() ) ) )
+                    {
+                    leftList.append( quad );
+                    continue;
+                    }
+                if( rightRect.contains( QPoint( quad.originalLeft(), quad.originalTop() ) ) )
+                    {
+                    rightList.append( quad );
+                    continue;
+                    }
+                }
+            paintDecoration( top, DecorationTop, region, topRect, data, topList, updateDeco );
+            paintDecoration( left, DecorationLeft, region, leftRect, data, leftList, updateDeco );
+            paintDecoration( right, DecorationRight, region, rightRect, data, rightList, updateDeco );
+            paintDecoration( bottom, DecorationBottom, region, bottomRect, data, bottomList, updateDeco );
+            }
+        }
+
     glPopMatrix();
     }
 
+void SceneOpenGL::Window::paintDecoration( const QPixmap* decoration, TextureType decorationType, const QRegion& region, const QRect& rect, const WindowPaintData& data, const WindowQuadList& quads, bool updateDeco )
+    {
+    if( quads.isEmpty())
+        return;
+    SceneOpenGL::Texture* decorationTexture;
+    switch( decorationType )
+        {
+        case DecorationTop:
+            decorationTexture = &topTexture;
+            break;
+        case DecorationLeft:
+            decorationTexture = &leftTexture;
+            break;
+        case DecorationRight:
+            decorationTexture = &rightTexture;
+            break;
+        case DecorationBottom:
+            decorationTexture = &bottomTexture;
+            break;
+        default:
+            return;
+        }
+    if( decorationTexture->texture() != None && !updateDeco )
+        {
+        // texture doesn't need updating, just bind it
+        glBindTexture( decorationTexture->target(), decorationTexture->texture());
+        }
+    else
+        {
+        bool success = decorationTexture->load( decoration->handle(), decoration->size(), decoration->depth() );
+        if( !success )
+            {
+            kDebug( 1212 ) << "Failed to bind decoartion";
+            return;
+            }
+        }
+    if( filter == ImageFilterGood )
+        {
+        // avoid unneeded mipmap generation by only using trilinear
+        // filtering when it actually makes a difference, that is with
+        // minification or changed vertices
+        if( options->smoothScale == 2
+            && ( data.quads.smoothNeeded() || data.xScale < 1 || data.yScale < 1 ))
+            {
+            decorationTexture->setFilter( GL_LINEAR_MIPMAP_LINEAR );
+            }
+        else
+            decorationTexture->setFilter( GL_LINEAR );
+        }
+    else
+        decorationTexture->setFilter( GL_NEAREST );
+    decorationTexture->setWrapMode( GL_CLAMP_TO_EDGE );
+    decorationTexture->bind();
+    decorationTexture->enableUnnormalizedTexCoords();
+
+    prepareStates( decorationType, data.opacity * data.decoration_opacity, data.brightness, data.saturation, data.shader );
+    float* vertices;
+    float* texcoords;
+    makeDecorationArrays( &vertices, &texcoords, quads, rect );
+    if( data.shader )
+        {
+        int texw = decoration->width();
+        int texh = decoration->height();
+        if( !GLTexture::NPOTTextureSupported() )
+            {
+            kWarning( 1212 ) << "NPOT textures not supported, wasting some memory" ;
+            texw = nearestPowerOfTwo(texw);
+            texh = nearestPowerOfTwo(texh);
+            }
+        data.shader->setUniform("textureWidth", (float)texw);
+        data.shader->setUniform("textureHeight", (float)texh);
+        }
+    renderGLGeometry( region, quads.count() * 4,
+            vertices, texcoords, NULL, 2, 0 );
+    delete[] vertices;
+    delete[] texcoords;
+    restoreStates( decorationType, data.opacity * data.decoration_opacity, data.brightness, data.saturation, data.shader );
+    decorationTexture->disableUnnormalizedTexCoords();
+    decorationTexture->unbind();
+    }
+
+void SceneOpenGL::Window::makeDecorationArrays( float** vertices, float** texcoords, const WindowQuadList& quads, const QRect& rect ) const
+    {
+    *vertices = new float[ quads.count() * 4 * 2 ];
+    *texcoords = new float[ quads.count() * 4 * 2 ];
+    float* vpos = *vertices;
+    float* tpos = *texcoords;
+    foreach( WindowQuad quad, quads )
+        {
+        *vpos++ = quad[ 0 ].x();
+        *vpos++ = quad[ 0 ].y();
+        *vpos++ = quad[ 1 ].x();
+        *vpos++ = quad[ 1 ].y();
+        *vpos++ = quad[ 2 ].x();
+        *vpos++ = quad[ 2 ].y();
+        *vpos++ = quad[ 3 ].x();
+        *vpos++ = quad[ 3 ].y();
+
+        *tpos++ = quad.originalLeft()-rect.x();
+        *tpos++ = quad.originalTop()-rect.y();
+        *tpos++ = quad.originalRight()-rect.x();
+        *tpos++ = quad.originalTop()-rect.y();
+        *tpos++ = quad.originalRight()-rect.x();
+        *tpos++ = quad.originalBottom()-rect.y();
+        *tpos++ = quad.originalLeft()-rect.x();
+        *tpos++ = quad.originalBottom()-rect.y();
+        }
+    }
+
 void SceneOpenGL::Window::renderQuads( int, const QRegion& region, const WindowQuadList& quads )
     {
     if( quads.isEmpty())
@@ -1294,40 +1624,83 @@ void SceneOpenGL::Window::renderQuads( int, const QRegion& region, const WindowQ
     delete[] texcoords;
     }
 
-void SceneOpenGL::Window::prepareStates( double opacity, double brightness, double saturation, GLShader* shader )
+void SceneOpenGL::Window::prepareStates( TextureType type, double opacity, double brightness, double saturation, GLShader* shader )
     {
     if(shader)
-        prepareShaderRenderStates( opacity, brightness, saturation, shader );
+        prepareShaderRenderStates( type, opacity, brightness, saturation, shader );
     else
-        prepareRenderStates( opacity, brightness, saturation );
+        prepareRenderStates( type, opacity, brightness, saturation );
     }
 
-void SceneOpenGL::Window::prepareShaderRenderStates( double opacity, double brightness, double saturation, GLShader* shader )
+void SceneOpenGL::Window::prepareShaderRenderStates( TextureType type, double opacity, double brightness, double saturation, GLShader* shader )
     {
     // setup blending of transparent windows
     glPushAttrib( GL_ENABLE_BIT );
     bool opaque = isOpaque() && opacity == 1.0;
+    if( type != Content )
+        opaque = false;
     if( !opaque )
         {
         glEnable( GL_BLEND );
-        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+        glBlendFunc( GL_ONE, GL_ONE_MINUS_SRC_ALPHA );
         }
     shader->setUniform("opacity", (float)opacity);
     shader->setUniform("saturation", (float)saturation);
     shader->setUniform("brightness", (float)brightness);
+
+    // setting texture width and heiht stored in shader
+    // only set if it is set by an effect that is not negative
+    float texw = shader->textureWidth();
+    if( texw >= 0.0f )
+        shader->setUniform("textureWidth", texw);
+    float texh = shader->textureHeight();
+    if( texh >= 0.0f )
+        shader->setUniform("textureHeight", texh);
     }
 
-void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness, double saturation )
+void SceneOpenGL::Window::prepareRenderStates( TextureType type, double opacity, double brightness, double saturation )
     {
+    Texture* tex;
+    bool alpha = false;
+    bool opaque = true;
+    switch( type )
+        {
+        case Content:
+            tex = &texture;
+            alpha = toplevel->hasAlpha();
+            opaque = isOpaque() && opacity == 1.0;
+            break;
+        case DecorationTop:
+            tex = &topTexture;
+            alpha = true;
+            opaque = false;
+            break;
+        case DecorationLeft:
+            tex = &leftTexture;
+            alpha = true;
+            opaque = false;
+            break;
+        case DecorationRight:
+            tex = &rightTexture;
+            alpha = true;
+            opaque = false;
+            break;
+        case DecorationBottom:
+            tex = &bottomTexture;
+            alpha = true;
+            opaque = false;
+            break;
+        default:
+            return;
+        }
     // setup blending of transparent windows
     glPushAttrib( GL_ENABLE_BIT );
-    bool opaque = isOpaque() && opacity == 1.0;
     if( !opaque )
         {
         glEnable( GL_BLEND );
-        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+        glBlendFunc( GL_ONE, GL_ONE_MINUS_SRC_ALPHA );
         }
-    if( saturation != 1.0 && texture.saturationSupported())
+    if( saturation != 1.0 && tex->saturationSupported())
         {
         // First we need to get the color from [0; 1] range to [0.5; 1] range
         glActiveTexture( GL_TEXTURE0 );
@@ -1341,7 +1714,7 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
         glTexEnvi( GL_TEXTURE_ENV, GL_OPERAND2_RGB, GL_SRC_ALPHA );
         const float scale_constant[] = { 1.0, 1.0, 1.0, 0.5};
         glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, scale_constant );
-        texture.bind();
+        tex->bind();
 
         // Then we take dot product of the result of previous pass and
         //  saturation_constant. This gives us completely unsaturated
@@ -1357,7 +1730,7 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
         glTexEnvi( GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_CONSTANT );
         glTexEnvi( GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR );
         glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, saturation_constant );
-        texture.bind();
+        tex->bind();
 
         // Finally we need to interpolate between the original image and the
         //  greyscale image to get wanted level of saturation
@@ -1377,9 +1750,9 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
         glTexEnvi( GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA );
         // And make primary color contain the wanted opacity
         glColor4f( opacity, opacity, opacity, opacity );
-        texture.bind();
+        tex->bind();
 
-        if( toplevel->hasAlpha() || brightness != 1.0f )
+        if( alpha || brightness != 1.0f )
             {
             glActiveTexture( GL_TEXTURE3 );
             glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE );
@@ -1388,12 +1761,12 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
             glTexEnvi( GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR );
             glTexEnvi( GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PRIMARY_COLOR );
             glTexEnvi( GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR );
-            if( toplevel->hasAlpha() )
+            // The color has to be multiplied by both opacity and brightness
+            float opacityByBrightness = opacity * brightness;
+            glColor4f( opacityByBrightness, opacityByBrightness, opacityByBrightness, opacity );
+            if( alpha )
                 {
-                // The color has to be multiplied by both opacity and brightness
-                float opacityByBrightness = opacity * brightness;
-                glColor4f( opacityByBrightness, opacityByBrightness, opacityByBrightness, opacity );
-                // Also multiply original texture's alpha by our opacity
+                // Multiply original texture's alpha by our opacity
                 glTexEnvi( GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE );
                 glTexEnvi( GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_TEXTURE0 );
                 glTexEnvi( GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA );
@@ -1402,14 +1775,12 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
                 }
             else
                 {
-                // Color has to be multiplied only by brightness
-                glColor4f( brightness, brightness, brightness, opacity );
                 // Alpha will be taken from previous stage
                 glTexEnvi( GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE );
                 glTexEnvi( GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_PREVIOUS );
                 glTexEnvi( GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA );
                 }
-            texture.bind();
+            tex->bind();
             }
 
         glActiveTexture(GL_TEXTURE0 );
@@ -1418,9 +1789,9 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
         {
         // the window is additionally configured to have its opacity adjusted,
         // do it
-        if( toplevel->hasAlpha())
+        float opacityByBrightness = opacity * brightness;
+        if( alpha)
             {
-            float opacityByBrightness = opacity * brightness;
             glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
             glColor4f( opacityByBrightness, opacityByBrightness, opacityByBrightness,
                 opacity);
@@ -1428,7 +1799,7 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
         else
             {
             // Multiply color by brightness and replace alpha by opacity
-            float constant[] = { brightness, brightness, brightness, opacity };
+            float constant[] = { opacityByBrightness, opacityByBrightness, opacityByBrightness, opacity };
             glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE );
             glTexEnvi( GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE );
             glTexEnvi( GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE );
@@ -1442,16 +1813,17 @@ void SceneOpenGL::Window::prepareRenderStates( double opacity, double brightness
         }
     }
 
-void SceneOpenGL::Window::restoreStates( double opacity, double brightness, double saturation, GLShader* shader )
+void SceneOpenGL::Window::restoreStates( TextureType type, double opacity, double brightness, double saturation, GLShader* shader )
     {
     if(shader)
-        restoreShaderRenderStates( opacity, brightness, saturation, shader );
+        restoreShaderRenderStates( type, opacity, brightness, saturation, shader );
     else
-        restoreRenderStates( opacity, brightness, saturation );
+        restoreRenderStates( type, opacity, brightness, saturation );
     }
 
-void SceneOpenGL::Window::restoreShaderRenderStates( double opacity, double brightness, double saturation, GLShader* shader )
+void SceneOpenGL::Window::restoreShaderRenderStates( TextureType type, double opacity, double brightness, double saturation, GLShader* shader )
     {
+    Q_UNUSED( type );
     Q_UNUSED( opacity );
     Q_UNUSED( brightness );
     Q_UNUSED( saturation );
@@ -1459,18 +1831,39 @@ void SceneOpenGL::Window::restoreShaderRenderStates( double opacity, double brig
     glPopAttrib();  // ENABLE_BIT
     }
 
-void SceneOpenGL::Window::restoreRenderStates( double opacity, double brightness, double saturation )
+void SceneOpenGL::Window::restoreRenderStates( TextureType type, double opacity, double brightness, double saturation )
     {
+    Texture* tex;
+    switch( type )
+        {
+        case Content:
+            tex = &texture;
+            break;
+        case DecorationTop:
+            tex = &topTexture;
+            break;
+        case DecorationLeft:
+            tex = &leftTexture;
+            break;
+        case DecorationRight:
+            tex = &rightTexture;
+            break;
+        case DecorationBottom:
+            tex = &bottomTexture;
+            break;
+        default:
+            return;
+        }
     if( opacity != 1.0 || saturation != 1.0 || brightness != 1.0f )
         {
-        if( saturation != 1.0 && texture.saturationSupported())
+        if( saturation != 1.0 && tex->saturationSupported())
             {
             glActiveTexture(GL_TEXTURE3);
-            glDisable( texture.target());
+            glDisable( tex->target());
             glActiveTexture(GL_TEXTURE2);
-            glDisable( texture.target());
+            glDisable( tex->target());
             glActiveTexture(GL_TEXTURE1);
-            glDisable( texture.target());
+            glDisable( tex->target());
             glActiveTexture(GL_TEXTURE0);
             }
         glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
