@@ -116,9 +116,6 @@ QMenu* Workspace::clientPopup()
         action->setIcon( KIcon( "wizard" ) );
         action->setData( Options::ApplicationRulesOp );
 
-        action = popup->addMenu( advanced_popup );
-        action->setText( i18n("Ad&vanced") );
-
         trans_popup = 0;
         if (compositing()){
             trans_popup = new QMenu( popup );
@@ -170,6 +167,9 @@ QMenu* Workspace::clientPopup()
         mShadeOpAction->setCheckable( true );
         mShadeOpAction->setData( Options::ShadeOp );
 
+        action = popup->addMenu( advanced_popup );
+        action->setText( i18n("Ad&vanced") );
+
         popup->addSeparator();
 
         if (!KGlobal::config()->isImmutable() &&
@@ -266,7 +266,8 @@ void Workspace::initDesktopPopup()
              this, SLOT( desktopPopupAboutToShow() ) );
 
     QAction *action = desk_popup->menuAction();
-    popup->insertAction(advanced_popup->menuAction(), action);
+    // set it as the first item
+    popup->insertAction( trans_popup ? trans_popup->menuAction() : mMoveOpAction, action);
     action->setText( i18n("To &Desktop") );
     }
 
@@ -345,27 +346,45 @@ void Workspace::readShortcuts()
 
     KAction *kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktops") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktops = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkThroughDesktopsKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktops (Reverse)") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktopsReverse = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughDesktopsKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktop List") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktopList = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkThroughDesktopListKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktop List (Reverse)") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktopListReverse = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughDesktopListKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Windows") );
     if ( kaction!=0 )
+        {
         cutWalkThroughWindows = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkThroughWindowsKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Windows (Reverse)") );
     if ( kaction!=0 )
+        {
         cutWalkThroughWindowsReverse = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughWindowsKeyChanged(QKeySequence)));
+        }
     discardPopup(); // so that it's recreated next time
     }
 
@@ -713,7 +732,7 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             break;
         case Options::MouseOpacityLess:
             if( !isDesktop() ) // No point in changing the opacity of the desktop
-                setOpacity( qMax( opacity() - 0.1, 0.0 ));
+                setOpacity( qMax( opacity() - 0.1, 0.1 ));
             break;
         case Options::MouseNothing:
             replay = true;
@@ -822,7 +841,7 @@ void Workspace::slotSwitchDesktopLeft()
 
 void Workspace::slotSwitchDesktopUp()
     {
-    int desktop = desktopUp( currentDesktop(), options->rollOverDesktops);
+    int desktop = desktopAbove( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -830,7 +849,7 @@ void Workspace::slotSwitchDesktopUp()
 
 void Workspace::slotSwitchDesktopDown()
     {
-    int desktop = desktopDown( currentDesktop(), options->rollOverDesktops);
+    int desktop = desktopBelow( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -1088,7 +1107,7 @@ void Workspace::slotWindowToDesktopLeft()
 
 void Workspace::slotWindowToDesktopUp()
     {
-    int d = desktopUp( currentDesktop(), options->rollOverDesktops);
+    int d = desktopAbove( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
@@ -1103,7 +1122,7 @@ void Workspace::slotWindowToDesktopUp()
 
 void Workspace::slotWindowToDesktopDown()
     {
-    int d = desktopDown( currentDesktop(), options->rollOverDesktops);
+    int d = desktopBelow( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
