@@ -38,6 +38,7 @@
 #include <Solid/OpticalDrive>
 #include <Solid/OpticalDisc>
 #include <KUrl>
+#include <KStandardDirs>
 
 // Plasma
 #include <Plasma/Containment>
@@ -47,6 +48,8 @@
 #include "core/favoritesmodel.h"
 #include "core/models.h"
 
+Q_DECLARE_METATYPE(QPersistentModelIndex)
+
 using namespace Kickoff;
 
 class ContextMenuFactory::Private
@@ -60,7 +63,7 @@ public:
         KUrl kUrl(url);
         KActionCollection actionCollection((QObject*)0);
         KFileItemList items;
-        QString mimeType = KMimeType::findByUrl(kUrl, 0, false, true)->name();
+        const QString mimeType = KMimeType::findByUrl(kUrl, 0, false, true)->name();
         items << KFileItem(url, mimeType, KFileItem::Unknown);
         KParts::BrowserExtension::PopupFlags browserFlags = KParts::BrowserExtension::DefaultPopupItems;
         if (items.first().isLocalFile()) {
@@ -101,18 +104,22 @@ ContextMenuFactory::~ContextMenuFactory()
     delete d;
 }
 
-void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &pos)
+void ContextMenuFactory::showContextMenu(QAbstractItemView *view,
+                                         const QPersistentModelIndex& index, const QPoint& pos)
 {
-    Q_ASSERT(view);
+    Q_UNUSED(pos);
+
+    if (!index.isValid()) {
+        return;
+    }
 
-    const QModelIndex index = view->indexAt(pos);
     const QString url = index.data(UrlRole).value<QString>();
 
     if (url.isEmpty()) {
         return;
     }
 
-    bool isFavorite = FavoritesModel::isFavorite(url);
+    const bool isFavorite = FavoritesModel::isFavorite(url);
 
     QList<QAction*> actions;
 
@@ -129,7 +136,6 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
         actions << favoriteAction;
     }
 
-
     // add to desktop
     QAction *addToDesktopAction = new QAction(this);
 
@@ -144,7 +150,15 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
     if ((d->applet) && (kurl.scheme() != "leave")) {
         Plasma::Containment *containment = d->applet->containment();
 
-        if (containment && containment->corona()) {
+        // There might be relative paths for .desktop installed in
+        // /usr/shar/applnk, we need to locate them
+        bool urlFound = true;
+        if (kurl.isRelative() && kurl.url().endsWith(".desktop")) {
+            kurl = KStandardDirs::locate("apps", url);
+            urlFound = !kurl.isEmpty();
+        }
+
+        if (urlFound && containment && containment->corona()) {
             Plasma::Containment *desktop = containment->corona()->containmentForScreen(containment->screen());
 
             if (desktop && desktop->immutability() == Plasma::Mutable) {
@@ -153,7 +167,7 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
             }
         }
 
-        if (containment && containment->immutability() == Plasma::Mutable &&
+        if (urlFound && containment && containment->immutability() == Plasma::Mutable &&
                 containment->containmentType() == Plasma::Containment::PanelContainment) {
             addToPanelAction->setText(i18n("Add to Panel"));
             actions << addToPanelAction;
@@ -171,7 +185,7 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
     }
 
     // device actions
-    QString udi = index.data(DeviceUdiRole).toString();
+    const QString udi = index.data(DeviceUdiRole).toString();
     Solid::Device device(udi);
     Solid::StorageAccess *access = device.as<Solid::StorageAccess>();
     QAction *ejectAction = 0;
@@ -192,9 +206,11 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
 
     // add view specific actions
     QAction *viewSeparator = new QAction(this);
-    viewSeparator->setSeparator(true);
-    actions << viewSeparator;
-    actions << viewActions(view);
+    if (view) {
+        viewSeparator->setSeparator(true);
+        actions << viewSeparator;
+        actions << viewActions(view);
+    }
 
     // display menu
     KMenu menu;
@@ -227,7 +243,7 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
                     Plasma::Containment *desktop = corona->containmentForScreen(containment->screen());
                     if (desktop) {
                         QVariantList args;
-                        args << url;
+                        args << kurl.url();
                         desktop->addApplet("icon", args);
                     }
                 }
@@ -239,7 +255,7 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
             Plasma::Containment *panel = d->applet->containment();
             if (panel) {
                 QVariantList args;
-                args << url;
+                args << kurl.url();
 
                 // move it to the middle of the panel
                 QRectF rect(panel->geometry().width() / 2, 0, 150, panel->boundingRect().height());
@@ -254,6 +270,7 @@ void ContextMenuFactory::showContextMenu(QAbstractItemView *view, const QPoint &
     delete viewSeparator;
     delete ejectAction;
 }
+
 void ContextMenuFactory::setViewActions(QAbstractItemView *view, const QList<QAction*>& actions)
 {
     if (actions.isEmpty()) {
