@@ -88,21 +88,25 @@ public:
             , autoHide(false)
             , visibleItemCount(10)
             , placement(Plasma::TopPosedLeftAlignedPopup)
-            , panelEdge(Plasma::BottomEdge) {
+            , panelEdge(Plasma::BottomEdge)
+    {
     }
 
-    ~Private() {
+    ~Private()
+    {
     }
 
     enum TabOrder { NormalTabOrder, ReverseTabOrder };
 
-    void setupEventHandler(QAbstractItemView *view) {
+    void setupEventHandler(QAbstractItemView *view)
+    {
         view->viewport()->installEventFilter(q);
         view->installEventFilter(q);
     }
 
     void addView(const QString& name, const QIcon& icon,
-                 QAbstractItemModel *model = 0, QAbstractItemView *view = 0) {
+                 QAbstractItemModel *model = 0, QAbstractItemView *view = 0)
+    {
         view->setFrameStyle(QFrame::NoFrame);
         // prevent the view from stealing focus from the search bar
         view->setFocusPolicy(Qt::NoFocus);
@@ -127,7 +131,8 @@ public:
         contentArea->addWidget(view);
     }
 
-    void initTabs() {
+    void initTabs()
+    {
         // Favorites view
         setupFavoritesView();
 
@@ -147,7 +152,8 @@ public:
         setupSearchView();
     }
 
-    void setupLeaveView() {
+    void setupLeaveView()
+    {
         leaveModel = new LeaveModel(q);
         leaveModel->updateModel();
         UrlItemView *view = new UrlItemView();
@@ -159,15 +165,16 @@ public:
         addView(i18n("Leave"), KIcon("system-shutdown"), leaveModel, view);
     }
 
-    void setupFavoritesView() {
-        FavoritesModel *model = new FavoritesModel(q);
+    void setupFavoritesView()
+    {
+        favoritesModel = new FavoritesModel(q);
         UrlItemView *view = new UrlItemView();
         ItemDelegate *delegate = new ItemDelegate(q);
         delegate->setRoleMapping(Plasma::Delegate::SubTitleRole, SubTitleRole);
         delegate->setRoleMapping(Plasma::Delegate::SubTitleMandatoryRole, SubTitleMandatoryRole);
         view->setItemDelegate(delegate);
         view->setItemStateProvider(delegate);
-        addView(i18n("Favorites"), KIcon("bookmarks"), model, view);
+        addView(i18n("Favorites"), KIcon("bookmarks"), favoritesModel, view);
 
         QAction *sortAscendingAction = new QAction(KIcon("view-sort-ascending"), 
                                                    i18n("Sort Alphabetically (A to Z)"), q);
@@ -176,9 +183,9 @@ public:
                                                     i18n("Sort Alphabetically (Z to A)"), q);  
 
 
-        connect(model, SIGNAL(rowsInserted(QModelIndex, int, int)), q, SLOT(focusFavoritesView()));
-        connect(sortAscendingAction, SIGNAL(triggered()), model, SLOT(sortFavoritesAscending()));
-        connect(sortDescendingAction, SIGNAL(triggered()), model, SLOT(sortFavoritesDescending()));
+        connect(favoritesModel, SIGNAL(rowsInserted(QModelIndex, int, int)), q, SLOT(focusFavoritesView()));
+        connect(sortAscendingAction, SIGNAL(triggered()), favoritesModel, SLOT(sortFavoritesAscending()));
+        connect(sortDescendingAction, SIGNAL(triggered()), favoritesModel, SLOT(sortFavoritesDescending()));
 
         favoritesView = view;
         QList<QAction*> actions;
@@ -186,8 +193,9 @@ public:
         contextMenuFactory->setViewActions(view, actions);
     }
 
-    void setupAllProgramsView() {
-        ApplicationModel *applicationModel = new ApplicationModel(q);
+    void setupAllProgramsView()
+    {
+        applicationModel = new ApplicationModel(q);
         applicationModel->setDuplicatePolicy(ApplicationModel::ShowLatestOnlyPolicy);
 
         applicationView = new FlipScrollView();
@@ -200,26 +208,28 @@ public:
                 applicationModel, applicationView);
     }
 
-    void setupRecentView() {
-        RecentlyUsedModel *model = new RecentlyUsedModel(q);
+    void setupRecentView()
+    {
+        recentlyUsedModel = new RecentlyUsedModel(q);
         UrlItemView *view = new UrlItemView();
         ItemDelegate *delegate = new ItemDelegate(q);
         delegate->setRoleMapping(Plasma::Delegate::SubTitleRole, SubTitleRole);
         delegate->setRoleMapping(Plasma::Delegate::SubTitleMandatoryRole, SubTitleMandatoryRole);
         view->setItemDelegate(delegate);
         view->setItemStateProvider(delegate);
-        addView(i18n("Recently Used"), KIcon("document-open-recent"), model, view);
+        addView(i18n("Recently Used"), KIcon("document-open-recent"), recentlyUsedModel, view);
 
         QAction *clearApplications = new QAction(KIcon("edit-clear-history"), i18n("Clear Recent Applications"), q);
         QAction *clearDocuments = new QAction(KIcon("edit-clear-history"), i18n("Clear Recent Documents"), q);
 
-        connect(clearApplications, SIGNAL(triggered()), model, SLOT(clearRecentApplications()));
-        connect(clearDocuments, SIGNAL(triggered()), model, SLOT(clearRecentDocuments()));
+        connect(clearApplications, SIGNAL(triggered()), recentlyUsedModel, SLOT(clearRecentApplications()));
+        connect(clearDocuments, SIGNAL(triggered()), recentlyUsedModel, SLOT(clearRecentDocuments()));
 
         contextMenuFactory->setViewActions(view, QList<QAction*>() << clearApplications << clearDocuments);
     }
 
-    void setupSystemView() {
+    void setupSystemView()
+    {
         SystemModel *model = new SystemModel(q);
         UrlItemView *view = new UrlItemView();
         ItemDelegate *delegate = new ItemDelegate(q);
@@ -231,7 +241,8 @@ public:
         addView(i18n("Computer"), systemIcon(), model, view);
     }
 
-    void setupSearchView() {
+    void setupSearchView()
+    {
         searchModel = new SearchModel(q);
         UrlItemView *view = new UrlItemView();
         ItemDelegate *delegate = new ItemDelegate(q);
@@ -258,12 +269,14 @@ public:
         searchView = view;
     }
 
-    void registerUrlHandlers() {
+    void registerUrlHandlers()
+    {
         UrlItemLauncher::addGlobalHandler(UrlItemLauncher::ExtensionHandler, "desktop", new ServiceItemHandler);
         UrlItemLauncher::addGlobalHandler(UrlItemLauncher::ProtocolHandler, "leave", new LeaveItemHandler);
     }
 
-    QIcon systemIcon() {
+    QIcon systemIcon()
+    {
         QList<Solid::Device> batteryList = Solid::Device::listFromType(Solid::DeviceInterface::Battery, QString());
 
         if (batteryList.isEmpty()) {
@@ -273,7 +286,8 @@ public:
         }
     }
 
-    void setNorthLayout(TabOrder tabOrder) {
+    void setNorthLayout(TabOrder tabOrder)
+    {
         contentSwitcher->setShape(QTabBar::RoundedNorth);
         QLayout * layout = q->layout();
         delete layout;
@@ -290,7 +304,8 @@ public:
         setTabOrder(tabOrder);
     }
 
-    void setSouthLayout(TabOrder tabOrder) {
+    void setSouthLayout(TabOrder tabOrder)
+    {
         contentSwitcher->setShape(QTabBar::RoundedSouth);
         QLayout * layout = q->layout();
         delete layout;
@@ -307,7 +322,8 @@ public:
         setTabOrder(tabOrder);
     }
 
-    void setWestLayout(TabOrder tabOrder) {
+    void setWestLayout(TabOrder tabOrder)
+    {
         contentSwitcher->setShape(QTabBar::RoundedWest);
         QLayout * layout = q->layout();
         delete layout;
@@ -339,7 +355,8 @@ public:
         setTabOrder(tabOrder);
     }
 
-    void setEastLayout(TabOrder tabOrder) {
+    void setEastLayout(TabOrder tabOrder)
+    {
         contentSwitcher->setShape(QTabBar::RoundedEast);
         QLayout * layout = q->layout();
         delete layout;
@@ -371,7 +388,8 @@ public:
         setTabOrder(tabOrder);
     }
 
-    void setTabOrder(TabOrder newOrder) {
+    void setTabOrder(TabOrder newOrder)
+    {
         // identify current TabOrder, assumes favoritesView is first in normal order
         TabOrder oldOrder;
         if (contentArea->widget(0) == favoritesView) {
@@ -418,12 +436,15 @@ public:
         QString tabToolTip;
         QString tabWhatsThis;
         QIcon tabIcon;
-        QWidget * widget;
+        QWidget *widget;
     };
 
     Launcher * const q;
     Plasma::Applet *applet;
     UrlItemLauncher *urlLauncher;
+    FavoritesModel *favoritesModel;
+    ApplicationModel  *applicationModel;
+    RecentlyUsedModel *recentlyUsedModel;
     SearchModel *searchModel;
     LeaveModel *leaveModel;
     SearchBar *searchBar;
@@ -456,6 +477,7 @@ Launcher::Launcher(Plasma::Applet *applet)
 {
     init();
     setApplet(applet);
+
 }
 
 void Launcher::init()
@@ -588,14 +610,48 @@ bool Launcher::autoHide() const
 
 void Launcher::setSwitchTabsOnHover(bool switchOnHover)
 {
+    if (d->applet && switchOnHover != d->contentSwitcher->switchTabsOnHover()) {
+        KConfigGroup cg = d->applet->globalConfig();
+        cg.writeEntry("SwitchTabsOnHover", switchOnHover);
+        emit configNeedsSaving();
+    }
+
     d->contentSwitcher->setSwitchTabsOnHover(switchOnHover);
 }
 
+void Launcher::setShowAppsByName(bool showAppsByName)
+{
+    //ApplicationModel *m_applicationModel = applicationModel;
+    const bool wasByName = d->applicationModel->nameDisplayOrder() == Kickoff::NameBeforeDescription;
+    if (d->applet && showAppsByName != wasByName) {
+        KConfigGroup cg = d->applet->config();
+        cg.writeEntry("ShowAppsByName", showAppsByName);
+        emit configNeedsSaving();
+    }
+
+    if (showAppsByName) {
+        d->applicationModel->setNameDisplayOrder(Kickoff::NameBeforeDescription);
+        d->recentlyUsedModel->setNameDisplayOrder(Kickoff::NameBeforeDescription);
+        d->favoritesModel->setNameDisplayOrder(Kickoff::NameBeforeDescription);
+        d->searchModel->setNameDisplayOrder(Kickoff::NameBeforeDescription);
+    } else {
+        d->applicationModel->setNameDisplayOrder(Kickoff::NameAfterDescription);
+        d->recentlyUsedModel->setNameDisplayOrder(Kickoff::NameAfterDescription);
+        d->favoritesModel->setNameDisplayOrder(Kickoff::NameAfterDescription);
+        d->searchModel->setNameDisplayOrder(Kickoff::NameAfterDescription);
+    }
+}
+
 bool Launcher::switchTabsOnHover() const
 {
     return d->contentSwitcher->switchTabsOnHover();
 }
 
+bool Launcher::showAppsByName() const
+{
+  return d->applicationModel->nameDisplayOrder() == Kickoff::NameBeforeDescription;
+}
+
 void Launcher::setVisibleItemCount(int count)
 {
     d->visibleItemCount = count;
@@ -608,14 +664,15 @@ int Launcher::visibleItemCount() const
 
 void Launcher::setApplet(Plasma::Applet *applet)
 {
-    d->applet = applet;
-    d->contextMenuFactory->setApplet(applet);
-
     KConfigGroup cg = applet->globalConfig();
     setSwitchTabsOnHover(cg.readEntry("SwitchTabsOnHover", switchTabsOnHover()));
 
     cg = applet->config();
+    setShowAppsByName(cg.readEntry("ShowAppsByName", showAppsByName()));
     setVisibleItemCount(cg.readEntry("VisibleItemsCount", visibleItemCount()));
+
+    d->applet = applet;
+    d->contextMenuFactory->setApplet(applet);
 }
 
 void Launcher::reset()
@@ -649,6 +706,8 @@ void Launcher::focusFavoritesView()
 {
     d->contentSwitcher->setCurrentIndex(d->contentArea->indexOf(d->favoritesView));
     d->contentArea->setCurrentWidget(d->favoritesView);
+    d->contentSwitcher->setVisible(true);
+    d->searchBar->clear();
 }
 
 bool Launcher::eventFilter(QObject *object, QEvent *event)
@@ -721,7 +780,8 @@ void Launcher::showViewContextMenu(const QPoint& pos)
 {
     QAbstractItemView *view = qobject_cast<QAbstractItemView*>(sender());
     if (view) {
-        d->contextMenuFactory->showContextMenu(view, pos);
+        const QModelIndex index = view->indexAt(pos);
+        d->contextMenuFactory->showContextMenu(view, index, pos);
     }
 }
 
@@ -765,21 +825,34 @@ void Launcher::setLauncherOrigin(const Plasma::PopupPlacement placement, Plasma:
     if (d->placement != placement) {
         d->placement = placement;
 
+        Private::TabOrder normalOrder;
+        Private::TabOrder reverseOrder;
+
+        //QTabBar attempts to be smart and flips what we say in rtl mode so say the opposite when the tabbar is horizontal
+        if (QApplication::layoutDirection() == Qt::RightToLeft) {
+            normalOrder = Private::ReverseTabOrder;
+            reverseOrder = Private::NormalTabOrder;
+        } else {
+            normalOrder = Private::NormalTabOrder;
+            reverseOrder = Private::ReverseTabOrder;
+        }
+
         switch (placement) {
         case Plasma::TopPosedRightAlignedPopup:
-            d->setSouthLayout(Private::ReverseTabOrder);
+            d->setSouthLayout(reverseOrder);
             break;
         case Plasma::LeftPosedTopAlignedPopup:
+            //when the tabbar is vertical it's fine
             d->setEastLayout(Private::NormalTabOrder);
             break;
         case Plasma::LeftPosedBottomAlignedPopup:
             d->setEastLayout(Private::ReverseTabOrder);
             break;
         case Plasma::BottomPosedLeftAlignedPopup:
-            d->setNorthLayout(Private::NormalTabOrder);
+            d->setNorthLayout(normalOrder);
             break;
         case Plasma::BottomPosedRightAlignedPopup:
-            d->setNorthLayout(Private::ReverseTabOrder);
+            d->setNorthLayout(reverseOrder);
             break;
         case Plasma::RightPosedTopAlignedPopup:
             d->setWestLayout(Private::NormalTabOrder);
@@ -789,7 +862,7 @@ void Launcher::setLauncherOrigin(const Plasma::PopupPlacement placement, Plasma:
             break;
         case Plasma::TopPosedLeftAlignedPopup:
         default:
-            d->setSouthLayout(Private::NormalTabOrder);
+            d->setSouthLayout(normalOrder);
             break;
         }
     }
