@@ -103,6 +103,13 @@ class Pager : public Plasma::Applet
             None
         };
 
+        enum CurrentDesktopSelected
+        {
+            DoNothing,
+            ShowDesktop,
+            ShowDashboard
+        };
+
         struct AnimInfo
         {
             int animId;
@@ -112,12 +119,14 @@ class Pager : public Plasma::Applet
         };
 
         DisplayedText m_displayedText;
+        CurrentDesktopSelected m_currentDesktopSelected;
         bool m_showWindowIcons;
         bool m_showOwnBackground;
         int m_rows;
         int m_columns;
         int m_desktopCount;
         int m_currentDesktop;
+        bool m_desktopDown;
         qreal m_widthScaleFactor;
         qreal m_heightScaleFactor;
         QSizeF m_size;
@@ -133,6 +142,7 @@ class Pager : public Plasma::Applet
         KSelectionOwner* m_desktopLayoutOwner;
         Plasma::FrameSvg *m_background;
         KColorScheme *m_colorScheme;
+        bool m_verticalFormFactor;
 
         // dragging of windows
         QRect m_dragOriginal;
@@ -146,6 +156,7 @@ class Pager : public Plasma::Applet
         // desktop switching on drop event
         int m_dragSwitchDesktop;
         QTimer* m_dragSwitchTimer;
+        bool m_ignoreNextSizeConstraint;
 
         static const int s_FadeInDuration = 50;
         static const int s_FadeOutDuration = 100;
