@@ -32,6 +32,7 @@
 #include <KMimeType>
 #include <KStandardDirs>
 #include <KWindowSystem>
+#include <KIconLoader>
 
 #include <plasma/containment.h>
 #include <plasma/dialog.h>
@@ -39,32 +40,38 @@
 
 #include "math.h"
 
+//TODO: Make Dialog resizeable (which also makes the number of rows in it configurable :)
+
 static const int s_defaultIconSize = 16;
 static const int s_defaultSpacing = 2;
 
 QuicklaunchApplet::QuicklaunchApplet(QObject *parent, const QVariantList &args)
   : Plasma::Applet(parent, args),
+    m_layout(0),
+    m_innerLayout(0),
+    m_timer(new QTimer(this)),
     m_visibleIcons(6),
-    m_rowCount(2),
-    m_dialogRowCount(2),
+    m_preferredIconSize(s_defaultIconSize),
+    m_iconSize(s_defaultIconSize),
+    m_dialogIconSize(s_defaultIconSize),
     m_dialog(0),
     m_dialogWidget(0),
     m_dialogLayout(0),
     m_addDialog(0),
     m_rightClickedIcon(0),
     m_addAction(0),
-    m_removeAction(0),
-    m_layout(0),
-    m_innerLayout(0)
+    m_removeAction(0)
 {
     setHasConfigurationInterface(true);
     setAcceptDrops(true);
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
+    connect(m_timer, SIGNAL(timeout()), this, SLOT(performUiRefactor()));
+    m_timer->setSingleShot(true);
 
-    // set our default size here
-    resize((m_visibleIcons / m_rowCount) * s_defaultIconSize +
-            (s_defaultSpacing * (m_visibleIcons + 1)),
-           m_rowCount * 22 + s_defaultSpacing * 3);
+    qreal left, top, right, bottom;
+    getContentsMargins(&left, &top, &right, &bottom);
+    resize(s_defaultIconSize * 3 + s_defaultSpacing * 4 + left + right,
+           s_defaultIconSize + s_defaultSpacing * 2 + top + bottom);
 }
 
 QuicklaunchApplet::~QuicklaunchApplet()
@@ -90,9 +97,9 @@ void QuicklaunchApplet::saveState(KConfigGroup &config) const
 void QuicklaunchApplet::init()
 {
     KConfigGroup cg = config();
-    m_rowCount = qMax(1, cg.readEntry("rowCount", m_rowCount));
-    m_visibleIcons = qMax(1, cg.readEntry("visibleIcons", m_visibleIcons));
-    m_dialogRowCount = qMax(1, cg.readEntry("dialogRowCount", m_dialogRowCount));
+    m_preferredIconSize = m_iconSize = qMax(s_defaultIconSize, (int)cg.readEntry("iconSize", contentsRect().height() / 2));
+    m_visibleIcons = qMax(-1, cg.readEntry("visibleIcons", m_visibleIcons));
+    m_dialogIconSize = qMax(s_defaultIconSize, (int)cg.readEntry("dialogIconSize", contentsRect().height() / 2));
 
     // Initialize outer layout
     m_layout = new QGraphicsLinearLayout(this);
@@ -101,7 +108,7 @@ void QuicklaunchApplet::init()
     setLayout(m_layout);
 
     // Initialize inner layout
-    m_innerLayout = new QuicklaunchLayout(0, m_rowCount);
+    m_innerLayout = new QuicklaunchLayout(1, this, 0);
     m_innerLayout->setContentsMargins(0, 0, 0, 0);
     m_innerLayout->setSpacing(0);
     m_layout->addItem(m_innerLayout);
@@ -113,7 +120,8 @@ void QuicklaunchApplet::init()
 
     QStringList desktopFiles = cg.readEntry("iconUrls", QStringList());
 
-    if (desktopFiles.isEmpty()) {
+    bool firstStart = desktopFiles.isEmpty();
+    if (firstStart) {
         QStringList defaultApps;
         defaultApps << "konqbrowser" << "dolphin" << "kopete";
 
@@ -121,17 +129,24 @@ void QuicklaunchApplet::init()
             KService::Ptr service = KService::serviceByStorageId(defaultApp);
             if (service && service->isValid()) {
                 QString path = service->entryPath();
-                kDebug() << path;
+                //kDebug() << path;
                 if (!path.isEmpty() && QDir::isAbsolutePath(path)) {
                     desktopFiles << path;
                 }
             }
         }
-        kDebug() << desktopFiles;
+        //kDebug() << desktopFiles;
     }
 
-    loadPrograms(desktopFiles);
-    refactorUi();
+    foreach (const QString &desktopFile, desktopFiles) {
+        addProgram(-1, desktopFile);
+    }
+
+    performUiRefactor();
+
+    if (firstStart) {
+        resize(sizeHint(Qt::PreferredSize));
+    }
 }
 
 QSizeF QuicklaunchApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
@@ -141,16 +156,11 @@ QSizeF QuicklaunchApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint
         if (!m_innerLayout) {
             return sizeHint;
         }
+        qreal newWidth = m_innerLayout->columnCount() * sizeHint.height() / qMax(1, m_innerLayout->preferredRowCount());
         if (m_icons.size() > m_visibleIcons) {
-            qreal factor = m_innerLayout->rowCount() * m_innerLayout->columnCount() + size().height();
-            if (factor) {
-                sizeHint.setWidth(sizeHint.height() / factor);
-            }
+            sizeHint.setWidth(newWidth + sizeHint.height());
         } else {
-            qreal factor = m_innerLayout->rowCount() * m_innerLayout->columnCount();
-            if (factor) {
-                sizeHint.setWidth(sizeHint.height() / factor);
-            }
+            sizeHint.setWidth(newWidth);
         }
         return sizeHint;
     }
@@ -160,50 +170,60 @@ QSizeF QuicklaunchApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint
 void QuicklaunchApplet::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & Plasma::SizeConstraint) {
-        //TODO: don't call so often
         refactorUi();
     }
 }
 
 void QuicklaunchApplet::refactorUi()
 {
+     //static int my_counter = 0;
+     //kDebug() << "refactorUi count = " << my_counter++;
+     m_timer->start(100);
+}
+
+void QuicklaunchApplet::performUiRefactor()
+{
+    //static int my_perforcount = 0;
+    //kDebug() << "performUiRefactor count = " << my_perforcount++;
     clearLayout(m_innerLayout);
 
+    //Don't accept values under 16 nor anything over applet's height
+    m_iconSize = qMin(qMax(m_preferredIconSize, s_defaultIconSize), (int)(contentsRect().height() - 2 * s_defaultSpacing));
+    m_dialogIconSize = qMax(m_dialogIconSize, s_defaultIconSize);
+
     if (m_dialogLayout) {
         clearLayout(m_dialogLayout);
-        m_dialogLayout->setRowCount(m_dialogRowCount);
-    }
-    int rowCount;
-    int iconWidth;
-    if (formFactor() == Plasma::Vertical) {
-        rowCount = qMin(m_rowCount, int(size().width()) / (s_defaultIconSize + s_defaultSpacing));
-        // prevent possible division by zero if size().width() is 0
-        rowCount = qMax(1, rowCount );
-        iconWidth = size().width() / rowCount;
-    } else {
-        rowCount = qMin(m_rowCount, int(size().height()) / (s_defaultIconSize + s_defaultSpacing));
-        // prevent possible division by zero if size().height() is 0
-        rowCount = qMax(1, rowCount );
-        iconWidth = qMax(s_defaultIconSize, int(size().height()) / rowCount);
+        m_dialogLayout->setPreferredRowCount((int)(size().height() / qMin(m_dialogIconSize, m_dialog->size().height())));
     }
 
-    m_innerLayout->setRowCount(rowCount);
-    const QSizeF minSize = QSizeF(iconWidth, iconWidth);
-    const QSizeF maxSize = QSizeF(iconWidth, iconWidth);
+    int rowCount;
+    if (formFactor() == Plasma::Vertical) {
+        int numIcons = qMin(m_icons.count(), m_visibleIcons);
+        int colCount = qMax(qreal(1), contentsRect().width() / m_iconSize);
+        rowCount = numIcons / colCount + numIcons % colCount;
+        // prevent possible division by zero if size().width() is 0
+    } else {
+        rowCount = contentsRect().height() / m_iconSize;
+        // prevent possible division by zero if size().height() is 0
+    }
+
+    rowCount = qMax(1, rowCount);
+    m_innerLayout->setPreferredRowCount(rowCount);
     int count = 0;
-    kDebug() << m_icons.count() << iconWidth << "pixel icons in" << rowCount
-             << "rows, with a max of" << m_visibleIcons << "visible";
+    //kDebug() << m_icons.count() << "pixel icons in" << rowCount
+    //         << "rows, with a max of" << m_visibleIcons << "visible";
     foreach (QuicklaunchIcon *icon, m_icons) {
         //icon->setMinimumSize(minSize);
         //icon->setMaximumSize(maxSize);
-        icon->resize(minSize);
-
         if (count < m_visibleIcons || m_visibleIcons == -1) {
+            icon->resize(QSize(m_iconSize, m_iconSize));
+            icon->setIconSize(m_iconSize);
             icon->show();
             m_innerLayout->addItem(icon);
         } else if (m_dialogLayout) {
-            icon->setMinimumSize(minSize);
-            icon->setMaximumSize(maxSize);
+            icon->setMinimumSize(QSize(m_dialogIconSize, m_dialogIconSize));//TODO: Remove maximum/minimum sizes
+            icon->setMaximumSize(QSize(m_dialogIconSize, m_dialogIconSize));
+            icon->setIconSize(m_dialogIconSize);
             icon->show();
             m_dialogLayout->addItem(icon);
         } else {
@@ -217,20 +237,26 @@ void QuicklaunchApplet::refactorUi()
     if (count > m_visibleIcons && m_visibleIcons != -1) {
         //m_arrow->setMinimumSize(minSize);
         //m_arrow->setMaximumSize(maxSize);
-        m_arrow->resize(minSize);
+        m_arrow->resize(QSize(size().height(), size().height()));
         m_layout->addItem(m_arrow);
         m_arrow->show();
     } else {
         m_arrow->hide();
     }
 
+    int cols = qMax(1,m_innerLayout->columnCount());
+    int icons = qMax(1, qMin(m_icons.size(), m_visibleIcons));
+
+    setPreferredSize(QSize((m_iconSize + 6) * cols,
+                           (m_iconSize + 6) * ceil(icons / (double)cols)));
+    
     if (m_dialog) {
         m_dialog->close();
         m_dialogLayout->updateGeometry();
         m_dialog->adjustSize();
     }
-    m_innerLayout->updateGeometry();
-    m_layout->updateGeometry();
+
+    update();
 }
 
 void QuicklaunchApplet::showDialog()
@@ -246,9 +272,9 @@ void QuicklaunchApplet::showDialog()
         m_dialog->setAcceptDrops(true);
         //m_dialog->installEventFilter(this);
         m_dialog->setContextMenuPolicy(Qt::ActionsContextMenu);
-        m_dialogLayout = new QuicklaunchLayout(m_dialogWidget, m_dialogRowCount);
+        m_dialogLayout = new QuicklaunchLayout(1, m_dialogWidget, m_dialogWidget);
         m_dialogWidget->setLayout(m_dialogLayout);
-        refactorUi();
+        performUiRefactor();
         m_dialog->setGraphicsWidget(m_dialogWidget);
     }
 
@@ -257,13 +283,11 @@ void QuicklaunchApplet::showDialog()
     } else {
         m_dialog->resize(m_dialogLayout->preferredSize().toSize());
         //m_dialog->updateGeometry();
-        if(containment() && containment()->corona()) {
-            kDebug() << "position:" << containment()->corona()->popupPosition(m_arrow, m_dialog->size()) << "dialog size:" << m_dialog->size() << "layout preferred-size:" << m_dialogLayout->preferredSize().toSize();
+        if (containment() && containment()->corona()) {
+            //kDebug() << "position:" << containment()->corona()->popupPosition(m_arrow, m_dialog->size()) << "dialog size:" << m_dialog->size() << "layout preferred-size:" << m_dialogLayout->preferredSize().toSize();
             m_dialog->move(containment()->corona()->popupPosition(m_arrow, m_dialog->size()));
         }
         KWindowSystem::setState(m_dialog->winId(), NET::SkipTaskbar);
-        //QPoint(popupPosition(m_dialog->sizeHint()).x() + (m_visibleIcons) * size().height() / m_rowCount,
-        //               popupPosition(m_dialog->sizeHint()).y()));
         m_dialog->show();
     }
 }
@@ -273,25 +297,39 @@ void QuicklaunchApplet::createConfigurationInterface(KConfigDialog *parent)
     QWidget *widget = new QWidget(parent);
     uiConfig.setupUi(widget);
     connect(parent, SIGNAL(accepted()), SLOT(configAccepted()));
-    uiConfig.rowCount->setValue(m_rowCount);
-    uiConfig.dialogRowCount->setValue(m_dialogRowCount);
-    uiConfig.dialogRowCount->hide();
-    uiConfig.dialogrowLabel->hide();
+
+    int dialogHeight = (int)KIconLoader::SizeEnormous;
+    int height = dialogHeight;
+
+    if (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical) {
+        height = contentsRect().height() - 2 * s_defaultSpacing;
+    }
+
+    uiConfig.iconSizeSpin->setMaximum(height);
+    uiConfig.iconSizeSlider->setMaximum(height);
+    uiConfig.dialogIconSizeSpin->setMaximum(dialogHeight);
+    uiConfig.dialogIconSizeSlider->setMaximum(dialogHeight);
+
+    uiConfig.iconSizeSpin->setMinimum(s_defaultIconSize);
+    uiConfig.iconSizeSlider->setMinimum(s_defaultIconSize);
+    uiConfig.dialogIconSizeSpin->setMinimum(s_defaultIconSize);
+    uiConfig.dialogIconSizeSlider->setMinimum(s_defaultIconSize);
+
+    uiConfig.iconSizeSpin->setValue(m_preferredIconSize);
+    uiConfig.iconSizeSlider->setValue(m_preferredIconSize);
+    uiConfig.dialogIconSizeSpin->setValue(m_dialogIconSize);
+    uiConfig.dialogIconSizeSlider->setValue(m_dialogIconSize);
+
     uiConfig.icons->setValue(m_visibleIcons);
     parent->addPage(widget, i18n("General"), icon());
 }
 
 void QuicklaunchApplet::configAccepted()
 {
-    bool changed = false;
-    int temp = uiConfig.rowCount->value();
-
     KConfigGroup cg = config();
-    if (temp != m_rowCount) {
-        m_rowCount = temp;
-        cg.writeEntry("rowCount", m_rowCount);
-        changed = true;
-    }
+
+    bool changed = false;
+    int temp;
 
     temp = uiConfig.icons->value();
     if (temp != m_visibleIcons) {
@@ -300,16 +338,24 @@ void QuicklaunchApplet::configAccepted()
         changed = true;
     }
 
-    temp = uiConfig.dialogRowCount->value();
-    if (temp != m_dialogRowCount) {
-        m_dialogRowCount = temp;
-        cg.writeEntry("dialogRowCount", m_dialogRowCount);
+    temp = uiConfig.iconSizeSpin->value();
+    if (temp != m_preferredIconSize) {
+        m_preferredIconSize = temp;
+        cg.writeEntry("iconSize", m_preferredIconSize);
+        changed = true;
+    }
+
+    temp = uiConfig.dialogIconSizeSpin->value();
+    if (temp != m_dialogIconSize) {
+        m_dialogIconSize = temp;
+        cg.writeEntry("dialogIconSize", m_dialogIconSize);
         changed = true;
     }
 
     if (changed) {
         emit configNeedsSaving();
-        refactorUi();
+        resize(sizeHint(Qt::PreferredSize));
+        performUiRefactor();
     }
 }
 
@@ -335,13 +381,128 @@ QList<QAction*> QuicklaunchApplet::contextActions(QuicklaunchIcon *icon)
     return tempActions;
 }
 
+void QuicklaunchApplet::dropApp(QGraphicsSceneDragDropEvent *event, bool droppedOnDialog)
+{
+    int pos = 0;
+    if (droppedOnDialog) {
+        QPointF point = event->pos();
+        for(int i = 0; i < m_dialogLayout->count(); i++) {
+           QGraphicsLayoutItem *item = m_dialogLayout->itemAt(i);
+           if (item->geometry().contains(point)) {
+               //m_dialogLayout->insertItem(dropedItem, i + 1);
+               pos = i + 1 + m_visibleIcons;
+               kDebug() << "The position of Droped Item in dialog is = " << pos;
+               break;
+           }
+        }
+    } else if (!m_icons.isEmpty()) {
+        //qreal left, top, bottom, right;
+        //getContentsMargins(&left, &top, &bottom, &right);
+        QPointF point = mapFromScene(event->scenePos());//) + QPointF(left, top);
+        int rowCount = m_innerLayout->rowCount();
+        //kDebug() << "RowCount = " << rowCount;
+        int colCount = m_innerLayout->columnCount();
+        int colWidth = m_innerLayout->geometry().width();
+        int col = 0;
+        while (col < colCount) {
+
+            if (col == colCount || col * rowCount >= m_icons.count()) {
+                break;
+            }
+
+            //kDebug() << col << m_icons.at(col * rowCount)->geometry().left() << point.x();
+            if (m_icons.at(col * rowCount)->geometry().left() > point.x()) {
+                //kDebug() << "broke col at" << col;
+                break;
+            }
+
+            ++col;
+        }
+
+        //int col = static_cast<int>((round(point.x()) * colCount / m_innerLayout->geometry().width()));
+        //col = (col >= colCount) ? colCount - 1 : col;
+        //int row = static_cast<int>(floor(point.y() * rowCount / m_innerLayout->geometry().height()));
+        //row = (row >= m_innerLayout->rowCount()) ? m_innerLayout->rowCount() - 1 : row;
+        //kDebug() << "doing rows";
+        int row = -1;
+        while (row < rowCount) {
+            ++row;
+            if (row == rowCount || row == m_icons.count()) {
+                //kDebug() << "row made rowCount" << row;
+                break;
+            }
+
+            //kDebug() << "row: " << row << m_icons.count();
+            if (m_icons.at(row)->geometry().center().y() > point.y()) {
+                //kDebug() << "broke row at" << row;
+                break;
+            }
+        }
+
+        //kDebug() << "row = " << row << "rows = " << rowCount << "cols = " << colCount << "col = " << col;
+        if (rowCount > 1) {
+            pos = row + rowCount * (col - 1);
+        } else {
+            pos = col;
+        }
+
+        //kDebug() << "position is " << pos;
+    }
+
+    if (dropHandler(pos, event->mimeData())) {
+       event->setDropAction(Qt::MoveAction);
+       event->accept();
+       saveConfig();
+       performUiRefactor();
+    }
+}
+
 bool QuicklaunchApplet::eventFilter(QObject * object, QEvent * event)
 {
-    Q_UNUSED(object)
+    if (event->type() == QEvent::GraphicsSceneMousePress) {
+        m_mousePressPos = static_cast<QGraphicsSceneMouseEvent*>(event)->pos();
+    }
+
+    if (event->type() == QEvent::GraphicsSceneMouseMove) {
+        QuicklaunchIcon * icon = qobject_cast<QuicklaunchIcon*>(object);
+        if (m_icons.contains(icon)) {
+            QGraphicsSceneMouseEvent *mouseEvent = static_cast<QGraphicsSceneMouseEvent*>(event);
+            int dragDistance = (mouseEvent->pos() - m_mousePressPos).toPoint().manhattanLength();
+            if (dragDistance >= QApplication::startDragDistance()) {
+                int numberOfPreviousIcons = m_icons.count();
+                QMimeData *mimeData = new QMimeData();
+                mimeData->setData("text/uri-list", icon->url().url().toAscii());
+                mimeData->setText(mimeData->text());
+
+                QDrag *drag = new QDrag(mouseEvent->widget());
+                drag->setMimeData(mimeData);
+                drag->setPixmap(icon->icon().pixmap(m_iconSize, m_iconSize));
+
+                Qt::DropAction dropAction = drag->exec();
+
+                // If the current number of icons is more than the previous amount then
+                // it means that the icon was re-arranged and not dragged outside the applet
+                // Therefore we remove the previous icon
+                if (dropAction == Qt::MoveAction && m_icons.count() > numberOfPreviousIcons) {
+                    m_icons.removeAll(icon);
+                    icon->hide();
+                    icon->deleteLater();
+
+                    KConfigGroup cg = config();
+                    saveState(cg);
+                    emit configNeedsSaving();
+
+                    performUiRefactor();
+                }
+            }
+        }
+    }
+
     if (event->type() == QEvent::GraphicsSceneDrop) {
-        dropEvent(static_cast<QGraphicsSceneDragDropEvent*>(event));
+        dropApp(static_cast<QGraphicsSceneDragDropEvent*>(event), object == m_dialogWidget);
         return true;
     }
+
     return QObject::eventFilter(object, event);
 }
 
@@ -359,59 +520,52 @@ void QuicklaunchApplet::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
 
 void QuicklaunchApplet::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
-    // Calculate position
-    QPointF point = mapFromScene(event->scenePos());
-    int rowCount = m_innerLayout->rowCount();
-    int cols = static_cast<int>(ceil(1.0 * qMin(m_icons.size(), m_visibleIcons) / rowCount));
-    int col = static_cast<int>((round(point.x()) * cols / m_innerLayout->geometry().width()));
-    col = (col >= cols) ? col - 1 : col;
-    int row = static_cast<int>(floor(point.y() * rowCount / m_innerLayout->geometry().height()));
-    row = (row >= m_rowCount) ? row - 1 : row;
-    int pos = row * cols + col;
-
-    if (pos >= m_icons.size()) {
-        pos = m_icons.size() - 1;
-    }
-
-    if (dropHandler(pos, event->mimeData())) {
-        event->setDropAction(Qt::MoveAction);
-        event->accept();
-        saveConfig();
-        refactorUi();
-    }
+    dropApp(static_cast<QGraphicsSceneDragDropEvent*>(event),false);
 }
 
-void QuicklaunchApplet::addProgram(int index, const QString &url)
+void QuicklaunchApplet::addProgram(int index, const QString &url, bool isNewIcon)
 {
     if (index < 0 || index > m_icons.size()) {
         index = m_icons.size();
     }
-    if( url.isEmpty() )
+
+    if (url.isEmpty()) {
         return;
+    }
+
     KUrl appUrl = KUrl(url);
     KIcon icon;
+    QString text;
+    QString genericName;
 
     if (appUrl.isLocalFile() && KDesktopFile::isDesktopFile(appUrl.toLocalFile())) {
         KDesktopFile *f = new KDesktopFile(appUrl.toLocalFile());
+
+        text = f->readName();
         icon = KIcon(f->readIcon());
+        genericName = f->readGenericName();
         delete f;
     } else {
         icon = KIcon(KMimeType::iconNameForUrl(appUrl));
     }
 
+    if (text.isNull()) {
+        text = appUrl.fileName();
+    }
+
     if (icon.isNull()) {
         icon = KIcon("unknown");
     }
 
-    QuicklaunchIcon *container = new QuicklaunchIcon(appUrl, icon, this);
+    QuicklaunchIcon *container = new QuicklaunchIcon(appUrl, text, icon, genericName, this);
     container->installEventFilter(this);
-    m_icons.insert(index, container);
-}
 
-void QuicklaunchApplet::loadPrograms(const QStringList &desktopFiles)
-{
-    foreach (const QString &desktopFile, desktopFiles) {
-        addProgram(-1, desktopFile);
+    m_icons.insert(index, container);
+
+    if (isNewIcon) {
+        KConfigGroup cg = config();
+        saveState(cg);
+        emit configNeedsSaving();
     }
 }
 
@@ -427,7 +581,11 @@ void QuicklaunchApplet::removeCurrentIcon()
     m_icons.removeAll(m_rightClickedIcon);
     m_rightClickedIcon->hide();
     m_rightClickedIcon->deleteLater();
-    refactorUi();
+    performUiRefactor();
+
+    KConfigGroup cg = config();
+    saveState(cg);
+    emit configNeedsSaving();
 }
 
 bool QuicklaunchApplet::dropHandler(const int pos, const QMimeData *mimedata)
@@ -438,7 +596,7 @@ bool QuicklaunchApplet::dropHandler(const int pos, const QMimeData *mimedata)
 
     KUrl::List urls = KUrl::List::fromMimeData(mimedata);
 
-    if (!urls.count()) {
+    if (urls.isEmpty()) {
         return false;
     }
 
@@ -448,8 +606,8 @@ bool QuicklaunchApplet::dropHandler(const int pos, const QMimeData *mimedata)
     }
 
     foreach (const KUrl &url, urls) {
-        if(KDesktopFile::isDesktopFile(url.toLocalFile())) {
-            addProgram(pos, url.toLocalFile());
+        if (KDesktopFile::isDesktopFile(url.toLocalFile())) {
+            addProgram(pos, url.toLocalFile(), true);
         }
     }
     return true;
@@ -473,8 +631,8 @@ void QuicklaunchApplet::showAddInterface()
 void QuicklaunchApplet::addAccepted()
 {
     int insertplace = m_rightClickedIcon ? m_icons.indexOf(m_rightClickedIcon) : m_icons.size();
-    addProgram(insertplace, addUi.urlIcon->url().url());
-    refactorUi();
+    addProgram(insertplace, addUi.urlIcon->url().url(), true);
+    performUiRefactor();
 }
 
 K_EXPORT_PLASMA_APPLET(quicklaunch, QuicklaunchApplet)
