@@ -19,17 +19,21 @@
 #include "net.h"
 #include <Plasma/SignalPlotter>
 #include <Plasma/Theme>
+#include <Plasma/ToolTipManager>
 #include <KConfigDialog>
 #include <QTimer>
 #include <QGraphicsLinearLayout>
 
 SM::Net::Net(QObject *parent, const QVariantList &args)
     : SM::Applet(parent, args)
+    , m_rx("network/interfaces/(\\w+)/transmitter/data")
 {
     setHasConfigurationInterface(true);
     resize(234 + 20 + 23, 135 + 20 + 25);
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
+    m_sourceTimer.setSingleShot(true);
+    connect(&m_sourceTimer, SIGNAL(timeout()), this, SLOT(sourcesAdded()));
 }
 
 SM::Net::~Net()
@@ -38,40 +42,50 @@ SM::Net::~Net()
 
 void SM::Net::init()
 {
+    KGlobal::locale()->insertCatalog("plasma_applet_system-monitor");
     KConfigGroup cg = config();
     setEngine(dataEngine("systemmonitor"));
     setInterval(cg.readEntry("interval", 2) * 1000);
     setTitle(i18n("Network"));
-    if (engine()->sources().count() == 0) {
-        connect(engine(), SIGNAL(sourceAdded(QString)), this, SLOT(initLater(const QString)));
-    } else {
-        parseSources();
+    
+    m_showTopBar = cg.readEntry("showTopBar", true);
+    m_showBackground = cg.readEntry("showBackground", true);
+    m_inColor = cg.readEntry("inColor", QColor("#d2d200"));
+    m_outColor = cg.readEntry("outColor", QColor("#f20000"));
+    
+    connect(engine(), SIGNAL(sourceAdded(const QString&)),
+            this, SLOT(sourceAdded(const QString&)));
+    connect(engine(), SIGNAL(sourceRemoved(const QString&)),
+            this, SLOT(sourceRemoved(const QString&)));
+    if (!engine()->sources().isEmpty()) {
+        sourcesAdded();
     }
 }
 
-void SM::Net::parseSources()
+void SM::Net::sourceAdded(const QString& name)
 {
-    QRegExp rx("network/interfaces/(\\w+)/transmitter/data");
-
-    foreach (const QString& s, engine()->sources()) {
-        if (rx.indexIn(s) != -1) {
-            //kDebug() << rx.cap(1);
-            if (rx.cap(1) != "lo") {
-                m_interfaces << s;
+    if (m_rx.indexIn(name) != -1) {
+        //kDebug() << m_rx.cap(1);
+        if (m_rx.cap(1) != "lo") {
+            m_interfaces << name;
+            if (!m_sourceTimer.isActive()) {
+                m_sourceTimer.start(0);
             }
         }
     }
+}
+
+void SM::Net::sourcesAdded()
+{
+    //kDebug() << m_interfaces;
     KConfigGroup cg = config();
     setItems(cg.readEntry("interfaces", m_interfaces));
     connectToEngine();
 }
 
-void SM::Net::initLater(const QString &name)
+void SM::Net::sourceRemoved(const QString& name)
 {
-    // How we know all (network) sources are ready???
-    if (name == "ps") {
-        QTimer::singleShot(0, this, SLOT(parseSources()));
-    }
+    m_interfaces.removeAll(name);
 }
 
 bool SM::Net::addMeter(const QString& source)
@@ -83,14 +97,12 @@ bool SM::Net::addMeter(const QString& source)
     QString interface = l[2];
     Plasma::Theme* theme = Plasma::Theme::defaultTheme();
     Plasma::SignalPlotter *plotter = new Plasma::SignalPlotter(this);
-    QColor color = theme->color(Plasma::Theme::TextColor);
-    plotter->addPlot(QColor(((color.red() - 128) * 0.65) + 128,
-                            ((color.green() - 128) * 0.65) + 128, 0, color.alpha()));
-    plotter->addPlot(QColor(((color.red() - 128) * 0.90) + 128, 0, 0, color.alpha()));
+    plotter->addPlot(m_inColor);
+    plotter->addPlot(m_outColor);
     plotter->setUseAutoRange(true);
     plotter->setThinFrame(false);
-    plotter->setShowLabels(true);
-    plotter->setShowTopBar(false);
+    plotter->setShowLabels(false);
+    plotter->setShowTopBar(m_showTopBar);
     plotter->setShowVerticalLines(false);
     plotter->setShowHorizontalLines(false);
     plotter->setStackPlots(true);
@@ -101,7 +113,12 @@ bool SM::Net::addMeter(const QString& source)
     plotter->setHorizontalLinesColor(theme->color(Plasma::Theme::HighlightColor));
     plotter->setVerticalLinesColor(theme->color(Plasma::Theme::HighlightColor));
     plotter->setHorizontalLinesCount(4);
-    //plotter->setSvgBackground("widgets/plot-background");
+    if (m_showBackground) {
+        plotter->setSvgBackground("widgets/plot-background");
+    } else {
+        plotter->setSvgBackground(QString());
+        plotter->setBackgroundColor(Qt::transparent);
+    }
     plotter->setTitle(interface);
     plotter->setUnit("KiB/s");
     appendPlotter(interface, plotter);
@@ -114,13 +131,7 @@ bool SM::Net::addMeter(const QString& source)
 void SM::Net::themeChanged()
 {
     Plasma::Theme* theme = Plasma::Theme::defaultTheme();
-    QColor color = theme->color(Plasma::Theme::TextColor);
-    foreach (Plasma::SignalPlotter *plotter, plotters().values()) {
-        plotter->removePlot(1);
-        plotter->removePlot(0);
-        plotter->addPlot(QColor(((color.red() - 128) * 0.65) + 128,
-                                ((color.green() - 128) * 0.65) + 128, 0, color.alpha()));
-        plotter->addPlot(QColor(((color.red() - 128) * 0.90) + 128, 0, 0, color.alpha()));
+    foreach (Plasma::SignalPlotter *plotter, plotters()) {
         plotter->setFontColor(theme->color(Plasma::Theme::HighlightColor));
         plotter->setHorizontalLinesColor(theme->color(Plasma::Theme::HighlightColor));
         plotter->setVerticalLinesColor(theme->color(Plasma::Theme::HighlightColor));
@@ -137,7 +148,19 @@ void SM::Net::dataUpdated(const QString& source,
        Plasma::SignalPlotter *plotter = plotters()[interface];
         if (plotter) {
             plotter->addSample(m_data[interface]);
-            //plotter->addSample(QList<double>() << 50 << 25);
+            if (mode() == SM::Applet::Panel) {
+                m_html[source] = QString("<tr><td>%1</td><td>in %2</td><td>out %3</td></tr>")
+                        .arg(plotter->title())
+                        .arg(m_data[interface][0])
+                        .arg(m_data[interface][1]);
+                QString html = "<table>";
+                foreach (const QString& s, m_html.keys()) {
+                    html += m_html[s];
+                }
+                html += "</table>";
+                Plasma::ToolTipContent data(title(), html);
+                Plasma::ToolTipManager::self()->setContent(this, data);
+            }
         }
         m_data[interface].clear();
     }
@@ -145,14 +168,13 @@ void SM::Net::dataUpdated(const QString& source,
 
 void SM::Net::createConfigurationInterface(KConfigDialog *parent)
 {
-   QWidget *widget = new QWidget();
-   ui.setupUi(widget);
-   m_model.clear();
-   m_model.setHorizontalHeaderLabels(QStringList() << i18n("Network interface"));
-   QStandardItem *parentItem = m_model.invisibleRootItem();
+    QWidget *widget = new QWidget();
+    ui.setupUi(widget);
+    m_model.clear();
+    m_model.setHorizontalHeaderLabels(QStringList() << i18n("Network Interface"));
+    QStandardItem *parentItem = m_model.invisibleRootItem();
 
-
-   foreach (const QString& interface, m_interfaces) {
+    foreach (const QString& interface, m_interfaces) {
         QString ifname = interface.split('/')[2];
         QStandardItem *item1 = new QStandardItem(ifname);
         item1->setEditable(false);
@@ -162,14 +184,29 @@ void SM::Net::createConfigurationInterface(KConfigDialog *parent)
             item1->setCheckState(Qt::Checked);
         }
         parentItem->appendRow(QList<QStandardItem *>() << item1);
-   }
-   ui.treeView->setModel(&m_model);
-   ui.treeView->resizeColumnToContents(0);
-   ui.intervalSpinBox->setValue(interval() / 1000);
+    }
+    ui.treeView->setModel(&m_model);
+    ui.treeView->resizeColumnToContents(0);
+    ui.intervalSpinBox->setValue(interval() / 1000);
+    updateSpinBoxSuffix(interval() / 1000);
+    connect(ui.intervalSpinBox, SIGNAL(valueChanged(int)), this, SLOT(updateSpinBoxSuffix(int)));
+    parent->addPage(widget, i18n("Interfaces"), "network-workgroup");
 
-   parent->addPage(widget, i18n("Interfaces"), "network-workgroup");
-   connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
-   connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+    widget = new QWidget();
+    uiAdv.setupUi(widget);
+    uiAdv.showTopBarCheckBox->setChecked(m_showTopBar);
+    uiAdv.showBackgroundCheckBox->setChecked(m_showBackground);
+    uiAdv.inColorCombo->setColor(m_inColor);
+    uiAdv.outColorCombo->setColor(m_outColor);
+    parent->addPage(widget, i18n("Advanced"), "preferences-other");
+
+    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
+    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+}
+
+void SM::Net::updateSpinBoxSuffix(int interval)                                                       
+{
+    ui.intervalSpinBox->setSuffix(i18np(" second", " seconds", interval));
 }
 
 void SM::Net::configAccepted()
@@ -192,6 +229,12 @@ void SM::Net::configAccepted()
     cg.writeEntry("interval", interval);
     interval *= 1000;
     setInterval(interval);
+    
+    cg.writeEntry("showTopBar", m_showTopBar = uiAdv.showTopBarCheckBox->isChecked());
+    cg.writeEntry("showBackground", m_showBackground = uiAdv.showBackgroundCheckBox->isChecked());
+    cg.writeEntry("inColor", m_inColor = uiAdv.inColorCombo->color());
+    cg.writeEntry("outColor", m_outColor = uiAdv.outColorCombo->color());
+
     emit configNeedsSaving();
     connectToEngine();
 }
