@@ -20,9 +20,6 @@
 
 // Own
 #include "taskgroupitem.h"
-#include "layoutwidget.h"
-#include "windowtaskitem.h"
-#include "tasksmenu.h"
 
 // Qt
 #include <QGraphicsSceneContextMenuEvent>
@@ -51,14 +48,16 @@
 #include <Plasma/ToolTipManager>
 #include <Plasma/Corona>
 #include <Plasma/Containment>
+#include <Plasma/Dialog>
 
 #include "tasks.h"
-#include "layoutwidget.h"
+#include "taskitemlayout.h"
+#include "windowtaskitem.h"
 
-TaskGroupItem::TaskGroupItem(QGraphicsWidget *parent, Tasks *applet, const bool showTooltip)
-    : AbstractTaskItem(parent, applet, showTooltip),
+TaskGroupItem::TaskGroupItem(QGraphicsWidget *parent, Tasks *applet)
+    : AbstractTaskItem(parent, applet),
       m_group(0),
-      m_expandedLayout(0),
+      m_tasksLayout(0),
       m_popupMenuTimer(0),
       m_lastActivated(-1),
       m_activeTaskIndex(0),
@@ -66,12 +65,23 @@ TaskGroupItem::TaskGroupItem(QGraphicsWidget *parent, Tasks *applet, const bool
       m_forceRows(false),
       m_splitPosition(0),
       m_parentSplitGroup(0),
-      m_childSplitGroup(0)
+      m_childSplitGroup(0),
+      m_offscreenWidget(0),
+      m_offscreenLayout(0),
+      m_collapsed(true),
+      m_mainLayout(0),
+      m_popupDialog(0),
+      m_popupLostFocus(false)
 {
     setAcceptDrops(true);
 }
 
 
+TaskGroupItem::~TaskGroupItem()
+{
+    delete m_tasksLayout;
+}
+
 bool TaskGroupItem::isSplit()
 {
     return m_childSplitGroup != 0;
@@ -110,16 +120,16 @@ TaskGroupItem * TaskGroupItem::splitGroup()
 void TaskGroupItem::setSplitIndex(int position)
 {
     //kDebug() << position;
-    Q_ASSERT(m_expandedLayout);
+    Q_ASSERT(m_tasksLayout);
     Q_ASSERT(m_parentSplitGroup);
 
     for (int i = position ; i < m_parentSplitGroup->group()->members().size() ; i++) {
         //kDebug() << "add item to childSplitGroup" << i;
         AbstractGroupableItem *item = m_parentSplitGroup->group()->members().at(i);
         if (!m_groupMembers.contains(item)) {
-            m_groupMembers.insert(item, m_parentSplitGroup->abstractItem(item));
+            m_groupMembers.insert(item, m_parentSplitGroup->abstractTaskItem(item));
         }
-        m_expandedLayout->addTaskItem(abstractItem(item));
+        m_tasksLayout->addTaskItem(abstractTaskItem(item));
     }
     m_splitPosition = position;
 }
@@ -127,26 +137,26 @@ void TaskGroupItem::setSplitIndex(int position)
 TaskGroupItem * TaskGroupItem::splitGroup(int newSplitPosition)
 {
     //kDebug() << "split position" << newSplitPosition;
-    Q_ASSERT(m_expandedLayout);
+    Q_ASSERT(m_tasksLayout);
 
     //remove all items which move to the splitgroup from the expandedLayout
     for (int i = newSplitPosition ; i < m_groupMembers.size() ; i++) {
         AbstractGroupableItem *item = group()->members().at(i);
-        m_expandedLayout->removeTaskItem(abstractItem(item));
+        m_tasksLayout->removeTaskItem(abstractTaskItem(item));
         //kDebug() << "remove from parentSplitGroup" << i;
     }
     //add items which arent in the splitgroup anymore and should be displayed again
     if (m_splitPosition) { //if 0 is the init value and shouldn't happen otherwise
         for (int i = m_splitPosition ; i < newSplitPosition ; i++) {
             AbstractGroupableItem *item = group()->members().at(i);
-            m_expandedLayout->addTaskItem(abstractItem(item));
+            m_tasksLayout->addTaskItem(abstractTaskItem(item));
             //kDebug() << "add Item to parentSplitGroup" << i;
         }
     }
 
     if (!m_childSplitGroup) {
         //kDebug() << "Normal scene " << scene();
-        m_childSplitGroup = new  TaskGroupItem(this, m_applet, true);
+        m_childSplitGroup = new  TaskGroupItem(this, m_applet);
         m_childSplitGroup->setSplitGroup(m_group);
     }
 
@@ -174,14 +184,27 @@ void TaskGroupItem::activate()
 
 void TaskGroupItem::close()
 {
-//  m_applet->removeGroupTask(m_group);
+    //kDebug();
+    //close the popup if the group is removed
+    if (m_popupDialog) {
+        m_popupDialog->hide();
+        m_popupDialog->deleteLater();
+        m_popupDialog = 0;
+    }
+
+    if (m_group) {
+        disconnect(m_group, 0, this, 0);
+    }
+
     m_group = 0;
 }
 
 
 void TaskGroupItem::updateTask(::TaskManager::TaskChanges changes)
 {
-    Q_ASSERT(m_group);
+    if (!m_group) {
+        return;
+    }
 
     bool needsUpdate = false;
     // task flags
@@ -221,7 +244,7 @@ void TaskGroupItem::updateTask(::TaskManager::TaskChanges changes)
         setText(m_group->name());
     }
 
-    if (m_showingTooltip &&
+    if (Plasma::ToolTipManager::self()->isVisible(this) &&
         (changes & TaskManager::IconChanged ||
         changes & TaskManager::NameChanged ||
         changes & TaskManager::DesktopChanged)) {
@@ -242,35 +265,53 @@ void TaskGroupItem::updateToolTip()
 
     Plasma::ToolTipContent data(m_group->name(),
                                 i18nc("Which virtual desktop a window is currently on", "On %1",
-                                    KWindowSystem::desktopName(m_group->desktop())));
+                                       KWindowSystem::desktopName(m_group->desktop())));
 //    data.image = m_group->icon().pixmap(QSize::small);
 //    data.windowToPreview = m_task->window();
 
+    int i = 0;
+    QList<WId> windows;
+
+    foreach (AbstractGroupableItem *item, m_group->members()) {
+        if (++i > 4) {
+            break;
+        }
+
+        TaskManager::TaskItem *taskItem = qobject_cast<TaskManager::TaskItem *>(item);
+
+        if (taskItem) {
+            windows.append(taskItem->task()->window());
+        }
+    }
+
+    data.setWindowsToPreview(windows);
+
     Plasma::ToolTipManager::self()->setContent(this, data);
 }
 
 
 void TaskGroupItem::reload()
 {
-    kDebug();
-    QList <AbstractItemPtr> itemsToRemove = m_groupMembers.keys();
+    QList <AbstractGroupableItem *> itemsToRemove = m_groupMembers.keys();
 
-    foreach (AbstractItemPtr item, group()->members()) {
+    foreach (AbstractGroupableItem *item, group()->members()) {
         if (!item) {
             kDebug() << "invalid Item";
             continue;
         }
+
         itemsToRemove.removeAll(item);
         itemAdded(item);
 
         if (item->isGroupItem()) {
-            TaskGroupItem *group = qobject_cast<TaskGroupItem *>(abstractItem(item));
+            TaskGroupItem *group = qobject_cast<TaskGroupItem *>(abstractTaskItem(item));
             if (group) {
                 group->reload();
             }
         }
     }
-    foreach (AbstractItemPtr item, itemsToRemove) { //remove unused items
+
+    foreach (AbstractGroupableItem * item, itemsToRemove) { //remove unused items
         if (!item) {
             kDebug() << "invalid Item";
             continue;
@@ -282,21 +323,32 @@ void TaskGroupItem::reload()
 void TaskGroupItem::setGroup(TaskManager::GroupPtr group)
 {
     //kDebug();
-    m_group = group;
-    m_abstractItem = qobject_cast<AbstractItemPtr>(group);
-    if (!m_abstractItem) {
-        kDebug() << "error";
+    if (m_group == group) {
+        kDebug() << "already have this group!";
+        return;
     }
 
-    connect(m_group, SIGNAL(itemRemoved(AbstractItemPtr)), this, SLOT(itemRemoved(AbstractItemPtr)));
-    connect(m_group, SIGNAL(itemAdded(AbstractItemPtr)), this, SLOT(itemAdded(AbstractItemPtr)));
+    if (m_group) {
+        disconnect(m_group, 0, this, 0);
+    }
+
+    m_group = group;
+    m_abstractItem = qobject_cast<AbstractGroupableItem *>(group);
+
+    if (m_abstractItem) {
+        connect(m_abstractItem, SIGNAL(destroyed(QObject*)), this, SLOT(clearAbstractItem()));
+    }
+
+    connect(m_group, SIGNAL(destroyed(QObject*)), this, SLOT(clearGroup()));
+    connect(m_group, SIGNAL(itemRemoved(AbstractGroupableItem *)), this, SLOT(itemRemoved(AbstractGroupableItem *)));
+    connect(m_group, SIGNAL(itemAdded(AbstractGroupableItem *)), this, SLOT(itemAdded(AbstractGroupableItem *)));
 
     //connect(m_group, SIGNAL(destroyed()), this, SLOT(close()));
 
     connect(m_group, SIGNAL(changed(::TaskManager::TaskChanges)),
             this, SLOT(updateTask(::TaskManager::TaskChanges)));
 
-    connect(m_group, SIGNAL(itemPositionChanged(AbstractItemPtr)), this, SLOT(itemPositionChanged(AbstractItemPtr)));
+    connect(m_group, SIGNAL(itemPositionChanged(AbstractGroupableItem *)), this, SLOT(itemPositionChanged(AbstractGroupableItem *)));
     connect(m_group, SIGNAL(groupEditRequest()), this, SLOT(editGroup()));
 
     //Add already existing items
@@ -311,6 +363,11 @@ TaskManager::GroupPtr TaskGroupItem::group() const
     return m_group;
 }
 
+void TaskGroupItem::clearGroup()
+{
+    m_group = 0;
+}
+
 void TaskGroupItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *e)
 {
     //kDebug();
@@ -327,7 +384,7 @@ void TaskGroupItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *e)
 
     QAction *a;
 
-    if (m_expandedLayout) {
+    if (!collapsed()) {
         a = new QAction(i18n("Collapse Group"), this);
         connect(a, SIGNAL(triggered()), this, SLOT(collapse()));
     } else {
@@ -341,32 +398,36 @@ void TaskGroupItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *e)
     TaskManager::BasicMenu menu(qobject_cast<QWidget*>(this), m_group, &m_applet->groupManager(), actionList);
     menu.adjustSize();
 
+    if (m_applet->formFactor() != Plasma::Vertical) {
+        menu.setMinimumWidth(size().width());
+    }
+
     Q_ASSERT(m_applet->containment());
     Q_ASSERT(m_applet->containment()->corona());
     menu.exec(m_applet->containment()->corona()->popupPosition(this, menu.size()));
 }
 
-QList<AbstractTaskItem*> TaskGroupItem::memberList() const
+QHash<AbstractGroupableItem *, AbstractTaskItem*> TaskGroupItem::members() const
 {
-    return m_groupMembers.values();
+    return m_groupMembers;
 }
 
-AbstractTaskItem *TaskGroupItem::createAbstractItem(TaskManager::AbstractItemPtr groupableItem)
+int TaskGroupItem::count() const
+{
+    return m_groupMembers.count();
+}
+
+AbstractTaskItem *TaskGroupItem::createAbstractItem(TaskManager::AbstractGroupableItem * groupableItem)
 {
     //kDebug() << "item to create" << groupableItem << endl;
     AbstractTaskItem *item = 0;
 
-    if (m_groupMembers.contains(groupableItem)) {
-        //kDebug() << "existing item found";
-        return m_groupMembers.value(groupableItem);
-    }
-
     if (groupableItem->isGroupItem()) {
-        TaskGroupItem *groupItem = new TaskGroupItem(this, m_applet, m_applet->showTooltip());
+        TaskGroupItem *groupItem = new TaskGroupItem(this, m_applet);
         groupItem->setGroup(static_cast<TaskManager::TaskGroup*>(groupableItem));
         item = groupItem;
     } else { //it's a window task
-        WindowTaskItem *windowItem = new WindowTaskItem(this, m_applet, m_applet->showTooltip());
+        WindowTaskItem *windowItem = new WindowTaskItem(this, m_applet);
         windowItem->setTask(static_cast<TaskManager::TaskItem*>(groupableItem));
         item = windowItem;
     }
@@ -376,12 +437,14 @@ AbstractTaskItem *TaskGroupItem::createAbstractItem(TaskManager::AbstractItemPtr
         return 0;
     }
 
+    if (m_collapsed) {
+        item->setPreferredOffscreenSize();
+    }
+
     return item;
 }
 
-
-
-void TaskGroupItem::itemAdded(TaskManager::AbstractItemPtr groupableItem)
+void TaskGroupItem::itemAdded(TaskManager::AbstractGroupableItem * groupableItem)
 {
     //kDebug();
     if (!m_applet) {
@@ -390,7 +453,13 @@ void TaskGroupItem::itemAdded(TaskManager::AbstractItemPtr groupableItem)
     }
 
     //returns the corresponding item or creates a new one
-    AbstractTaskItem *item = createAbstractItem(groupableItem);
+    bool isNew = false;
+    AbstractTaskItem *item = m_groupMembers.value(groupableItem);
+
+    if (!item) {
+        item = createAbstractItem(groupableItem);
+        isNew = true;
+    }
 
     if (!item) {
         kDebug() << "invalid Item";
@@ -400,55 +469,62 @@ void TaskGroupItem::itemAdded(TaskManager::AbstractItemPtr groupableItem)
     m_groupMembers[groupableItem] = item;
     item->setParentItem(this);
 
-    if (collapsed()) {
-        item->hide();
-        QRect rect = iconGeometry();
-        item->publishIconGeometry(rect);
-    } else if (isSplit()) {
+    if (isSplit()) {
         splitGroup(m_splitPosition);
         //emit changed();
         //m_childSplitGroup->reload();
-    } else {
-        m_expandedLayout->addTaskItem(item);
+    } else if (m_tasksLayout) { //add to layout either for popup or expanded group
+        m_tasksLayout->addTaskItem(item);
+    } else { //collapsed and no layout so far
+        item->hide();
+        QRect rect = iconGeometry();
+        item->publishIconGeometry(rect);
     }
 
     if (item->isActive()) {
         //kDebug() << "item is Active" ;
         m_activeTaskIndex = indexOf(item);
-    } else if (m_group->members().size() == 1) {
+    } else if (!m_group || m_group->members().size() == 1) {
         m_activeTaskIndex = 0;
     }
 
-    connect(item, SIGNAL(activated(AbstractTaskItem*)),
-            this, SLOT(updateActive(AbstractTaskItem*)));
+    if (isNew) {
+        connect(item, SIGNAL(activated(AbstractTaskItem*)),
+                this, SLOT(updateActive(AbstractTaskItem*)));
 
+        TaskGroupItem *group = qobject_cast<TaskGroupItem*>(item);
+        if (group) {
+            connect(item, SIGNAL(changed()), this, SLOT(relayoutItems()));
+        }
+    }
 }
 
-void TaskGroupItem::itemRemoved(TaskManager::AbstractItemPtr groupableItem)
+void TaskGroupItem::itemRemoved(TaskManager::AbstractGroupableItem * groupableItem)
 {
     //kDebug();
     if (!m_applet) {
         kDebug() << "No Applet";
         return;
     }
-    AbstractTaskItem *item = m_groupMembers.take(groupableItem);;
+
+    AbstractTaskItem *item = m_groupMembers.take(groupableItem);
 
     if (!item) {
         kDebug() << "Item not found";
         return;
     }
+    //kDebug() << item->text();
 
     disconnect(item, 0, 0, 0);
 
-    if (m_expandedLayout) {
-        m_expandedLayout->removeTaskItem(item);
+    if (m_tasksLayout) {
+        m_tasksLayout->removeTaskItem(item);
     }
 
     item->close();
     item->deleteLater();
 }
 
-
 bool TaskGroupItem::isWindowItem() const
 {
     return false;
@@ -456,13 +532,13 @@ bool TaskGroupItem::isWindowItem() const
 
 bool TaskGroupItem::isActive() const
 {
-    kDebug() << "Not Implemented";
+    //kDebug() << "Not Implemented";
     return false;
 }
 
 void TaskGroupItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 { //TODO add delay so we can still drag group items
-    if (event->buttons() & Qt::LeftButton) {
+    if ((event->buttons() & Qt::LeftButton) && !m_popupLostFocus) {
         if (m_applet->groupManager().sortingStrategy() == TaskManager::GroupManager::ManualSorting ||
             m_applet->groupManager().groupingStrategy() == TaskManager::GroupManager::ManualGrouping) {
             if (!m_popupMenuTimer) {
@@ -499,13 +575,85 @@ void TaskGroupItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 
 void TaskGroupItem::popupMenu()
 {
-    if (!m_expandedLayout) {
-        TaskManager::TasksMenu menu(qobject_cast<QWidget*>(this), m_group,  &m_applet->groupManager(), m_applet);
-        menu.adjustSize();
-        Q_ASSERT(m_applet->containment());
-        Q_ASSERT(m_applet->containment()->corona());
-        menu.exec(m_applet->containment()->corona()->popupPosition(this, menu.size()));
+    //kDebug();
+    if (!m_collapsed) {
+        return;
     }
+
+    if (!m_offscreenWidget) {
+        foreach (AbstractTaskItem *member, m_groupMembers) {
+            member->setPreferredOffscreenSize();
+        }
+
+        tasksLayout()->invalidate();
+        m_offscreenWidget = new QGraphicsWidget(this);
+        m_offscreenLayout = new QGraphicsLinearLayout(m_offscreenWidget);
+        m_offscreenLayout->setContentsMargins(0,0,0,0); //default are 4 on each side
+        m_offscreenLayout->addItem(tasksLayout());
+        m_offscreenWidget->setLayout(m_offscreenLayout);
+        m_applet->containment()->corona()->addOffscreenWidget(m_offscreenWidget);
+        m_offscreenLayout->activate();
+    }
+
+    if (!m_popupDialog) {
+        // Initialize popup dialog
+        m_popupDialog = new Plasma::Dialog();
+        KWindowSystem::setState(m_popupDialog->winId(), NET::SkipTaskbar| NET::SkipPager);
+        m_popupDialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
+        //TODO in the future it may be possible to use the Qt::Popup flag instead of the eventFilter, but for now the focus works better with the eventFilter
+        m_popupDialog->installEventFilter(this);
+
+        int left, top, right, bottom;
+        m_popupDialog->getContentsMargins(&left, &top, &right, &bottom);
+        m_offscreenWidget->setMinimumWidth(size().width() - left - right);
+        m_popupDialog->setGraphicsWidget(m_offscreenWidget);
+    }
+
+    if (m_popupDialog->isVisible()) {
+        m_popupDialog->clearFocus();
+        if (m_applet->location() != Plasma::Floating) {
+            m_popupDialog->animatedHide(Plasma::locationToInverseDirection(m_applet->location()));
+        } else {
+            m_popupDialog->hide();
+        }
+    } else {
+        m_tasksLayout->setOrientation(Plasma::Vertical);
+        m_tasksLayout->setMaximumRows(1);
+        m_offscreenWidget->adjustSize();
+        if (m_applet->containment() && m_applet->containment()->corona()) {
+            m_popupDialog->move(m_applet->containment()->corona()->popupPosition(this, m_popupDialog->size()));
+        }
+        KWindowSystem::setState(m_popupDialog->winId(), NET::SkipTaskbar| NET::SkipPager);
+        if (m_applet->location() != Plasma::Floating) {
+            m_popupDialog->animatedShow(Plasma::locationToDirection(m_applet->location()));
+        } else {
+            m_popupDialog->show();
+        }
+        m_popupDialog->raise();
+        KWindowSystem::activateWindow(m_popupDialog->winId());
+        //kDebug() << m_popupDialog->size() << m_tasksLayout->size();
+    }
+}
+
+bool TaskGroupItem::eventFilter(QObject *watched, QEvent *event)
+{
+    if (watched == m_popupDialog && event->type() == QEvent::WindowDeactivate) {
+        Q_ASSERT(m_popupDialog);
+        m_popupLostFocus = true; //avoid opening it again when clicking on the group
+        if (m_applet->location() != Plasma::Floating) {
+            m_popupDialog->animatedHide(Plasma::locationToInverseDirection(m_applet->location()));
+        } else {
+            m_popupDialog->hide();
+        }
+        QTimer::singleShot(100, this, SLOT(clearPopupLostFocus()));
+    }
+
+    return QGraphicsWidget::eventFilter(watched, event);
+}
+
+void TaskGroupItem::clearPopupLostFocus()
+{
+    m_popupLostFocus = false;
 }
 
 void TaskGroupItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
@@ -516,43 +664,96 @@ void TaskGroupItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
     //kDebug();
     if (m_popupMenuTimer) {
+        //kDebug() << "popupTimer stop";
         m_popupMenuTimer->stop();
     } //Wait a bit before starting drag
     AbstractTaskItem::mouseMoveEvent(event);
 }
 
+void TaskGroupItem::resizeEvent(QGraphicsSceneResizeEvent *event)
+{
+    Q_UNUSED(event)
+
+    if (m_offscreenWidget && m_popupDialog) {
+        int left, top, right, bottom;
+        m_popupDialog->getContentsMargins(&left, &top, &right, &bottom);
+        m_offscreenWidget->setMinimumWidth(size().width() - left - right);
+    }
+}
+
 void TaskGroupItem::expand()
 {
-    if (m_expandedLayout) {
+    if (!collapsed()) {
         //kDebug() << "already expanded";
         return;
     }
 
-    //kDebug();
-    m_expandedLayout = new LayoutWidget(this, m_applet);
-    m_expandedLayout->setMaximumRows(m_maximumRows);
-    m_expandedLayout->setForceRows(m_forceRows);
+    if (m_popupDialog) {
+        m_popupDialog->hide();
+    }
 
-    //setLayout(m_expandedLayout);
+    if (m_offscreenLayout) {
+        m_offscreenLayout->removeItem(tasksLayout());
+    }
 
-    connect(m_applet, SIGNAL(constraintsChanged(Plasma::Constraints)), m_expandedLayout, SLOT(constraintsChanged(Plasma::Constraints)));
-    connect(m_expandedLayout, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updatePreferredSize()));
-    updatePreferredSize();
+    if (!m_mainLayout) { //this layout is needed since we can't take a layout directly from a widget without destroying it
+        m_mainLayout = new QGraphicsLinearLayout(this);
+        m_mainLayout->setContentsMargins(0,0,0,0); //default are 4 on each side
+        setLayout(m_mainLayout);
+    }
+
+    //set it back from the popup settings (always vertical and 1 row)
+    tasksLayout()->setOrientation(m_applet->formFactor());
+    tasksLayout()->setMaximumRows(m_maximumRows);
+
+    m_mainLayout->addItem(tasksLayout());
+
+    disconnect(m_applet, SIGNAL(constraintsChanged(Plasma::Constraints)), this, SLOT(constraintsChanged(Plasma::Constraints)));
+    connect(m_applet, SIGNAL(constraintsChanged(Plasma::Constraints)), this, SLOT(constraintsChanged(Plasma::Constraints)));
+    //connect(m_tasksLayout, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updatePreferredSize()));
+    m_collapsed = false;
+    tasksLayout()->layoutItems();
+    //kDebug() << tasksLayout()->preferredSize() << preferredSize() << m_groupMembers.count();
     emit changed();
     checkSettings();
     //kDebug() << "expanded";
-
 }
 
-LayoutWidget *TaskGroupItem::layoutWidget()
+void TaskGroupItem::constraintsChanged(Plasma::Constraints constraints)
 {
-    return m_expandedLayout;
+    //kDebug();
+    if (constraints & Plasma::SizeConstraint && m_tasksLayout) {
+        m_tasksLayout->layoutItems();
+    }
+
+    if (constraints & Plasma::FormFactorConstraint && m_tasksLayout) {
+        m_tasksLayout->setOrientation(m_applet->formFactor());
+    }
+}
+
+void TaskGroupItem::relayoutItems()
+{
+    if (m_tasksLayout) {
+        m_tasksLayout->layoutItems();
+    }
+}
+
+TaskItemLayout *TaskGroupItem::tasksLayout()
+{
+    if (!m_tasksLayout) {
+        m_tasksLayout = new TaskItemLayout(this, m_applet);
+        m_tasksLayout->setMaximumRows(m_maximumRows);
+        m_tasksLayout->setForceRows(m_forceRows);
+        m_tasksLayout->setOrientation(m_applet->formFactor());
+    }
+
+    return m_tasksLayout;
 }
 
 void TaskGroupItem::collapse()
 {
     //kDebug() << (int)this;
-    if (!m_expandedLayout) {
+    if (collapsed()) {
         //kDebug() << "already collapsed";
         return;
     }
@@ -563,13 +764,20 @@ void TaskGroupItem::collapse()
 
     //kDebug();
     unsplitGroup();
-    foreach (AbstractTaskItem *member, m_groupMembers) {
-        m_expandedLayout->removeTaskItem(member);
+
+    m_mainLayout->removeItem(tasksLayout());
+    if (m_offscreenLayout) {
+        m_offscreenLayout->addItem(tasksLayout());
+    } else {
+        foreach (AbstractTaskItem *member, m_groupMembers) {
+            scene()->removeItem(member);
+        }
     }
-    setLayout(0);
+
     //kDebug();
-    //delete m_expandedLayout;
-    m_expandedLayout = 0;
+    //delete m_tasksLayout;
+    disconnect(m_applet, SIGNAL(constraintsChanged(Plasma::Constraints)), this, SLOT(constraintsChanged(Plasma::Constraints)));
+    m_collapsed = true;
     updatePreferredSize();
     //kDebug();
     emit changed();
@@ -578,19 +786,28 @@ void TaskGroupItem::collapse()
 
 bool TaskGroupItem::collapsed() const
 {
-    return m_expandedLayout == 0;
+    return m_collapsed;
 }
 
 void TaskGroupItem::updatePreferredSize()
 {
-    if (layout()) {
-        setPreferredSize(layout()->preferredSize());
-        layout()->invalidate();
-        //kDebug() << "expanded group" << layout()->preferredSize();
-    } else {
+    if (m_collapsed) {
+        foreach (AbstractTaskItem *taskItem, m_groupMembers) {
+            taskItem->setPreferredOffscreenSize();
+        }
+
         //FIXME: copypaste from abstracttaskitem: to be done better with proper sizeHint()
         setPreferredSize(basicPreferredSize());
+    } else {
+        foreach (AbstractTaskItem *taskItem, m_groupMembers) {
+            taskItem->setPreferredOnscreenSize();
+        }
+
+        layout()->invalidate();
+        setPreferredSize(layout()->preferredSize());
+        //kDebug() << "expanded group" << layout()->preferredSize();
     }
+
     //kDebug() << preferredSize();
     emit sizeHintChanged(Qt::PreferredSize);
 }
@@ -601,18 +818,18 @@ AbstractTaskItem *TaskGroupItem::directMember(AbstractTaskItem *item)
 {
     Q_ASSERT(item);
     Q_ASSERT(m_group);
-    TaskManager::AbstractItemPtr directMember = m_group->directMember(item->abstractItem());
+    TaskManager::AbstractGroupableItem * directMember = m_group->directMember(item->abstractItem());
     if (!directMember) {
         kDebug() << "Error" << item->abstractItem();
     }
-    return abstractItem(directMember);
+    return abstractTaskItem(directMember);
 }
 
 void TaskGroupItem::paint(QPainter *painter,
                             const QStyleOptionGraphicsItem *option,
                             QWidget *widget)
 {
-    if (!m_expandedLayout) {
+    if (collapsed()) {
         AbstractTaskItem::paint(painter,option,widget);
     }/* else {
         if (m_group) {
@@ -634,8 +851,8 @@ void TaskGroupItem::editGroup()
     Q_ASSERT(m_applet);
     if (m_applet->groupManager().taskGrouper()->editableGroupProperties() & TaskManager::AbstractGroupingStrategy::Name) {
         bool ok;
-        QString text = QInputDialog::getText(qobject_cast<QWidget*>(this), tr("Edit Group"),
-                                            tr("New Group Name: "), QLineEdit::Normal,
+        QString text = QInputDialog::getText(qobject_cast<QWidget*>(this), i18n("Edit Group"),
+                                            i18n("New Group Name: "), QLineEdit::Normal,
                                             m_group->name(), &ok);
         if (ok && !text.isEmpty()) {
             m_group->setName(text);
@@ -644,33 +861,36 @@ void TaskGroupItem::editGroup()
 }
 
 
-void  TaskGroupItem::itemPositionChanged(AbstractItemPtr item)
+void  TaskGroupItem::itemPositionChanged(AbstractGroupableItem * item)
 {
     //kDebug();
-    if (!m_expandedLayout) {
+    if (!m_tasksLayout) {
         return;
     }
 
     Q_ASSERT(item);
 
     if (item->isGroupItem()) {
-        TaskGroupItem *groupItem = static_cast<TaskGroupItem*>(abstractItem(item));
+        TaskGroupItem *groupItem = static_cast<TaskGroupItem*>(abstractTaskItem(item));
         if (groupItem) {
             groupItem->unsplitGroup();
         }
     }
 
-    AbstractTaskItem *taskItem = abstractItem(item);
+    AbstractTaskItem *taskItem = abstractTaskItem(item);
 
-    m_expandedLayout->removeTaskItem(taskItem);
-    m_expandedLayout->insert(m_group->members().indexOf(item), taskItem);
+    m_tasksLayout->removeTaskItem(taskItem);
+
+    if (m_group) {
+        m_tasksLayout->insert(m_group->members().indexOf(item), taskItem);
+    }
 }
 
 
 void TaskGroupItem::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 {
     //kDebug()<<"Drag enter";
-    if (!m_expandedLayout &&
+    if (collapsed() &&
         (event->mimeData()->hasFormat(TaskManager::Task::mimetype()) ||
          event->mimeData()->hasFormat(TaskManager::Task::groupMimetype()))) {
         event->acceptProposedAction();
@@ -689,7 +909,7 @@ void TaskGroupItem::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 
 AbstractTaskItem *TaskGroupItem::taskItemForWId(WId id)
 {
-    QHashIterator<AbstractItemPtr, AbstractTaskItem*> it(m_groupMembers);
+    QHashIterator<AbstractGroupableItem *, AbstractTaskItem*> it(m_groupMembers);
 
     while (it.hasNext()) {
         it.next();
@@ -714,7 +934,7 @@ AbstractTaskItem *TaskGroupItem::taskItemForWId(WId id)
 
 void TaskGroupItem::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
-    //kDebug() << "LayoutWidget dropEvent";
+    //kDebug() << "TaskItemLayout dropEvent";
     if (event->mimeData()->hasFormat(TaskManager::Task::mimetype()) ||
         event->mimeData()->hasFormat(TaskManager::Task::groupMimetype())) {
         bool ok;
@@ -734,7 +954,7 @@ void TaskGroupItem::dropEvent(QGraphicsSceneDragDropEvent *event)
             handleDroppedId(id, targetTask, event);
         }
 
-        //kDebug() << "LayoutWidget dropEvent done";
+        //kDebug() << "TaskItemLayout dropEvent done";
         event->acceptProposedAction();
     } else {
         event->ignore();
@@ -770,8 +990,8 @@ void TaskGroupItem::handleDroppedId(WId id, AbstractTaskItem *targetTask, QGraph
             m_applet->groupManager().manualGroupingRequest(taskItem->abstractItem(), m_group);
         } else if (targetTask->isWindowItem() && (group == m_group)) { //Both Items in same group
             //Group Items together
-            int targetIndex = m_group->members().indexOf(targetTask->abstractItem());
-            int sourceIndex = m_group->members().indexOf(taskItem->abstractItem());
+            int targetIndex = m_group ? m_group->members().indexOf(targetTask->abstractItem()) : 0;
+            int sourceIndex = m_group ? m_group->members().indexOf(taskItem->abstractItem()) : 0;
             TaskManager::ItemList members;
             members.append(targetTask->abstractItem());
             members.append(taskItem->abstractItem());
@@ -798,8 +1018,8 @@ void TaskGroupItem::handleDroppedId(WId id, AbstractTaskItem *targetTask, QGraph
         if (group == m_group) { //same group
             //kDebug() << "Drag within group";
             layoutTaskItem(taskItem, event->pos());
-        } else { //task item was dragged outside of group -> group move
-            AbstractTaskItem *directMember = abstractItem(m_group->directMember(group));
+        } else if (m_group) { //task item was dragged outside of group -> group move
+            AbstractTaskItem *directMember = abstractTaskItem(m_group->directMember(group));
             if (directMember) {
                 layoutTaskItem(directMember, event->pos()); //we need to get the group right under the receiver group
             }
@@ -809,11 +1029,11 @@ void TaskGroupItem::handleDroppedId(WId id, AbstractTaskItem *targetTask, QGraph
 
 void TaskGroupItem::layoutTaskItem(AbstractTaskItem* item, const QPointF &pos)
 {
-    if (!m_expandedLayout) {
+    if (!m_tasksLayout) {
         return;
     }
 
-    int insertIndex = m_expandedLayout->insertionIndexAt(pos);
+    int insertIndex = m_tasksLayout->insertionIndexAt(pos);
     // kDebug() << "Item inserting at: " << insertIndex << "of: " << numberOfItems();
     if (insertIndex == -1) {
         m_applet->groupManager().manualSortingRequest(item->abstractItem(), -1);
@@ -830,7 +1050,7 @@ void TaskGroupItem::layoutTaskItem(AbstractTaskItem* item, const QPointF &pos)
 
 void TaskGroupItem::updateActive(AbstractTaskItem *task)
 {
-    if (!m_expandedLayout) {
+    if (!m_tasksLayout) {
         return;
     }
 
@@ -846,13 +1066,18 @@ int TaskGroupItem::indexOf(AbstractTaskItem *task)
 
     int index = 0;
 
-    foreach (AbstractGroupableItem *item, group()->members()) {
-        AbstractTaskItem *taskItem = abstractItem(item);
+    foreach (AbstractGroupableItem *item, m_group->members()) {
+        AbstractTaskItem *taskItem = abstractTaskItem(item);
         if (taskItem) {
             if (task == taskItem) {
                 TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
                 if (groupItem) {
-                    return index + groupItem->indexOf(groupItem->activeSubTask());
+                    int subIndex = groupItem->indexOf(groupItem->activeSubTask());
+                    if (subIndex == -1) {
+                        index += groupItem->count();
+                    } else {
+                        return index + subIndex;
+                    }
                 }
 
                 return index;
@@ -862,7 +1087,7 @@ int TaskGroupItem::indexOf(AbstractTaskItem *task)
             if (groupItem) {
                 int subIndex = groupItem->indexOf(task);
                 if(subIndex == -1) {
-                    index += groupItem->memberList().count();
+                    index += groupItem->count();
                 } else {
                     return index+subIndex;
                 }
@@ -876,8 +1101,12 @@ int TaskGroupItem::indexOf(AbstractTaskItem *task)
 
 AbstractTaskItem * TaskGroupItem::activeSubTask()
 {
-    foreach(AbstractGroupableItem *item, group()->members()) {
-        AbstractTaskItem *taskItem = abstractItem(item);
+    if (!m_group) {
+        return 0;
+    }
+
+    foreach (AbstractGroupableItem *item, m_group->members()) {
+        AbstractTaskItem *taskItem = abstractTaskItem(item);
         if (taskItem) {
             if(taskItem->isActive()) {
                 TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
@@ -888,7 +1117,8 @@ AbstractTaskItem * TaskGroupItem::activeSubTask()
             }
         }
     }
-    return NULL;
+
+    return 0;
 }
 
 int TaskGroupItem::totalSubTasks()
@@ -896,11 +1126,11 @@ int TaskGroupItem::totalSubTasks()
     int count = 0;
 
     foreach(AbstractGroupableItem *item, group()->members()) {
-        AbstractTaskItem *taskItem = abstractItem(item);
+        AbstractTaskItem *taskItem = abstractTaskItem(item);
         if (taskItem) {
             TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
             if (groupItem) {
-                count += groupItem->memberList().count();
+                count += groupItem->count();
             } else {
                 count++;
             }
@@ -913,14 +1143,14 @@ int TaskGroupItem::totalSubTasks()
 AbstractTaskItem * TaskGroupItem::selectSubTask(int index)
 {
     foreach(AbstractGroupableItem *item, group()->members()) {
-        AbstractTaskItem *taskItem = abstractItem(item);
+        AbstractTaskItem *taskItem = abstractTaskItem(item);
         if (taskItem) {
             TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
             if (groupItem) {
-                if (index < groupItem->memberList().count()) {
-                   return groupItem->abstractItem(groupItem->group()->members().at(index));
+                if (index < groupItem->count()) {
+                   return groupItem->abstractTaskItem(groupItem->group()->members().at(index));
                 } else {
-                   index -= groupItem->memberList().count();
+                   index -= groupItem->count();
                 }
             } else if (index == 0) {
                 return taskItem;
@@ -944,10 +1174,10 @@ void TaskGroupItem::wheelEvent(QGraphicsSceneWheelEvent *event)
     if (event->delta() < 0) {
         m_activeTaskIndex++;
         if (m_activeTaskIndex >= subTasks) {
-            m_activeTaskIndex = 0;
+            m_activeTaskIndex = 0; // last item is spacer
         }
-        //mouse wheel up
     } else {
+        //mouse wheel up
         m_activeTaskIndex--;
         if (m_activeTaskIndex < 0) {
             m_activeTaskIndex = subTasks - 1; //last item is a spacer
@@ -957,6 +1187,7 @@ void TaskGroupItem::wheelEvent(QGraphicsSceneWheelEvent *event)
     //kDebug() << "Wheel event m_activeTaskIndex: " << m_activeTaskIndex << " of " << subTasks;
     AbstractTaskItem *taskItem = selectSubTask(m_activeTaskIndex);
     if (taskItem) {
+        stopWindowHoverEffect();
         taskItem->activate();
     }
 }
@@ -969,8 +1200,8 @@ int TaskGroupItem::maxRows()
 void TaskGroupItem::setMaxRows(int rows)
 {
     m_maximumRows = rows;
-    if (m_expandedLayout) {
-        m_expandedLayout->setMaximumRows(m_maximumRows);
+    if (m_tasksLayout) {
+        m_tasksLayout->setMaximumRows(m_maximumRows);
     }
 }
 
@@ -982,27 +1213,35 @@ bool TaskGroupItem::forceRows()
 void TaskGroupItem::setForceRows(bool forceRows)
 {
     m_forceRows = forceRows;
-    if (m_expandedLayout) {
-        m_expandedLayout->setForceRows(m_forceRows);
+    if (m_tasksLayout) {
+        m_tasksLayout->setForceRows(m_forceRows);
     }
 }
 
 int TaskGroupItem::optimumCapacity()
 {
-    if (m_expandedLayout) {
-        return m_expandedLayout->maximumRows() * m_expandedLayout->preferredColumns();
+    if (m_tasksLayout) {
+        return m_tasksLayout->maximumRows() * m_tasksLayout->preferredColumns();
     }
 
     return 1;
 }
 
-AbstractTaskItem* TaskGroupItem::abstractItem(AbstractItemPtr item)
+AbstractTaskItem* TaskGroupItem::abstractTaskItem(AbstractGroupableItem * item)
 {
+    AbstractTaskItem *abstractTaskItem = 0; 
     if (m_groupMembers.contains(item)) {
-        return m_groupMembers.value(item);
+        abstractTaskItem = m_groupMembers.value(item);
+    } else {
+        foreach(AbstractTaskItem *taskItem, m_groupMembers) {
+            TaskGroupItem *group = qobject_cast<TaskGroupItem*>(taskItem);
+            if (group) {
+                abstractTaskItem = group->abstractTaskItem(item);
+            }
+        }
     }
     //kDebug() << "item not found";
-    return 0;
+    return abstractTaskItem;
 }
 
 void TaskGroupItem::setAdditionalMimeData(QMimeData* mimeData)
@@ -1015,7 +1254,7 @@ void TaskGroupItem::setAdditionalMimeData(QMimeData* mimeData)
 void TaskGroupItem::publishIconGeometry() const
 {
     // only do this if we are a collapsed group, with a GroupPtr and members
-    if (m_expandedLayout || !m_group || m_groupMembers.isEmpty()) {
+    if (!collapsed() || !m_group || m_groupMembers.isEmpty()) {
         return;
     }
 
@@ -1039,5 +1278,10 @@ void TaskGroupItem::publishIconGeometry(const QRect &rect) const
     }
 }
 
+QWidget *TaskGroupItem::popupDialog() const
+{
+    return m_popupDialog;
+}
+
 #include "taskgroupitem.moc"
 
