@@ -28,7 +28,9 @@
 #include <QFileInfo>
 #include <QGraphicsScene>
 #include <QGraphicsView>
+#include <QMenu>
 #include <QPainter>
+#include <QSignalMapper>
 #include <QTimeLine>
 
 #include <KAuthorized>
@@ -58,8 +60,7 @@ using namespace Plasma;
 
 DefaultDesktop::DefaultDesktop(QObject *parent, const QVariantList &args)
     : Containment(parent, args),
-      m_lockDesktopAction(0),
-      m_appletBrowserAction(0),
+      m_addPanelsMenu(0),
       m_addPanelAction(0),
       m_runCommandAction(0),
       m_lockScreenAction(0),
@@ -68,51 +69,80 @@ DefaultDesktop::DefaultDesktop(QObject *parent, const QVariantList &args)
 {
     qRegisterMetaType<QImage>("QImage");
     qRegisterMetaType<QPersistentModelIndex>("QPersistentModelIndex");
-    connect(this, SIGNAL(appletAdded(Plasma::Applet *, const QPointF &)),
-            this, SLOT(onAppletAdded(Plasma::Applet *, const QPointF &)));
-    connect(this, SIGNAL(appletRemoved(Plasma::Applet *)),
-            this, SLOT(onAppletRemoved(Plasma::Applet *)));
 
     m_layout = new DesktopLayout;
-    m_layout->setAutoWorkingArea(false);
     m_layout->setAlignment(Qt::AlignTop|Qt::AlignLeft);
     m_layout->setPlacementSpacing(20);
-    m_layout->setScreenSpacing(5);
+    m_layout->setScreenSpacing(0);
     m_layout->setShiftingSpacing(0);
     m_layout->setTemporaryPlacement(true);
     m_layout->setVisibilityTolerance(0.5);
-    setLayout(m_layout);
 
     resize(800, 600);
+
+    setHasConfigurationInterface(true);
     //kDebug() << "!!! loading desktop";
 }
 
+DefaultDesktop::~DefaultDesktop()
+{
+    delete m_addPanelsMenu;
+}
+
 void DefaultDesktop::constraintsEvent(Plasma::Constraints constraints)
 {
-    if (constraints & Plasma::ImmutableConstraint && m_appletBrowserAction) {
-        // we need to update the menu items that have already been created
-        bool locked = immutability() != Mutable;
-        m_addPanelAction->setVisible(!locked);
+    if (constraints & Plasma::ImmutableConstraint) {
+        if (m_addPanelAction) {
+            // we need to update the menu items that have already been created
+            bool locked = immutability() != Mutable;
+            m_addPanelAction->setVisible(!locked);
+        }
     }
 
+    if (constraints & Plasma::SizeConstraint) {
+        refreshWorkingArea();
+    }
+
+    //FIXME: are refres on working are change and refresh on resize mutually exclusive?
     if (constraints & Plasma::StartupCompletedConstraint) {
         connect(corona(), SIGNAL(availableScreenRegionChanged()),
                 this, SLOT(refreshWorkingArea()));
         refreshWorkingArea();
+
+        connect(this, SIGNAL(appletAdded(Plasma::Applet *, const QPointF &)),
+                this, SLOT(onAppletAdded(Plasma::Applet *, const QPointF &)));
+        connect(this, SIGNAL(appletRemoved(Plasma::Applet *)),
+                this, SLOT(onAppletRemoved(Plasma::Applet *)));
+
+        foreach (Applet *applet, applets()) {
+            m_layout->addItem(applet, true, false);
+            connect(applet, SIGNAL(appletTransformedByUser()), this, SLOT(onAppletTransformedByUser()));
+            connect(applet, SIGNAL(appletTransformedItself()), this, SLOT(onAppletTransformedItself()));
+        }
+
+        m_layout->adjustPhysicalPositions();
     }
 }
 
 void DefaultDesktop::addPanel()
+{
+    KPluginInfo::List panelPlugins = listContainmentsOfType("panel");
+
+    if (!panelPlugins.isEmpty()) {
+        addPanel(panelPlugins.first().pluginName());
+    }
+}
+
+void DefaultDesktop::addPanel(const QString &plugin)
 {
     if (corona()) {
-        // make a panel at the top
-        Containment* panel = corona()->addContainment("panel");
+        Containment* panel = corona()->addContainment(plugin);
         panel->showConfigurationInterface();
 
         panel->setScreen(screen());
 
         QList<Plasma::Location> freeEdges = corona()->freeEdges(screen());
-        kDebug() << freeEdges;
+        //kDebug() << freeEdges;
         Plasma::Location destination;
         if (freeEdges.contains(Plasma::TopEdge)) {
             destination = Plasma::TopEdge;
@@ -131,12 +161,34 @@ void DefaultDesktop::addPanel()
         panel->updateConstraints(Plasma::StartupCompletedConstraint);
         panel->flushPendingConstraintsEvents();
 
-        if (destination == Plasma::LeftEdge || destination == Plasma::RightEdge) {
-            panel->setMinimumSize(10, 35);
-            panel->setMaximumSize(35, corona()->screenGeometry(screen()).height());
-            panel->resize(QSize(35, corona()->screenGeometry(screen()).height()));
+        const QRect screenGeom = corona()->screenGeometry(screen());
+        const QRegion availGeom = corona()->availableScreenRegion(screen());
+        int minH = 10;
+        int minW = 10;
+        int w = 35;
+        int h = 35;
+
+        if (destination == Plasma::LeftEdge) {
+            QRect r = availGeom.intersected(QRect(0, 0, w, screenGeom.height())).boundingRect();
+            h = r.height();
+            minW = 35;
+        } else if (destination == Plasma::RightEdge) {
+            QRect r = availGeom.intersected(QRect(screenGeom.width() - w, 0, w, screenGeom.height())).boundingRect();
+            h = r.height();
+            minW = 35;
+        } else if (destination == Plasma::TopEdge) {
+            QRect r = availGeom.intersected(QRect(0, 0, screenGeom.width(), h)).boundingRect();
+            w = r.width();
+            minH = 35;
+        } else if (destination == Plasma::BottomEdge) {
+            QRect r = availGeom.intersected(QRect(0, screenGeom.height() - h, screenGeom.width(), h)).boundingRect();
+            w = r.width();
+            minH = 35;
         }
 
+        panel->setMinimumSize(minW, minH);
+        panel->setMaximumSize(w, h);
+        panel->resize(w, h);
     }
 }
 
@@ -171,21 +223,39 @@ void DefaultDesktop::lockScreen()
 
 QList<QAction*> DefaultDesktop::contextualActions()
 {
-    //TODO: should we offer "Switch User" here?
+    if (!m_runCommandAction) {
+        KPluginInfo::List panelPlugins = listContainmentsOfType("panel");
 
-    if (!m_appletBrowserAction) {
-        m_appletBrowserAction = action("add widgets");
+        if (panelPlugins.size() == 1) {
+            m_addPanelAction = new QAction(i18n("Add Panel"), this);
+            connect(m_addPanelAction, SIGNAL(triggered(bool)), this, SLOT(addPanel()));
+        } else if (!panelPlugins.isEmpty()) {
+            m_addPanelsMenu = new QMenu();
+            m_addPanelAction = m_addPanelsMenu->menuAction();
+            m_addPanelAction->setText(i18n("Add Panel"));
 
-        m_addPanelAction = new QAction(i18n("Add Panel"), this);
-        connect(m_addPanelAction, SIGNAL(triggered(bool)), this, SLOT(addPanel()));
-        m_addPanelAction->setIcon(KIcon("list-add"));
+            QSignalMapper *mapper = new QSignalMapper(this);
+            connect(mapper, SIGNAL(mapped(QString)), this, SLOT(addPanel(QString)));
+
+            foreach (const KPluginInfo &plugin, panelPlugins) {
+                QAction *action = new QAction(plugin.name(), this);
+                if (!plugin.icon().isEmpty()) {
+                    action->setIcon(KIcon(plugin.icon()));
+                }
+
+                mapper->setMapping(action, plugin.pluginName());
+                connect(action, SIGNAL(triggered(bool)), mapper, SLOT(map()));
+                m_addPanelsMenu->addAction(action);
+            }
+        }
+
+        if (m_addPanelAction) {
+            m_addPanelAction->setIcon(KIcon("list-add"));
+        }
 
         m_runCommandAction = new QAction(i18n("Run Command..."), this);
-        connect(m_runCommandAction, SIGNAL(triggered(bool)), this, SLOT(runCommand()));
         m_runCommandAction->setIcon(KIcon("system-run"));
-
-        m_setupDesktopAction = action("activity settings");
-        m_lockDesktopAction = action("lock widgets");
+        connect(m_runCommandAction, SIGNAL(triggered(bool)), this, SLOT(runCommand()));
 
         m_lockScreenAction = new QAction(i18n("Lock Screen"), this);
         m_lockScreenAction->setIcon(KIcon("system-lock-screen"));
@@ -206,14 +276,29 @@ QList<QAction*> DefaultDesktop::contextualActions()
         actions.append(m_runCommandAction);
     }
 
-    actions.append(m_appletBrowserAction);
-    actions.append(m_addPanelAction);
-    actions.append(m_setupDesktopAction);
-    if (screen() == -1) {
-        actions.append(action("remove"));
+    QAction *appletBrowserAction = action("add widgets");
+    if (appletBrowserAction) {
+        actions.append(appletBrowserAction);
     }
 
-    actions.append(m_lockDesktopAction);
+    if (m_addPanelAction) {
+        actions.append(m_addPanelAction);
+    }
+
+    QAction *setupDesktopAction = action("activity settings");
+    if (setupDesktopAction) {
+        actions.append(setupDesktopAction);
+    }
+
+    QAction *removeAction = action("remove");
+    if (screen() == -1 && removeAction) {
+        actions.append(removeAction);
+    }
+
+    QAction *lockDesktopAction = action("lock widgets");
+    if (lockDesktopAction) {
+        actions.append(lockDesktopAction);
+    }
 
     actions.append(m_separator);
 
@@ -234,7 +319,7 @@ void DefaultDesktop::logout()
         return;
     }
 #ifndef Q_WS_WIN
-    KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmDefault,
+    KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmYes,
                                 KWorkSpace::ShutdownTypeDefault,
                                 KWorkSpace::ShutdownModeDefault);
 #endif
@@ -244,19 +329,16 @@ void DefaultDesktop::onAppletAdded(Plasma::Applet *applet, const QPointF &pos)
 {
     if (dropping || pos != QPointF(-1,-1) || applet->geometry().topLeft() != QPointF(0,0)) {
         // add item to the layout using the current position
-        m_layout->addItem(applet, true, applet->geometry());
+        m_layout->addItem(applet, true, false);
     } else {
-        /*
-            There seems to be no uniform way to get the applet's preferred size.
-            Regular applets properly set their current size when created, but report useless size hints.
-            Proxy widget applets don't set their size properly, but report valid size hints. However, they
-            will obtain proper size if we just re-set the geometry to the current value.
-        */
-        applet->setGeometry(applet->geometry());
-        m_layout->addItem(applet, true, applet->geometry().size());
+        // auto-position
+        m_layout->addItem(applet, true, true);
     }
 
-    connect(applet, SIGNAL(geometryChanged()), this, SLOT(onAppletGeometryChanged()));
+    m_layout->adjustPhysicalPositions();
+
+    connect(applet, SIGNAL(appletTransformedByUser()), this, SLOT(onAppletTransformedByUser()));
+    connect(applet, SIGNAL(appletTransformedItself()), this, SLOT(onAppletTransformedItself()));
 }
 
 void DefaultDesktop::onAppletRemoved(Plasma::Applet *applet)
@@ -264,14 +346,22 @@ void DefaultDesktop::onAppletRemoved(Plasma::Applet *applet)
     for (int i=0; i < m_layout->count(); i++) {
         if (applet == m_layout->itemAt(i)) {
             m_layout->removeAt(i);
+            m_layout->adjustPhysicalPositions();
             return;
         }
     }
 }
 
-void DefaultDesktop::onAppletGeometryChanged()
+void DefaultDesktop::onAppletTransformedByUser()
 {
-    m_layout->itemGeometryChanged((Applet *)sender());
+    m_layout->itemTransformed((Applet *)sender(), DesktopLayout::ItemTransformUser);
+    m_layout->adjustPhysicalPositions();
+}
+
+void DefaultDesktop::onAppletTransformedItself()
+{
+    m_layout->itemTransformed((Applet *)sender(), DesktopLayout::ItemTransformSelf);
+    m_layout->adjustPhysicalPositions();
 }
 
 void DefaultDesktop::refreshWorkingArea()
@@ -298,6 +388,7 @@ void DefaultDesktop::refreshWorkingArea()
     if (workingGeom != QRectF()) {
         //kDebug() << "!!!!!!!!!!!!! workingGeom is" << workingGeom;
         m_layout->setWorkingArea(workingGeom);
+        m_layout->adjustPhysicalPositions();
     }
 }
 
