@@ -21,18 +21,20 @@
 
 #include <limits>
 
-#include <QApplication>
-#include <QGraphicsLinearLayout>
-#include <QPainter>
-#include <QBitmap>
-#include <QDesktopWidget>
-#include <QGridLayout>
-#include <QLabel>
-#include <QComboBox>
 #include <QAction>
+#include <QApplication>
+#include <QBitmap>
+#include <QComboBox>
+#include <QDesktopWidget>
+#include <QGraphicsLinearLayout>
+#include <QGridLayout>
 #include <QGraphicsLayout>
 #include <QGraphicsSceneDragDropEvent>
-
+#include <QLabel>
+#include <QMenu>
+#include <QTimer>
+#include <QPainter>
+#include <QSignalMapper>
 
 #include <KDebug>
 #include <KIcon>
@@ -54,7 +56,8 @@ class Spacer : public QGraphicsWidget
 {
 public:
     Spacer(QGraphicsWidget *parent)
-         : QGraphicsWidget(parent)
+         : QGraphicsWidget(parent),
+           m_visible(true)
     {
         setAcceptDrops(true);
     }
@@ -63,6 +66,7 @@ public:
     {}
 
     Panel *panel;
+    bool m_visible;
 
 protected:
     void dropEvent(QGraphicsSceneDragDropEvent *event)
@@ -76,6 +80,10 @@ protected:
         Q_UNUSED(option)
         Q_UNUSED(widget)
 
+        if (!m_visible) {
+            return;
+        }
+
         //TODO: make this a pretty gradient?
         painter->setRenderHint(QPainter::Antialiasing);
         QPainterPath p = Plasma::PaintUtils::roundedRectangle(contentsRect().adjusted(1, 1, -2, -2), 4);
@@ -88,19 +96,21 @@ protected:
 
 Panel::Panel(QObject *parent, const QVariantList &args)
     : Containment(parent, args),
+      m_addPanelsMenu(0),
       m_configureAction(0),
       m_addPanelAction(0),
       m_currentSize(QSize(Kephal::ScreenUtils::screenSize(screen()).width(), 35)),
       m_maskDirty(true),
+      m_canResize(true),
       m_spacerIndex(-1),
-      m_spacer(0)
+      m_spacer(0),
+      m_lastSpace(0)
 {
     m_background = new Plasma::FrameSvg(this);
     m_background->setImagePath("widgets/panel-background");
     m_background->setEnabledBorders(Plasma::FrameSvg::AllBorders);
     connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(backgroundChanged()));
     setZValue(150);
-    setContainmentType(Containment::PanelContainment);
     resize(m_currentSize);
     setMinimumSize(m_currentSize);
     setMaximumSize(m_currentSize);
@@ -114,14 +124,25 @@ Panel::Panel(QObject *parent, const QVariantList &args)
 
 Panel::~Panel()
 {
+    delete m_addPanelsMenu;
 }
 
 void Panel::init()
 {
+    setContainmentType(Containment::PanelContainment);
+
     Containment::init();
     //FIXME: This should be enabled, but in that case proxywidgets won't get rendered
     //setFlag(ItemClipsChildrenToShape, true);
 
+    QGraphicsLinearLayout *lay = new QGraphicsLinearLayout(this);
+    lay->setContentsMargins(0, 0, 0, 0);
+    lay->setSpacing(4);
+    lay->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
+    setLayout(lay);
+    updateBorders(geometry().toRect());
+    lay->setMaximumSize(size());
+
     KConfigGroup cg = config("Configuration");
 
     m_currentSize = cg.readEntry("minimumSize", m_currentSize);
@@ -134,6 +155,10 @@ void Panel::init()
     setMinimumSize(cg.readEntry("minimumSize", m_currentSize));
     setMaximumSize(cg.readEntry("maximumSize", m_currentSize));
     setDrawWallpaper(false);
+
+    m_lastSpaceTimer = new QTimer(this);
+    m_lastSpaceTimer->setSingleShot(true);
+    connect(m_lastSpaceTimer, SIGNAL(timeout()), this, SLOT(adjustLastSpace()));
 }
 
 QList<QAction*> Panel::contextualActions()
@@ -143,14 +168,61 @@ QList<QAction*> Panel::contextualActions()
         m_configureAction->setIcon(KIcon("configure"));
         connect(m_configureAction, SIGNAL(triggered()), this, SIGNAL(toolBoxToggled()));
 
-        m_addPanelAction = new QAction(i18n("Add Panel"), this);
-        connect(m_addPanelAction, SIGNAL(triggered(bool)), this, SLOT(addPanel()));
-        m_addPanelAction->setIcon(KIcon("list-add"));
+        KPluginInfo::List panelPlugins = listContainmentsOfType("panel");
+
+        if (panelPlugins.size() == 1) {
+            m_addPanelAction = new QAction(i18n("Add Panel"), this);
+            connect(m_addPanelAction, SIGNAL(triggered(bool)), this, SLOT(addPanel()));
+        } else if (!panelPlugins.isEmpty()) {
+            m_addPanelsMenu = new QMenu();
+            m_addPanelAction = m_addPanelsMenu->menuAction();
+            m_addPanelAction->setText(i18n("Add Panel"));
+
+            QSignalMapper *mapper = new QSignalMapper(this);
+            connect(mapper, SIGNAL(mapped(QString)), this, SLOT(addPanel(QString)));
+
+            foreach (const KPluginInfo &plugin, panelPlugins) {
+                QAction *action = new QAction(plugin.name(), this);
+                if (!plugin.icon().isEmpty()) {
+                    action->setIcon(KIcon(plugin.icon()));
+                }
+
+                mapper->setMapping(action, plugin.pluginName());
+                connect(action, SIGNAL(triggered(bool)), mapper, SLOT(map()));
+                m_addPanelsMenu->addAction(action);
+            }
+        }
+
+        if (m_addPanelAction) {
+            m_addPanelAction->setIcon(KIcon("list-add"));
+        }
+
         constraintsEvent(Plasma::ImmutableConstraint);
     }
 
     QList<QAction*> actions;
-    actions << action("add widgets") << m_addPanelAction << action("lock widgets") << m_configureAction << action("remove");
+
+    QAction *a = action("add widgets");
+    if (a) {
+        actions.append(a);
+    }
+
+    if (m_addPanelAction) {
+        actions.append(m_addPanelAction);
+    }
+
+    a = action("lock widgets");
+    if (a) {
+        actions.append(a);
+    }
+
+    actions.append(m_configureAction);
+
+    a = action("remove");
+    if (a) {
+        actions.append(a);
+    }
+
     return actions;
 }
 
@@ -159,6 +231,52 @@ void Panel::backgroundChanged()
     constraintsEvent(Plasma::LocationConstraint);
 }
 
+void Panel::adjustLastSpace()
+{
+    QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(layout());
+    if (!lay) {
+        return;
+    }
+
+    bool useSpacer = true;
+
+    if (formFactor() == Plasma::Vertical) {
+        foreach (Applet *applet, applets()) {
+            if (applet->sizePolicy().verticalPolicy() == QSizePolicy::Expanding || applet->sizePolicy().verticalPolicy() == QSizePolicy::MinimumExpanding) {
+                useSpacer = false;
+                break;
+            }
+        }
+    } else {
+        foreach (Applet *applet, applets()) {
+            if (applet->sizePolicy().horizontalPolicy() & QSizePolicy::ExpandFlag) {
+                useSpacer = false;
+                break;
+            }
+        }
+    }
+
+    if (useSpacer) {
+        if (!m_lastSpace) {
+            m_lastSpace = new Spacer(this);
+            m_lastSpace->panel = this;
+            m_lastSpace->m_visible = false;
+            m_lastSpace->setPreferredSize(0,0);
+            m_lastSpace->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+            lay->addItem(m_lastSpace);
+        }
+    } else {
+        lay->removeItem(m_lastSpace);
+        delete m_lastSpace;
+        m_lastSpace = 0;
+    }
+}
+
+void Panel::enableUpdateSize()
+{
+    m_canResize = true;
+}
+
 void Panel::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
 {
     // this gets called whenever an applet is added, and we add it to our layout
@@ -211,18 +329,33 @@ void Panel::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
         }
     }
 
+    lay->removeItem(m_lastSpace);
+
     if (insertIndex == -1) {
         lay->addItem(applet);
     } else {
         lay->insertItem(insertIndex, applet);
     }
 
+    if (m_lastSpace) {
+        lay->addItem(m_lastSpace);
+    }
+
+    //FIXME: there must be some beter way to do this rather than this rather error prone arbitrary wait
+    m_lastSpaceTimer->start(2000);
+
     connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
 }
 
 void Panel::appletRemoved(Plasma::Applet* applet)
 {
     QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(layout());
+    if (!lay) {
+        return;
+    }
+
+    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
+
     lay->removeItem(applet);
 
     //shrink the panel if possible
@@ -232,13 +365,15 @@ void Panel::appletRemoved(Plasma::Applet* applet)
         resize(size().width(), size().height() - applet->size().height());
     }
     layout()->setMaximumSize(size());
+
+    m_lastSpaceTimer->start(200);
 }
 
 void Panel::updateSize()
 {
     Plasma::Applet *applet = qobject_cast<Plasma::Applet *>(sender());
 
-    if (applet) {
+    if (m_canResize && applet) {
         if (formFactor() == Plasma::Horizontal) {
             const int delta = applet->preferredWidth() - applet->size().width();
             //setting the preferred width when delta = 0 and preferredWidth() < minimumWidth()
@@ -254,19 +389,30 @@ void Panel::updateSize()
         }
 
         resize(preferredSize());
+        //for a while we won't execute updateSize() again
+        m_canResize = false;
+        QTimer::singleShot(400, this, SLOT(enableUpdateSize()));
     }
 }
 
 void Panel::addPanel()
 {
-    if (corona()) {
-        Containment* panel = corona()->addContainment("panel");
-        panel->showConfigurationInterface();
+    KPluginInfo::List panelPlugins = listContainmentsOfType("panel");
 
+    if (!panelPlugins.isEmpty()) {
+        addPanel(panelPlugins.first().pluginName());
+    }
+}
+
+void Panel::addPanel(const QString &plugin)
+{
+    if (corona()) {
+        Containment* panel = corona()->addContainment(plugin);
+        panel->showConfigurationInterface();
         panel->setScreen(screen());
 
         QList<Plasma::Location> freeEdges = corona()->freeEdges(screen());
-        kDebug() << freeEdges;
+        //kDebug() << freeEdges;
         Plasma::Location destination;
         if (freeEdges.contains(Plasma::TopEdge)) {
             destination = Plasma::TopEdge;
@@ -285,16 +431,38 @@ void Panel::addPanel()
         panel->updateConstraints(Plasma::StartupCompletedConstraint);
         panel->flushPendingConstraintsEvents();
 
-        if (destination == Plasma::LeftEdge ||
-            destination == Plasma::RightEdge) {
-            panel->setMinimumSize(10, 35);
-            panel->setMaximumSize(35,Kephal::ScreenUtils::screenSize(screen()).height());
-            panel->resize(QSize(35, Kephal::ScreenUtils::screenSize(screen()).height()));
+        const QRect screenGeom = corona()->screenGeometry(screen());
+        const QRegion availGeom = corona()->availableScreenRegion(screen());
+        int minH = 10;
+        int minW = 10;
+        int w = 35;
+        int h = 35;
+
+        if (destination == Plasma::LeftEdge) {
+            QRect r = availGeom.intersected(QRect(0, 0, w, screenGeom.height())).boundingRect();
+            h = r.height();
+            minW = 35;
+        } else if (destination == Plasma::RightEdge) {
+            QRect r = availGeom.intersected(QRect(screenGeom.width() - w, 0, w, screenGeom.height())).boundingRect();
+            h = r.height();
+            minW = 35;
+        } else if (destination == Plasma::TopEdge) {
+            QRect r = availGeom.intersected(QRect(0, 0, screenGeom.width(), h)).boundingRect();
+            w = r.width();
+            minH = 35;
+        } else if (destination == Plasma::BottomEdge) {
+            QRect r = availGeom.intersected(QRect(0, screenGeom.height() - h, screenGeom.width(), h)).boundingRect();
+            w = r.width();
+            minH = 35;
         }
+
+        panel->setMinimumSize(minW, minH);
+        panel->setMaximumSize(w, h);
+        panel->resize(w, h);
     }
 }
 
-void Panel::updateBorders(const QRect &geom)
+void Panel::updateBorders(const QRect &geom, bool themeChange)
 {
     Plasma::Location loc = location();
     FrameSvg::EnabledBorders enabledBorders = FrameSvg::AllBorders;
@@ -341,10 +509,12 @@ void Panel::updateBorders(const QRect &geom)
             enabledBorders ^= FrameSvg::LeftBorder;
             leftWidth = 0;
         }
+
         if (geom.y() <= r.y()) {
             enabledBorders ^= FrameSvg::TopBorder;
             topHeight = 0;
         }
+
         if (geom.bottom() >= r.bottom()) {
             enabledBorders ^= FrameSvg::BottomBorder;
             bottomHeight = 0;
@@ -365,14 +535,10 @@ void Panel::updateBorders(const QRect &geom)
         QSizeF s = box->boundingRect().size();
         if (formFactor() == Vertical) {
             //hardcoded extra margin for the toolbox right now
-            bottomHeight += s.height();;
+            bottomHeight += s.height();
             //Default to horizontal for now
         } else {
-            if (QApplication::layoutDirection() == Qt::RightToLeft) {
-                leftWidth += s.width();
-            } else {
-                rightWidth += s.width();
-            }
+            rightWidth += s.width();
         }
     }
 
@@ -394,8 +560,27 @@ void Panel::updateBorders(const QRect &geom)
         default:
             break;
         }
+
+        qreal oldLeft = leftWidth;
+        qreal oldTop = topHeight;
+        qreal oldRight = rightWidth;
+        qreal oldBottom = bottomHeight;
+
+        if (themeChange) {
+            layout()->getContentsMargins(&oldLeft, &oldTop, &oldRight, &oldBottom);
+        }
+
         layout()->setContentsMargins(leftWidth, topHeight, rightWidth, bottomHeight);
+        if (formFactor() == Plasma::Vertical) {
+            setPreferredHeight(preferredHeight() - (oldBottom - bottomHeight));
+        } else if (QApplication::layoutDirection() == Qt::LeftToRight) {
+            setPreferredWidth(preferredWidth() - (oldRight - rightWidth));
+        } else {
+            setPreferredWidth(preferredWidth() - (oldLeft - leftWidth));
+        }
+
         layout()->invalidate();
+        resize(preferredSize());
     }
 
     update();
@@ -403,35 +588,18 @@ void Panel::updateBorders(const QRect &geom)
 
 void Panel::constraintsEvent(Plasma::Constraints constraints)
 {
-    kDebug() << "constraints updated with" << constraints << "!!!!!!";
+    //kDebug() << "constraints updated with" << constraints << "!!!!!!";
 
     m_maskDirty = true;
 
     if (constraints & Plasma::FormFactorConstraint) {
         Plasma::FormFactor form = formFactor();
         Qt::Orientation layoutDirection = form == Plasma::Vertical ? Qt::Vertical : Qt::Horizontal;
-        // create our layout!
+        // create or set up our layout!
         if (layout()) {
-            QGraphicsLayout *lay = layout();
-            QGraphicsLinearLayout * linearLay = dynamic_cast<QGraphicsLinearLayout *>(lay);
-            if (linearLay) {
-                linearLay->setOrientation(layoutDirection);
-            }
+            QGraphicsLinearLayout * linearLay = static_cast<QGraphicsLinearLayout *>(layout());
             linearLay->setMaximumSize(size());
-        } else {
-            QGraphicsLinearLayout *lay = new QGraphicsLinearLayout(this);
-            lay->setOrientation(layoutDirection);
-            lay->setContentsMargins(0, 0, 0, 0);
-            lay->setSpacing(4);
-            lay->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
-            setLayout(lay);
-            updateBorders(geometry().toRect());
-
-            foreach (Applet *applet, applets()) {
-                lay->addItem(applet);
-            }
-
-            lay->setMaximumSize(size());
+            linearLay->setOrientation(layoutDirection);
         }
     }
 
@@ -439,10 +607,10 @@ void Panel::constraintsEvent(Plasma::Constraints constraints)
     if (constraints & Plasma::LocationConstraint || constraints & Plasma::SizeConstraint) {
         m_currentSize = geometry().size().toSize();
         QRectF screenRect = screen() >= 0 ? Kephal::ScreenUtils::screenGeometry(screen()) :
-            geometry();
+                                            geometry();
 
         if ((formFactor() == Horizontal && m_currentSize.width() >= screenRect.width()) ||
-                (formFactor() == Vertical && m_currentSize.height() >= screenRect.height())) {
+            (formFactor() == Vertical && m_currentSize.height() >= screenRect.height())) {
             m_background->setElementPrefix(location());
         } else {
             switch (location()) {
@@ -464,17 +632,17 @@ void Panel::constraintsEvent(Plasma::Constraints constraints)
         }
 
         m_background->resizeFrame(m_currentSize);
-    }
 
-    //FIXME: this seems the only way to correctly resize the layout the first time when the
-    // saved panel size is less than the default is to setting a maximum size.
-    // this shouldn't happen. maybe even a qgraphicslayout bug?
-    if (layout() && (constraints & Plasma::SizeConstraint)) {
-        layout()->setMaximumSize(size());
-    }
+        //FIXME: this seems the only way to correctly resize the layout the first time when the
+        // saved panel size is less than the default is to setting a maximum size.
+        // this shouldn't happen. maybe even a qgraphicslayout bug?
+        if (layout() && (constraints & Plasma::SizeConstraint)) {
+            layout()->setMaximumSize(size());
+        }
 
-    if (constraints & Plasma::LocationConstraint) {
-        setFormFactorFromLocation(location());
+        if (constraints & Plasma::LocationConstraint) {
+            setFormFactorFromLocation(location());
+        }
     }
 
     if (constraints & Plasma::ImmutableConstraint) {
@@ -490,10 +658,7 @@ void Panel::constraintsEvent(Plasma::Constraints constraints)
             m_configureAction->setVisible(unlocked);
         }
 
-        QGraphicsView *panelView = view();
-        if (panelView) {
-            updateBorders(panelView->geometry());
-        }
+        updateBorders(geometry().toRect());
     }
 }
 
@@ -505,6 +670,10 @@ void Panel::saveState(KConfigGroup &config) const
 
 void Panel::themeUpdated()
 {
+    if (!layout()) {
+        return;
+    }
+
     //if the theme is changed all the calculations needs to be done again
     //and resize based on the change in the theme bordersize
 
@@ -533,7 +702,7 @@ void Panel::themeUpdated()
         setMinimumHeight(newSize.height());
     }
 
-    updateBorders(geometry().toRect());
+    updateBorders(geometry().toRect(), true);
 }
 
 void Panel::paintInterface(QPainter *painter,
@@ -543,7 +712,6 @@ void Panel::paintInterface(QPainter *painter,
     Q_UNUSED(contentsRect)
     //FIXME: this background drawing is bad and ugly =)
     // draw the background untransformed (saves lots of per-pixel-math)
-    painter->save();
     painter->resetTransform();
 
     const Containment::StyleOption *containmentOpt = qstyleoption_cast<const Containment::StyleOption *>(option);
@@ -558,7 +726,7 @@ void Panel::paintInterface(QPainter *painter,
         m_lastViewGeom = viewGeom;
 
         updateBorders(viewGeom);
-        if (containmentOpt && containmentOpt->view) {
+        if (containmentOpt && containmentOpt->view && !m_background->mask().isEmpty()) {
             containmentOpt->view->setMask(m_background->mask());
         }
     }
@@ -568,9 +736,6 @@ void Panel::paintInterface(QPainter *painter,
     painter->setRenderHint(QPainter::Antialiasing);
 
     m_background->paintFrame(painter, option->exposedRect);
-
-    // restore transformation and composition mode
-    painter->restore();
 }
 
 void Panel::setFormFactorFromLocation(Plasma::Location loc) {
