@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2007-2008 by Shawn Starr <shawn.starr@rogers.com>       *
+ *   Copyright (C) 2007-2009 by Shawn Starr <shawn.starr@rogers.com>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,8 +19,8 @@
 
 /* Ion for BBC Weather from UKMET Office */
 
-#ifndef _ION_BBCUKMET_H
-#define _ION_BBCUKMET_H
+#ifndef ION_BBCUKMET_H
+#define ION_BBCUKMET_H
 
 #include <QtXml/QXmlStreamReader>
 #include <QRegExp>
@@ -44,12 +44,13 @@ public:
     // Current observation information.
     QString obsTime;
     int iconPeriodHour;
-    //QString iconPeriodAP;
+    int iconPeriodMinute;
+    double longitude;
+    double latitude;
 
     QString condition;
     QString conditionIcon;
     QString temperature_C;
-    QString temperature_F;
     QString windDirection;
     QString windSpeed_miles;
     QString humidity;
@@ -88,6 +89,9 @@ public:
     QString observationTime(const QString& source);
     //bool night(const QString& source);
     int periodHour(const QString& source);
+    int periodMinute(const QString& source);
+    double periodLatitude(const QString& source);
+    double periodLongitude(const QString& source);
     QString condition(const QString& source);
     QMap<QString, QString> temperature(const QString& source);
     QMap<QString, QString> wind(const QString& source);
@@ -96,10 +100,13 @@ public:
     QMap<QString, QString> pressure(const QString& source);
     QVector<QString> forecasts(const QString& source);
 
+public slots:
+    virtual void reset();
+
 protected slots:
     void setup_slotDataArrived(KIO::Job *, const QByteArray &);
     void setup_slotJobFinished(KJob *);
-    void setup_slotRedirected(KIO::Job *, const KUrl &url);
+    //void setup_slotRedirected(KIO::Job *, const KUrl &url);
     void observation_slotDataArrived(KIO::Job *, const QByteArray &);
     void observation_slotJobFinished(KJob *);
     void forecast_slotDataArrived(KIO::Job *, const QByteArray &);
@@ -119,7 +126,7 @@ private:
     void validate(const QString& source); // Sync data source with Applet
     void getFiveDayForecast(const QString& source);
     void getXMLData(const QString& source);
-    bool readSearchXMLData(const QString& source, QXmlStreamReader& xml);
+    void readSearchHTMLData(const QString& source, const QByteArray& html);
     bool readFiveDayForecastXMLData(const QString& source, QXmlStreamReader& xml);
     void parseSearchLocations(const QString& source, QXmlStreamReader& xml);
 
@@ -129,13 +136,15 @@ private:
     void parseWeatherChannel(const QString& source, WeatherData& data, QXmlStreamReader& xml);
     void parseWeatherObservation(const QString& source, WeatherData& data, QXmlStreamReader& xml);
     void parseFiveDayForecast(const QString& source, QXmlStreamReader& xml);
+    void parsePlaceForecast(const QString& source, QXmlStreamReader& xml);
+    void parseWeatherForecast(const QString& source, QXmlStreamReader& xml);
     void parseUnknownElement(QXmlStreamReader& xml);
 
 private:
     class Private;
-    Private *const d;
+    Private * d;
 };
 
-K_EXPORT_PLASMA_ION(bbcukmet, UKMETIon)
+K_EXPORT_PLASMA_DATAENGINE(bbcukmet, UKMETIon)
 
 #endif
