@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2007-2008 by Shawn Starr <shawn.starr@rogers.com>       *
+ *   Copyright (C) 2007-2009 by Shawn Starr <shawn.starr@rogers.com>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -24,28 +24,16 @@
 class NOAAIon::Private : public QObject
 {
 public:
-    Private() {
-        m_url = 0;
-    }
-    ~Private() {
-        delete m_url;
-    }
-
-private:
     struct XMLMapInfo {
         QString stateName;
         QString stationName;
+        QString stationID;
         QString XMLurl;
-        QString sourceOptions;
     };
 
 public:
     // Key dicts
-    QHash<QString, NOAAIon::Private::XMLMapInfo> m_place;
-    QHash<QString, QString> m_locations;
-    QString m_state;
-    QString m_station_name;
-    QString m_xmlurl;
+    QHash<QString, NOAAIon::Private::XMLMapInfo> m_places;
 
     // Weather information
     QHash<QString, WeatherData> m_weatherData;
@@ -54,10 +42,13 @@ public:
     QMap<KJob *, QXmlStreamReader*> m_jobXml;
     QMap<KJob *, QString> m_jobList;
     QXmlStreamReader m_xmlSetup;
-    KUrl *m_url;
-    KIO::TransferJob *m_job;
 
     QDateTime m_dateFormat;
+    bool emitWhenSetup;
+
+    Private() : emitWhenSetup(false) {
+    }
+
 };
 
 QMap<QString, IonInterface::WindDirections> NOAAIon::setupWindIconMappings(void)
@@ -101,6 +92,15 @@ NOAAIon::NOAAIon(QObject *parent, const QVariantList &args)
     Q_UNUSED(args)
 }
 
+void NOAAIon::reset()
+{
+    delete d;
+    d = new Private();
+    d->emitWhenSetup = true;
+    setInitialized(false);
+    getXMLSetup();
+}
+
 NOAAIon::~NOAAIon()
 {
     // Destroy dptr
@@ -117,20 +117,32 @@ void NOAAIon::init()
 QStringList NOAAIon::validate(const QString& source) const
 {
     QStringList placeList;
-    QHash<QString, QString>::const_iterator it = d->m_locations.constBegin();
-    while (it != d->m_locations.constEnd()) {
-        if (it.value().toLower().contains(source.toLower())) {
-            placeList.append(QString("place|%1").arg(it.value().split('|')[1]));
+    QString station;
+    QString sourceNormalized = source.toUpper();
+
+    QHash<QString, NOAAIon::Private::XMLMapInfo>::const_iterator it = d->m_places.constBegin();
+    // If the source name might look like a station ID, check these too and return the name
+    bool checkState = source.count() == 2;
+
+    while (it != d->m_places.constEnd()) {
+        if (checkState) {
+            if (it.value().stateName == source) {
+                placeList.append(QString("place|").append(it.key()));
+            }
+        } else if (it.key().toUpper().contains(sourceNormalized)) {
+            placeList.append(QString("place|").append(it.key()));
+        } else if (it.value().stationID == sourceNormalized) {
+            station = QString("place|").append(it.key());
         }
+
         ++it;
     }
 
-    // Check if placeList is empty if so, return nothing.
-    if (placeList.isEmpty()) {
-        return QStringList();
+    placeList.sort();
+    if (!station.isEmpty()) {
+        placeList.prepend(station);
     }
 
-    placeList.sort();
     return placeList;
 }
 
@@ -144,43 +156,46 @@ bool NOAAIon::updateIonSource(const QString& source)
 
     // Guard: if the size of array is not 2 then we have bad data, return an error
     if (sourceAction.size() < 2) {
-        setData(source, "validate", QString("noaa|timeout"));
+        setData(source, "validate", "noaa|malformed");
         return true;
     }
-    
-    if (sourceAction[1] == QString("validate")) {
-        kDebug() << "Initiate Validating of place: " << sourceAction[2];
-        QStringList result = validate(QString("%1|%2").arg(sourceAction[0]).arg(sourceAction[2]));
+
+    if (sourceAction[1] == "validate" && sourceAction.size() > 2) {
+        QStringList result = validate(sourceAction[2]);
 
         if (result.size() == 1) {
-            setData(source, "validate", QString("noaa|valid|single|%1").arg(result.join("|")));
+            setData(source, "validate", QString("noaa|valid|single|").append(result.join("|")));
             return true;
         } else if (result.size() > 1) {
-            setData(source, "validate", QString("noaa|valid|multiple|%1").arg(result.join("|")));
+            setData(source, "validate", QString("noaa|valid|multiple|").append(result.join("|")));
             return true;
         } else if (result.size() == 0) {
-            setData(source, "validate", QString("noaa|invalid|single|%1").arg(sourceAction[2]));
+            setData(source, "validate", QString("noaa|invalid|single|").append(sourceAction[2]));
             return true;
         }
 
-    } else if (sourceAction[1] == QString("weather")) {
+    } else if (sourceAction[1] == "weather" && sourceAction.size() > 2) {
         getXMLData(source);
         return true;
+    } else {
+        setData(source, "validate", "noaa|malformed");
+        return true;
     }
+
     return false;
 }
 
 // Parses city list and gets the correct city based on ID number
 void NOAAIon::getXMLSetup()
 {
-    d->m_url = new KUrl("http://www.weather.gov/data/current_obs/index.xml");
-
-    KIO::TransferJob *job = KIO::get(d->m_url->url(), KIO::NoReload, KIO::HideProgressInfo);
+    KIO::TransferJob *job = KIO::get(KUrl("http://www.weather.gov/data/current_obs/index.xml"), KIO::NoReload, KIO::HideProgressInfo);
 
     if (job) {
         connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
                 SLOT(setup_slotDataArrived(KIO::Job *, const QByteArray &)));
         connect(job, SIGNAL(result(KJob *)), this, SLOT(setup_slotJobFinished(KJob *)));
+    } else {
+        kDebug() << "Could not create place name list transfer job";
     }
 }
 
@@ -190,26 +205,23 @@ void NOAAIon::getXMLData(const QString& source)
     KUrl url;
 
     QString dataKey = source;
-    dataKey.remove("|weather");
-    url = d->m_place[dataKey].XMLurl;
-
-    kDebug() << "URL Location: " << url.url();
+    dataKey.remove("noaa|weather|");
+    url = d->m_places[dataKey].XMLurl;
 
     // If this is empty we have no valid data, send out an error and abort.
-    //
-    if (url.url().isEmpty()) { 
-        setData(source, "validate", QString("noaa|timeout"));
+    if (url.url().isEmpty()) {
+        setData(source, "validate", QString("noaa|malformed"));
         return;
     }
 
-    d->m_job = KIO::get(url.url(), KIO::Reload, KIO::HideProgressInfo);
-    d->m_jobXml.insert(d->m_job, new QXmlStreamReader);
-    d->m_jobList.insert(d->m_job, source);
+    KIO::TransferJob * const m_job = KIO::get(url.url(), KIO::Reload, KIO::HideProgressInfo);
+    d->m_jobXml.insert(m_job, new QXmlStreamReader);
+    d->m_jobList.insert(m_job, source);
 
-    if (d->m_job) {
-        connect(d->m_job, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
-                SLOT(slotDataArrived(KIO::Job *, const QByteArray &)));
-        connect(d->m_job, SIGNAL(result(KJob *)), this, SLOT(slotJobFinished(KJob *)));
+    if (m_job) {
+        connect(m_job, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
+             SLOT(slotDataArrived(KIO::Job *, const QByteArray &)));
+        connect(m_job, SIGNAL(result(KJob *)), this, SLOT(slotJobFinished(KJob *)));
     }
 }
 
@@ -238,44 +250,61 @@ void NOAAIon::slotDataArrived(KIO::Job *job, const QByteArray &data)
 void NOAAIon::slotJobFinished(KJob *job)
 {
     // Dual use method, if we're fetching location data to parse we need to do this first
-    setData(d->m_jobList[job], Data());
-    readXMLData(d->m_jobList[job], *d->m_jobXml[job]);
+    removeAllData(d->m_jobList[job]);
+    QXmlStreamReader *reader = d->m_jobXml.value(job);
+    if (reader) {
+        readXMLData(d->m_jobList[job], *reader);
+    }
+
     d->m_jobList.remove(job);
-    delete d->m_jobXml[job];
     d->m_jobXml.remove(job);
+    delete reader;
 }
 
 void NOAAIon::setup_slotJobFinished(KJob *job)
 {
     Q_UNUSED(job)
-    readXMLSetup();
-    setInitialized(true);
+    const bool success = readXMLSetup();
+    setInitialized(success);
+    if (d->emitWhenSetup) {
+        d->emitWhenSetup = false;
+        emit(resetCompleted(this, success));
+    }
 }
 
 void NOAAIon::parseStationID()
 {
-    QString tmp;
+    QString state;
+    QString stationName;
+    QString stationID;
+    QString xmlurl;
+
     while (!d->m_xmlSetup.atEnd()) {
         d->m_xmlSetup.readNext();
 
         if (d->m_xmlSetup.isEndElement() && d->m_xmlSetup.name() == "station") {
+            if (!xmlurl.isEmpty()) {
+                NOAAIon::Private::XMLMapInfo info;
+                info.stateName = state;
+                info.stationName = stationName;
+                info.stationID = stationID;
+                info.XMLurl = xmlurl;
+
+                QString tmp = stationName + ", " + state; // Build the key name.
+                d->m_places[tmp] = info;
+            }
             break;
         }
 
         if (d->m_xmlSetup.isStartElement()) {
-            if (d->m_xmlSetup.name() == "state") {
-                d->m_state = d->m_xmlSetup.readElementText();
+            if (d->m_xmlSetup.name() == "station_id") {
+                stationID = d->m_xmlSetup.readElementText();
+            } else if (d->m_xmlSetup.name() == "state") {
+                state = d->m_xmlSetup.readElementText();
             } else if (d->m_xmlSetup.name() == "station_name") {
-                d->m_station_name = d->m_xmlSetup.readElementText();
+                stationName = d->m_xmlSetup.readElementText();
             } else if (d->m_xmlSetup.name() == "xml_url") {
-                d->m_xmlurl = d->m_xmlSetup.readElementText();
-
-                tmp = "noaa|" + d->m_station_name + ", " + d->m_state; // Build the key name.
-                d->m_place[tmp].stateName = d->m_state;
-                d->m_place[tmp].stationName = d->m_station_name;
-                d->m_place[tmp].XMLurl = d->m_xmlurl.replace("http://", "http://www.");
-
-                d->m_locations[tmp] = tmp;
+                xmlurl = d->m_xmlSetup.readElementText().replace("http://", "http://www.");
             } else {
                 parseUnknownElement(d->m_xmlSetup);
             }
@@ -305,16 +334,18 @@ void NOAAIon::parseStationList()
 // Parse the city list and store into a QMap
 bool NOAAIon::readXMLSetup()
 {
+    bool success = false;
     while (!d->m_xmlSetup.atEnd()) {
         d->m_xmlSetup.readNext();
 
         if (d->m_xmlSetup.isStartElement()) {
             if (d->m_xmlSetup.name() == "wx_station_index") {
                 parseStationList();
+                success = true;
             }
         }
     }
-    return !d->m_xmlSetup.error();
+    return (!d->m_xmlSetup.error() && success);
 }
 
 WeatherData NOAAIon::parseWeatherSite(WeatherData& data, QXmlStreamReader& xml)
@@ -343,6 +374,10 @@ WeatherData NOAAIon::parseWeatherSite(WeatherData& data, QXmlStreamReader& xml)
                 data.locationName = xml.readElementText();
             } else if (xml.name() == "station_id") {
                 data.stationID = xml.readElementText();
+            } else if (xml.name() == "latitude") {
+                data.stationLat = xml.readElementText();
+            } else if (xml.name() == "longitude") {
+                data.stationLon = xml.readElementText();
             } else if (xml.name() == "observation_time") {
                 data.observationTime = xml.readElementText();
                 QStringList tmpDateStr = data.observationTime.split(' ');
@@ -436,88 +471,95 @@ void NOAAIon::parseUnknownElement(QXmlStreamReader& xml)
 void NOAAIon::updateWeather(const QString& source)
 {
     QMap<QString, QString> dataFields;
-    QStringList fieldList;
+    Plasma::DataEngine::Data data;
 
-    setData(source, "Country", country(source));
-    setData(source, "Place", place(source));
-    setData(source, "Station", station(source));
+    data.insert("Country", country(source));
+    data.insert("Place", place(source));
+    data.insert("Station", station(source));
+
+    data.insert("Latitude", latitude(source));
+    data.insert("Longitude", longitude(source));
 
     // Real weather - Current conditions
-    setData(source, "Observation Period", observationTime(source));
-    setData(source, "Current Conditions", condition(source));
-
+    data.insert("Observation Period", observationTime(source));
+    data.insert("Current Conditions", condition(source));
+    kDebug() << "i18n condition string: " << qPrintable(condition(source));
 // FIXME: We'll need major fuzzy logic, this isn't pretty: http://www.weather.gov/xml/current_obs/weather.php
     //QMap<QString, ConditionIcons> conditionList;
     //conditionList = conditionIcons();
 
-/*
-    if ((periodHour(source) >= 0 && periodHour(source) < 6) || (periodHour(source) >= 18)) {
-        // Night
-        // - Fill in condition fuzzy logic 
-    } else {
-        // Day
-        // - Fill in condition fuzzy logic
-    }
-*/
-    setData(source, "Condition Icon", "weather-none-available");
+    /*
+        if ((periodHour(source) >= 0 && periodHour(source) < 6) || (periodHour(source) >= 18)) {
+            // Night
+            // - Fill in condition fuzzy logic
+        } else {
+            // Day
+            // - Fill in condition fuzzy logic
+        }
+    */
+    data.insert("Condition Icon", "weather-none-available");
 
     dataFields = temperature(source);
-    setData(source, "Temperature", dataFields["temperature"]);
+    data.insert("Temperature", dataFields["temperature"]);
 
     if (dataFields["temperature"] != "N/A") {
-        setData(source, "Temperature Unit", dataFields["temperatureUnit"]);
+        data.insert("Temperature Unit", dataFields["temperatureUnit"]);
     }
 
     // Do we have a comfort temperature? if so display it
     if (dataFields["comfortTemperature"] != "N/A") {
         if (d->m_weatherData[source].windchill_F != "NA") {
-            setData(source, "Windchill", QString("%1").arg(dataFields["comfortTemperature"]));
-            setData(source, "Humidex", "N/A");
+            data.insert("Windchill", QString("%1").arg(dataFields["comfortTemperature"]));
+            data.insert("Humidex", "N/A");
         }
         if (d->m_weatherData[source].heatindex_F != "NA" && d->m_weatherData[source].temperature_F.toInt() != d->m_weatherData[source].heatindex_F.toInt()) {
-            setData(source, "Humidex", QString("%1").arg(dataFields["comfortTemperature"]));
-            setData(source, "Windchill", "N/A");
+            data.insert("Humidex", QString("%1").arg(dataFields["comfortTemperature"]));
+            data.insert("Windchill", "N/A");
         }
     } else {
-        setData(source, "Windchill", "N/A");
-        setData(source, "Humidex", "N/A");
+        data.insert("Windchill", "N/A");
+        data.insert("Humidex", "N/A");
     }
 
-    setData(source, "Dewpoint", dewpoint(source));
+    data.insert("Dewpoint", dewpoint(source));
     if (dewpoint(source) != "N/A") {
-        setData(source, "Dewpoint Unit", dataFields["temperatureUnit"]);
+        data.insert("Dewpoint Unit", dataFields["temperatureUnit"]);
     }
 
     dataFields = pressure(source);
-    setData(source, "Pressure", dataFields["pressure"]);
+    data.insert("Pressure", dataFields["pressure"]);
 
     if (dataFields["pressure"] != "N/A") {
-        setData(source, "Pressure Unit", dataFields["pressureUnit"]);
+        data.insert("Pressure Unit", dataFields["pressureUnit"]);
     }
 
     dataFields = visibility(source);
-    setData(source, "Visibility", dataFields["visibility"]);
+    data.insert("Visibility", dataFields["visibility"]);
 
     if (dataFields["visibility"] != "N/A") {
-        setData(source, "Visibility Unit", dataFields["visibilityUnit"]);
+        data.insert("Visibility Unit", dataFields["visibilityUnit"]);
     }
 
-    setData(source, "Humidity", humidity(source));
+    if (humidity(source) != "N/A") {
+        data.insert("Humidity", QString("%1%").arg(humidity(source))); // FIXME: Turn '%' into a unit field
+    }
 
     // Set number of forecasts per day/night supported, none for this ion right now
-    setData(source, QString("Total Weather Days"), 0);
+    data.insert(QString("Total Weather Days"), 0);
 
     dataFields = wind(source);
-    setData(source, "Wind Speed", dataFields["windSpeed"]);
+    data.insert("Wind Speed", dataFields["windSpeed"]);
 
     if (dataFields["windSpeed"] != "Calm") {
-        setData(source, "Wind Speed Unit", dataFields["windUnit"]);
+        data.insert("Wind Speed Unit", dataFields["windUnit"]);
     }
 
-    setData(source, "Wind Gust", dataFields["windGust"]);
-    setData(source, "Wind Gust Unit", dataFields["windGustUnit"]);
-    setData(source, "Wind Direction", getWindDirectionIcon(windIcons(), dataFields["windDirection"].toLower()));
-    setData(source, "Credit", "Data provided by NOAA National Weather Service");
+    data.insert("Wind Gust", dataFields["windGust"]);
+    data.insert("Wind Gust Unit", dataFields["windGustUnit"]);
+    data.insert("Wind Direction", getWindDirectionIcon(windIcons(), dataFields["windDirection"].toLower()));
+    data.insert("Credit", i18n("Data provided by NOAA National Weather Service"));
+
+    setData(source, data);
 }
 
 QString NOAAIon::country(const QString& source)
@@ -525,15 +567,27 @@ QString NOAAIon::country(const QString& source)
     Q_UNUSED(source);
     return QString("USA");
 }
+
 QString NOAAIon::place(const QString& source)
 {
     return d->m_weatherData[source].locationName;
 }
+
 QString NOAAIon::station(const QString& source)
 {
     return d->m_weatherData[source].stationID;
 }
 
+QString NOAAIon::latitude(const QString& source)
+{
+    return d->m_weatherData[source].stationLat;
+}
+
+QString NOAAIon::longitude(const QString& source)
+{
+    return d->m_weatherData[source].stationLon;
+}
+
 QString NOAAIon::observationTime(const QString& source)
 {
     return d->m_weatherData[source].observationTime;
@@ -559,7 +613,7 @@ QString NOAAIon::condition(const QString& source)
     if (d->m_weatherData[source].weather.isEmpty() || d->m_weatherData[source].weather == "NA") {
         d->m_weatherData[source].weather = "N/A";
     }
-    return d->m_weatherData[source].weather;
+    return i18nc("weather condition", d->m_weatherData[source].weather.toUtf8());
 }
 
 QString NOAAIon::dewpoint(const QString& source)
@@ -572,7 +626,7 @@ QString NOAAIon::humidity(const QString& source)
     if (d->m_weatherData[source].humidity == "NA") {
         return QString("N/A");
     } else {
-        return QString("%1%").arg(d->m_weatherData[source].humidity);
+        return (d->m_weatherData[source].humidity);
     }
 }
 
@@ -583,8 +637,13 @@ QMap<QString, QString> NOAAIon::visibility(const QString& source)
         visibilityInfo.insert("visibility", QString("N/A"));
         return visibilityInfo;
     }
-    visibilityInfo.insert("visibility", d->m_weatherData[source].visibility);
-    visibilityInfo.insert("visibilityUnit", QString::number(WeatherUtils::Miles));
+    if (d->m_weatherData[source].visibility == "NA") {
+        visibilityInfo.insert("visibility", QString("N/A"));
+        visibilityInfo.insert("visibilityUnit", QString::number(WeatherUtils::NoUnit));
+    } else {
+        visibilityInfo.insert("visibility", d->m_weatherData[source].visibility);
+        visibilityInfo.insert("visibilityUnit", QString::number(WeatherUtils::Miles));
+    }
     return visibilityInfo;
 }
 
@@ -614,8 +673,13 @@ QMap<QString, QString> NOAAIon::pressure(const QString& source)
         return pressureInfo;
     }
 
-    pressureInfo.insert("pressure", d->m_weatherData[source].pressure);
-    pressureInfo.insert("pressureUnit", QString::number(WeatherUtils::InchesHG));
+    if (d->m_weatherData[source].pressure == "NA") {
+        pressureInfo.insert("pressure", "N/A");
+        pressureInfo.insert("visibilityUnit", QString::number(WeatherUtils::NoUnit));
+    } else {
+        pressureInfo.insert("pressure", d->m_weatherData[source].pressure);
+        pressureInfo.insert("pressureUnit", QString::number(WeatherUtils::InchesHG));
+    }
     return pressureInfo;
 }
 
@@ -625,26 +689,26 @@ QMap<QString, QString> NOAAIon::wind(const QString& source)
 
     // May not have any winds
     if (d->m_weatherData[source].windSpeed == "NA") {
-        windInfo.insert("windSpeed", "Calm");
+        windInfo.insert("windSpeed", i18nc("wind speed", "Calm"));
         windInfo.insert("windUnit", QString::number(WeatherUtils::NoUnit));
     } else {
         windInfo.insert("windSpeed", QString::number(d->m_weatherData[source].windSpeed.toFloat(), 'f', 1));
-        windInfo.insert("windUnit", QString::number(WeatherUtils::MilesAnHour));
+        windInfo.insert("windUnit", QString::number(WeatherUtils::MilesPerHour));
     }
 
     // May not always have gusty winds
-    if (d->m_weatherData[source].windGust == "NA") {
+    if (d->m_weatherData[source].windGust == "NA" || d->m_weatherData[source].windGust == "N/A") {
         windInfo.insert("windGust", "N/A");
         windInfo.insert("windGustUnit", QString::number(WeatherUtils::NoUnit));
     } else {
         windInfo.insert("windGust", QString::number(d->m_weatherData[source].windGust.toFloat(), 'f', 1));
-        windInfo.insert("windGustUnit", QString::number(WeatherUtils::MilesAnHour));
+        windInfo.insert("windGustUnit", QString::number(WeatherUtils::MilesPerHour));
     }
 
     if (d->m_weatherData[source].windDirection.isEmpty()) {
         windInfo.insert("windDirection", "N/A");
     } else {
-        windInfo.insert("windDirection", d->m_weatherData[source].windDirection);
+        windInfo.insert("windDirection", i18nc("wind direction", d->m_weatherData[source].windDirection.toUtf8()));
     }
     return windInfo;
 }
