@@ -18,7 +18,6 @@
 
 #include "locationrunner.h"
 
-#include <QAction>
 #include <QDir>
 #include <QStringList>
 
@@ -42,6 +41,8 @@ LocationsRunner::LocationsRunner(QObject *parent, const QVariantList& args)
     // set the name shown after the result in krunner window
     setObjectName("Locations");
     setIgnoredTypes(Plasma::RunnerContext::Executable | Plasma::RunnerContext::ShellCommand);
+    addSyntax(Plasma::RunnerSyntax(":q:",
+              i18n("Finds local directories and files, network locations and Internet sites with paths matching :q:.")));
 }
 
 LocationsRunner::~LocationsRunner()
@@ -57,7 +58,12 @@ static void processUrl(KUrl &url, const QString &term)
         if (idx != -1) {
             url.setPath(term.mid(idx));
         }
-        url.setProtocol("http");
+        if (term.startsWith("ftp")) {
+            url.setProtocol("ftp");
+        }
+        else {
+            url.setProtocol("http");
+        }
     }
 }
 
@@ -88,7 +94,6 @@ void LocationsRunner::match(Plasma::RunnerContext &context)
         match.setText(i18n("Open %1", term));
         match.setIcon(KIcon("system-help"));
         match.setRelevance(1);
-        match.setRelevance(1);
         match.setType(Plasma::QueryMatch::ExactMatch);
         match.setId("help");
         context.addMatch(term, match);
@@ -97,7 +102,7 @@ void LocationsRunner::match(Plasma::RunnerContext &context)
                 term.contains(QRegExp("^[a-zA-Z0-9\\-\\.]+\\.[a-zA-Z]{2,6}")))) {
         KUrl url(term);
         processUrl(url, term);
-        QMutexLocker lock(bigLock());
+
         if (!KProtocolInfo::isKnownProtocol(url.protocol())) {
             return;
         }
@@ -108,7 +113,7 @@ void LocationsRunner::match(Plasma::RunnerContext &context)
         match.setData(url.url());
 
         if (KProtocolInfo::isHelperProtocol(url.protocol())) {
-            //kDebug() << "helper protocol" << url.protocol() <<"call external application" ; 
+            //kDebug() << "helper protocol" << url.protocol() <<"call external application" ;
             match.setText(i18n("Launch with %1", KProtocolInfo::exec(url.protocol())));
         } else {
             //kDebug() << "protocol managed by browser" << url.protocol();
@@ -131,13 +136,13 @@ void LocationsRunner::match(Plasma::RunnerContext &context)
 
 void LocationsRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
 {
-    QString data = match.data().toString();
     const QString location = context.query();
-    Plasma::RunnerContext::Type type = context.type();
 
     if (location.isEmpty()) {
         return;
     }
+    QString data = match.data().toString();
+    Plasma::RunnerContext::Type type = context.type();
 
     //kDebug() << "command: " << context.query();
     //kDebug() << "url: " << location << data;
@@ -145,7 +150,7 @@ void LocationsRunner::run(const Plasma::RunnerContext &context, const Plasma::Qu
     KUrl urlToRun(location);
 
     if ((type == Plasma::RunnerContext::NetworkLocation || type == Plasma::RunnerContext::UnknownType) &&
-        data.startsWith("http://")) {
+        (data.startsWith("http://") || data.startsWith("ftp://"))) {
         // the text may have changed while we were running, so we have to refresh
         // our content
         processUrl(urlToRun, location);
