@@ -67,6 +67,7 @@ void Nepomuk::SearchRunner::init()
 
     // we are way less important than others, mostly because we are slow
     setPriority( LowPriority );
+    addSyntax(Plasma::RunnerSyntax(":q:", i18n("Finds files, documents and other content that matches :q: using the desktop search system.")));
 }
 
 
@@ -110,15 +111,14 @@ void Nepomuk::SearchRunner::run( const Plasma::RunnerContext&, const Plasma::Que
     Nepomuk::Resource res = match.data().value<Nepomuk::Resource>();
     KUrl url;
 
-    if( res.hasType( Soprano::Vocabulary::NAO::Tag() ) ) {
-        url.setProtocol( "nepomuksearch" );
-        url.setPath( QString( "/hasTag:\"%1\"" ).arg( res.genericLabel() ) );
-     }
-    else {
+    if (res.hasType( Soprano::Vocabulary::NAO::Tag())) {
+        url.setProtocol("nepomuksearch");
+        url.setPath(QString("/hasTag:\"%1\"").arg(res.genericLabel()));
+    } else {
         url = res.resourceUri();
     }
 
-    (void)new KRun( url, 0 );
+    (void)new KRun(url, 0);
 }
 
 K_EXPORT_PLASMA_RUNNER(nepomuksearchrunner, Nepomuk::SearchRunner)
