@@ -21,13 +21,14 @@ require 'plasma_applet'
 
 module PlasmaScripting
   class Applet < Qt::Object
-  slots  "setImmutability(Plasma::ImmutabilityType)",
+    slots  "setImmutability(Plasma::ImmutabilityType)",
             :destroy,
             :showConfigurationInterface,
             :raise,
             :lower,
             :flushPendingConstraintsEvents,
-            :init
+            :init,
+            'initExtenderItem(Plasma::ExtenderItem*)'
 
     signals :releaseVisualFocus,
             :geometryChanged,
@@ -43,6 +44,7 @@ module PlasmaScripting
       connect(@applet_script.applet, SIGNAL(:geometryChanged), self, SIGNAL(:geometryChanged))
       connect(@applet_script.applet, SIGNAL(:configNeedsSaving), self, SIGNAL(:configNeedsSaving))
       connect(@applet_script.applet, SIGNAL(:activate), self, SIGNAL(:activate))
+      connect(@applet_script.applet, SIGNAL('extenderItemRestored(Plasma::ExtenderItem*)'), self, SLOT('initExtenderItem(PlasmExtenderItem*)'))
     end
 
     # If a method is called on a PlasmaScripting::Applet instance is found to be missing
@@ -63,9 +65,19 @@ module PlasmaScripting
       end
     end
 
+    def configChanged
+    end
+
     def paintInterface(painter, option, contentsRect)
     end
 
+    def initExtenderItem(item)
+       puts "Missing implementation of initExtenderItem in the applet " \
+                   "#{item.config.readEntry('SourceAppletPluginName', '')}" \
+                   "!\n Any applet that uses extenders should implement initExtenderItem to " \
+                   "instantiate a widget."
+    end
+
     def size
       @applet_script.size
     end
@@ -74,6 +86,18 @@ module PlasmaScripting
       @applet_script.shape
     end
 
+    def resize(*args)
+      if args.length == 1 && args[0].kindof?(Qt::Size)
+        puts "Warning: invalid resize() call. Add this to your metadata.desktop file:"
+        puts "X-Plasma-DefautSize=%d,%d" % [args[0].width, args[0].height]
+      elsif args.length == 2 
+        puts "Warning: invalid resize() call. Add this to your metadata.desktop file:"
+        puts "X-Plasma-DefautSize=%d,%d" % [args[0].to_i, args[1].to_i]
+      else
+        super(*args)
+      end
+    end
+
     def constraintsEvent(constraints)
     end
 
@@ -133,12 +157,25 @@ module PlasmaScripting
       @applet_script.applet.flushPendingConstraintsEvents
     end
   end
+
+  class Containment < Applet
+    def initialize(parent, args = nil)
+      super(parent, args)
+    end
+  end
+
+  class PopupApplet < Applet
+    def initialize(parent, args = nil)
+      super(parent, args)
+    end
+  end
 end
 
 module PlasmaScriptengineRuby
   class Applet < Plasma::AppletScript
     def initialize(parent, args)
       super(parent)
+      @applet = applet
     end
 
     def camelize(str)
@@ -166,6 +203,10 @@ module PlasmaScriptengineRuby
       return true
     end
 
+    def configChanged
+      @applet_script.configChanged
+    end
+
     def paintInterface(painter, option, contentsRect)
       @applet_script.paintInterface(painter, option, contentsRect)
     end
@@ -435,6 +476,16 @@ module Plasma
     end
   end
 
+  class ScrollWidget < Qt::Base
+    def initialize(parent = nil)
+      if parent.kind_of?(PlasmaScripting::Applet)
+        super(parent.applet_script.applet)
+      else
+        super
+      end
+    end
+  end
+
   class SignalPlotter < Qt::Base
     def initialize(parent = nil)
       if parent.kind_of?(PlasmaScripting::Applet)
@@ -455,6 +506,24 @@ module Plasma
     end
   end
 
+  class SpinBox < Qt::Base
+    def initialize(parent = nil)
+      if parent.kind_of?(PlasmaScripting::Applet)
+        super(parent.applet_script.applet)
+      else
+        super
+      end
+    end
+
+    def range=(arg)
+      if arg.kind_of? Range
+        return super(arg.begin, arg.exclude_end?  ? arg.end - 1 : arg.end)
+      else
+        return super(arg)
+      end
+    end
+  end
+
   class SvgWidget < Qt::Base
     def initialize(*args)
       if args.length > 0 && args[0].kind_of?(PlasmaScripting::Applet)
@@ -479,6 +548,16 @@ module Plasma
     end
   end
 
+  class TextBrowser < Qt::Base
+    def initialize(parent = nil)
+      if parent.kind_of?(PlasmaScripting::Applet)
+        super(parent.applet_script.applet)
+      else
+        super
+      end
+    end
+  end
+
   class TextEdit < Qt::Base
     def initialize(parent = nil)
       if parent.kind_of?(PlasmaScripting::Applet)
@@ -489,6 +568,16 @@ module Plasma
     end
   end
 
+  class ToolButton < Qt::Base
+    def initialize(parent = nil)
+      if parent.kind_of?(PlasmaScripting::Applet)
+        super(parent.applet_script.applet)
+      else
+        super
+      end
+    end
+  end
+
   class ToolTipManager < Qt::Base
     def setContent(widget, data)
       if widget.kind_of?(PlasmaScripting::Applet)
@@ -579,6 +668,16 @@ module Plasma
     end
   end
 
+  class VideoWidget < Qt::Base
+    def initialize(parent = nil)
+      if parent.kind_of?(PlasmaScripting::Applet)
+        super(parent.applet_script.applet)
+      else
+        super
+      end
+    end
+  end
+
 end
 
 module Qt
