@@ -135,8 +135,8 @@ void AppletBrowserWidgetPrivate::initFilters()
     filterModel.addSeparator(i18n("Categories:"));
 
     foreach (const QString &category, Plasma::Applet::listCategories(application)) {
-        filterModel.addFilter(category,
-                              KCategorizedItemsViewModels::Filter("category", category));
+        filterModel.addFilter(i18n(category.toLocal8Bit()),
+                              KCategorizedItemsViewModels::Filter("category", category.toLower()));
     }
 }
 
@@ -199,7 +199,7 @@ void AppletBrowserWidgetPrivate::initRunningApplets()
         QObject::connect(containment, SIGNAL(appletRemoved(Plasma::Applet*)), q, SLOT(appletRemoved(Plasma::Applet*)));
 
         foreach (Applet *applet, containment->applets()) {
-            runningApplets[applet->name()]++;
+            runningApplets[applet->pluginName()]++;
         }
     }
 
@@ -268,7 +268,7 @@ void AppletBrowserWidget::addApplet()
 
 void AppletBrowserWidgetPrivate::appletAdded(Plasma::Applet *applet)
 {
-    QString name = applet->name();
+    QString name = applet->pluginName();
     //kDebug() << name;
 
     runningApplets[name]++;
@@ -315,7 +315,7 @@ void AppletBrowserWidget::destroyApplets(const QString &name)
     foreach (Containment *containment, c->containments()) {
         QList<Applet*> applets = containment->applets();
         foreach (Applet *applet, applets) {
-            if (applet->name() == name) {
+            if (applet->pluginName() == name) {
                 d->appletNames.remove(applet);
                 applet->disconnect(this);
                 applet->destroy();
@@ -327,6 +327,27 @@ void AppletBrowserWidget::destroyApplets(const QString &name)
     d->itemModel.setRunningApplets(name, 0);
 }
 
+/* This is just a wrapper around KAboutApplicationDialog that deletes
+the KAboutData object that it is associated with, when it is deleted.
+This is required to free memory correctly when KAboutApplicationDialog
+is called with a temporary KAboutData that is allocated on the heap.
+(see the code below, in AppletBrowserWidget::infoAboutApplet())
+*/
+class KAboutApplicationDialog2 : public KAboutApplicationDialog
+{
+public:
+    KAboutApplicationDialog2(KAboutData *ab, QWidget *parent = 0)
+    : KAboutApplicationDialog(ab, parent), m_ab(ab) {}
+
+    ~KAboutApplicationDialog2()
+    {
+        delete m_ab;
+    }
+
+private:
+    KAboutData *m_ab;
+};
+
 void AppletBrowserWidget::infoAboutApplet(const QString &name)
 {
     if (!d->containment) {
@@ -335,20 +356,20 @@ void AppletBrowserWidget::infoAboutApplet(const QString &name)
 
     KPluginInfo::List applets = Plasma::Applet::listAppletInfo();
     foreach (const KPluginInfo &info, applets) {
-        if (info.name() == name) {
-            KAboutData aboutData = KAboutData(info.name().toUtf8(),
+        if (info.pluginName() == name) {
+            KAboutData *aboutData = new KAboutData(info.name().toUtf8(),
                                               info.name().toUtf8(),
                                               ki18n(info.name().toUtf8()),
                                               info.version().toUtf8(), ki18n(info.comment().toUtf8()),
                                               info.fullLicense().key(), ki18n(QByteArray()), ki18n(QByteArray()), info.website().toLatin1(),
                                               info.email().toLatin1());
 
-            aboutData.setProgramIconName(info.icon());
+            aboutData->setProgramIconName(info.icon());
 
-            aboutData.addAuthor(ki18n(info.author().toUtf8()), ki18n(QByteArray()), info.email().toLatin1());
+            aboutData->addAuthor(ki18n(info.author().toUtf8()), ki18n(QByteArray()), info.email().toLatin1());
 
             //TODO should recycle this dialog if it is called up twice
-            KAboutApplicationDialog *aboutDialog = new KAboutApplicationDialog(&aboutData, this);
+            KAboutApplicationDialog *aboutDialog = new KAboutApplicationDialog2(aboutData, this);
             aboutDialog->show();
             break;
         }
@@ -429,7 +450,7 @@ void AppletBrowserPrivate::populateWidgetsMenu()
     widgetsMenu->addAction(action);
 
     KService::List offers = KServiceTypeTrader::self()->query("Plasma/PackageStructure");
-    foreach (const KService::Ptr service, offers) {
+    foreach (const KService::Ptr &service, offers) {
         //kDebug() << service->property("X-Plasma-ProvidesWidgetBrowser");
         if (service->property("X-Plasma-ProvidesWidgetBrowser").toBool()) {
             QAction *action = new QAction(KIcon("applications-internet"),
