@@ -19,7 +19,6 @@
 
 #include "desktopview.h"
 
-#include <QAction>
 #include <QFile>
 #include <QWheelEvent>
 #include <QCoreApplication>
@@ -29,6 +28,9 @@
 #include <KRun>
 #include <KToggleAction>
 #include <KWindowSystem>
+#include <NETRootInfo>
+#include <KAction>
+#include <KActionCollection>
 
 #include <Plasma/Applet>
 #include <Plasma/Corona>
@@ -53,9 +55,23 @@
 DesktopView::DesktopView(Plasma::Containment *containment, int id, QWidget *parent)
     : Plasma::View(containment, id, parent),
       m_dashboard(0),
-      m_dashboardFollowsDesktop(true)
+      m_dashboardFollowsDesktop(true),
+      m_init(false),
+      m_actions(shortcutActions(this))
 {
-    setFocusPolicy(Qt::NoFocus);
+    setAttribute(Qt::WA_TranslucentBackground, false);
+    //setCacheMode(QGraphicsView::CacheNone);
+
+    /*FIXME: Work around for a (maybe) Qt bug:
+     * QApplication::focusWidget() can't track focus change in QGraphicsProxyWidget
+     *   wrapped normal widget (eg. QLineEdit), if the QGraphicsView has called 
+     *   setFocusPolicy(Qt::NoFocus)
+     * I've created a bug report to Qt Software.
+     * There is also a simple reproduce program in case you're interested in this bug:
+     *   ftp://fearee:public@public.sjtu.edu.cn/reproduce.tar.gz
+     */
+
+    //setFocusPolicy(Qt::NoFocus);
 #ifdef Q_WS_WIN
     setWindowFlags(Qt::FramelessWindowHint);
     SetWindowPos(winId(), HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
@@ -65,12 +81,7 @@ DesktopView::DesktopView(Plasma::Containment *containment, int id, QWidget *pare
     setWindowFlags(windowFlags() | Qt::FramelessWindowHint);
 #endif
 
-    if (AppSettings::perVirtualDesktopViews()) {
-        kDebug() << "setting to desktop" << containment->desktop() + 1;
-        KWindowSystem::setOnDesktop(winId(), containment->desktop() + 1);
-    } else {
-        KWindowSystem::setOnAllDesktops(winId(), true);
-    }
+    checkDesktopAffiliation();
 
     KWindowSystem::setType(winId(), NET::Desktop);
     lower();
@@ -78,18 +89,15 @@ DesktopView::DesktopView(Plasma::Containment *containment, int id, QWidget *pare
     if (containment) {
         containment->enableAction("zoom in", false);
         containment->enableAction("add sibling containment", false);
+        connect(containment->corona(), SIGNAL(containmentAdded(Plasma::Containment *)), this, SLOT(containmentAdded(Plasma::Containment *)));
     }
-    //FIXME should we have next/prev or up/down/left/right or what?
-    QAction *action = new QAction(i18n("Next Activity"), this);
-    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-    action->setShortcut(QKeySequence("ctrl+shift+n"));
+
+    m_actions->addAssociatedWidget(this);
+    QAction *action = m_actions->action("next");
     connect(action, SIGNAL(triggered()), this, SLOT(nextContainment()));
-    addAction(action);
-    action = new QAction(i18n("Previous Activity"), this);
-    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-    action->setShortcut(QKeySequence("ctrl+shift+p"));
+    action = m_actions->action("prev");
     connect(action, SIGNAL(triggered()), this, SLOT(previousContainment()));
-    addAction(action);
+
 
     const int w = 25;
     QPixmap tile(w * 2, w * 2);
@@ -103,7 +111,12 @@ DesktopView::DesktopView(Plasma::Containment *containment, int id, QWidget *pare
     QBrush b(tile);
     setBackgroundBrush(tile);
 
-    adjustSize();
+    m_dashboardFollowsDesktop = (dashboardContainment() == 0);
+
+    // since Plasma::View has a delayed init we need to
+    // put a delay also for this call in order to be sure
+    // to have correct information set (e.g. screen())
+    QTimer::singleShot(0, this, SLOT(adjustSize()));
 
     Kephal::Screens *screens = Kephal::Screens::self();
     connect(screens, SIGNAL(screenResized(Kephal::Screen *, QSize, QSize)),
@@ -117,6 +130,41 @@ DesktopView::~DesktopView()
     delete m_dashboard;
 }
 
+void DesktopView::checkDesktopAffiliation()
+{
+    if (AppSettings::perVirtualDesktopViews()) {
+        m_desktop = containment() ? containment()->desktop() + 1 : -1;
+        kDebug() << "setting to desktop" << m_desktop;
+        KWindowSystem::setOnDesktop(winId(), m_desktop);
+    } else {
+        m_desktop = -1;
+        KWindowSystem::setOnAllDesktops(winId(), true);
+    }
+}
+
+KActionCollection* DesktopView::shortcutActions(QObject *parent)
+{
+    KActionCollection *actions = new KActionCollection(parent);
+    actions->setConfigGroup("Shortcuts-DesktopView");
+
+    //FIXME should we have next/prev or up/down/left/right or what?
+    KAction *action = actions->addAction("next");
+    action->setText(i18n("Next Activity"));
+    action->setShortcut(QKeySequence(Qt::ALT + Qt::Key_D, Qt::Key_Right));
+
+    action = actions->addAction("prev");
+    action->setText(i18n("Previous Activity"));
+    action->setShortcut(QKeySequence(Qt::ALT + Qt::Key_D, Qt::Key_Left));
+
+    actions->readSettings();
+    return actions;
+}
+
+void DesktopView::updateShortcuts()
+{
+    m_actions->readSettings();
+}
+
 void DesktopView::toggleDashboard()
 {
     if (!m_dashboard) {
@@ -126,23 +174,61 @@ void DesktopView::toggleDashboard()
 
         m_dashboardFollowsDesktop = true;
         KConfigGroup cg = config();
-        Plasma::Containment *dc = containment();
-        int containmentId = cg.readEntry("DashboardContainment", 0);
-        if (containmentId > 0) {
-            foreach (Plasma::Containment *c, containment()->corona()->containments()) {
-                if ((int)c->id() == containmentId) {
-                    dc = c;
-                    m_dashboardFollowsDesktop = false;
-                    break;
-                }
-            }
+        Plasma::Containment *dc = dashboardContainment();
+        if (dc) {
+            dc->resize(size());
+            dc->enableAction("remove", false);
+            m_dashboardFollowsDesktop = false;
+        } else {
+            dc = containment();
         }
-        m_dashboard = new DashboardView(dc, 0);
+
+        m_dashboard = new DashboardView(dc, this);
         m_dashboard->addActions(actions());
     }
 
+    //If a separate dashboard is used we must use the screen of this containment instead of the dashboard one
+    if (!m_dashboardFollowsDesktop && containment()) {
+        m_dashboard->setGeometry(Kephal::ScreenUtils::screenGeometry(containment()->screen()));
+    }
     m_dashboard->toggleVisibility();
-    kDebug() << "toggling dashboard for screen" << screen() << m_dashboard->isVisible();
+    //kDebug() << "toggling dashboard for screen" << screen() << m_dashboard->isVisible();
+}
+
+Plasma::Containment *DesktopView::dashboardContainment() const
+{
+    KConfigGroup cg = config();
+    Plasma::Containment *dc = 0;
+    int containmentId = cg.readEntry("DashboardContainment", 0);
+
+    if (containmentId > 0) {
+        foreach (Plasma::Containment *c, containment()->corona()->containments()) {
+            if ((int)c->id() == containmentId) {
+                dc = c;
+                break;
+            }
+        }
+    }
+
+    return dc;
+}
+
+void DesktopView::setDashboardContainment(Plasma::Containment *containment)
+{
+    if (containment) {
+        config().writeEntry("DashboardContainment", containment->id());
+        if (m_dashboard) {
+            m_dashboard->setContainment(containment);
+        }
+    } else {
+        if (dashboardContainment()) {
+            dashboardContainment()->destroy(false);
+        }
+        config().writeEntry("DashboardContainment", 0);
+        if (m_dashboard) {
+            m_dashboard->setContainment(View::containment());
+        }
+    }
 }
 
 void DesktopView::screenResized(Kephal::Screen *s)
@@ -167,8 +253,10 @@ void DesktopView::adjustSize()
     QRect geom = Kephal::ScreenUtils::screenGeometry(screen());
     kDebug() << "screen" << screen() << "geom" << geom;
     setGeometry(geom);
-    containment()->resize(geom.size());
-    kDebug() << "Containment's geom after resize" << containment()->geometry(); 
+    if (containment()) {
+        containment()->resize(geom.size());
+        kDebug() << "Containment's geom after resize" << containment()->geometry();
+    }
 
     if (m_dashboard) {
         m_dashboard->setGeometry(geom);
@@ -182,13 +270,30 @@ bool DesktopView::isDashboardVisible() const
     return m_dashboard && m_dashboard->isVisible();
 }
 
+bool DesktopView::dashboardFollowsDesktop() const
+{
+    return m_dashboardFollowsDesktop;
+}
+
+void DesktopView::setDashboardFollowsDesktop(bool follow)
+{
+    m_dashboardFollowsDesktop = follow;
+
+    if (follow) {
+        config().writeEntry("DashboardContainment", containment()->id());
+    } else {
+        config().writeEntry("DashboardContainment", 0);
+    }
+}
+
 void DesktopView::setContainment(Plasma::Containment *containment)
 {
     Plasma::Containment *oldContainment = this->containment();
-    if (containment == oldContainment) {
+    if (m_init && containment == oldContainment) {
         return;
     }
 
+    m_init = true;
     Plasma::ZoomLevel zoomLevel = PlasmaApp::self()->desktopZoomLevel();
     if (zoomLevel == Plasma::DesktopZoom && containment) {
         //make sure actions are up-to-date
@@ -201,15 +306,110 @@ void DesktopView::setContainment(Plasma::Containment *containment)
         m_dashboard->setContainment(containment);
     }
 
-    if (oldContainment && zoomLevel == Plasma::DesktopZoom) {
-        //make sure actions are up-to-date
-        oldContainment->enableAction("zoom in", false);
-        oldContainment->enableAction("add sibling containment", false);
+    if (oldContainment) {
+        disconnect(oldContainment, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxOpened()));
+        disconnect(oldContainment, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxClosed()));
+        if (zoomLevel == Plasma::DesktopZoom) {
+            //make sure actions are up-to-date
+            oldContainment->enableAction("zoom in", false);
+            oldContainment->enableAction("add sibling containment", false);
+        }
+    }
+
+    if (containment) {
+        connect(containment, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxOpened()));
     }
 
     View::setContainment(containment);
 }
 
+void DesktopView::toolBoxOpened()
+{
+    if (isDashboardVisible()) {
+        return;
+    }
+
+#ifndef Q_WS_WIN
+    NETRootInfo info(QX11Info::display(), NET::Supported);
+    if (!info.isSupported(NET::WM2ShowingDesktop)) {
+        return;
+    }
+#endif
+    Plasma::Containment *c = containment();
+    disconnect(c, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxOpened()));
+    connect(c, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxClosed()));
+    connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)),
+            this, SLOT(showDesktopUntoggled(WId)));
+
+#ifndef Q_WS_WIN
+    info.setShowingDesktop(true);
+#endif
+}
+
+void DesktopView::toolBoxClosed()
+{
+    if (isDashboardVisible()) {
+        return;
+    }
+
+#ifndef Q_WS_WIN
+    NETRootInfo info(QX11Info::display(), NET::Supported);
+    if (!info.isSupported(NET::WM2ShowingDesktop)) {
+        return;
+    }
+#endif
+
+    Plasma::Containment *c = containment();
+    disconnect(c, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxClosed()));
+    disconnect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)),
+               this, SLOT(showDesktopUntoggled(WId)));
+    connect(c, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxOpened()));
+
+#ifndef Q_WS_WIN
+    info.setShowingDesktop(false);
+#endif
+}
+
+void DesktopView::showDesktopUntoggled(WId id)
+{
+    if (isDashboardVisible()) {
+        return;
+    }
+
+    disconnect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)),
+               this, SLOT(showDesktopUntoggled(WId)));
+
+    Plasma::Containment *c = containment();
+    if (c) {
+        disconnect(c, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxClosed()));
+        connect(c, SIGNAL(toolBoxToggled()), this, SLOT(toolBoxOpened()));
+        containment()->setToolBoxOpen(false);
+    }
+
+#ifndef Q_WS_WIN
+    Plasma::ZoomLevel zoomLevel = PlasmaApp::self()->desktopZoomLevel();
+    NETRootInfo info(QX11Info::display(), NET::Supported);
+    if (zoomLevel == Plasma::DesktopZoom && info.isSupported(NET::WM2ShowingDesktop)) {
+        info.setShowingDesktop(false);
+    }
+    KWindowSystem::activateWindow(id);
+#endif
+}
+
+void DesktopView::containmentAdded(Plasma::Containment *c)
+{
+    Q_UNUSED(c)
+
+    if (transform().m11() != 1) {
+        QTimer::singleShot(0, this, SLOT(syncSceneRect()));
+    }
+}
+
+void DesktopView::syncSceneRect()
+{
+    setSceneRect(QRectF(0, 0, scene()->sceneRect().right(), scene()->sceneRect().bottom() + TOOLBOX_MARGIN));
+}
+
 void DesktopView::zoomIn(Plasma::ZoomLevel zoomLevel)
 {
     if (zoomLevel == Plasma::DesktopZoom) {
@@ -230,6 +430,8 @@ void DesktopView::zoomIn(Plasma::ZoomLevel zoomLevel)
             }
             setSceneRect(containment()->geometry());
         }
+
+        toolBoxClosed();
     } else if (zoomLevel == Plasma::GroupZoom) {
         qreal factor = Plasma::scalingFactor(zoomLevel);
         factor = factor / matrix().m11();
@@ -270,7 +472,7 @@ void DesktopView::wheelEvent(QWheelEvent* event)
         return;
     }
 
-    QGraphicsView::wheelEvent(event);
+    Plasma::View::wheelEvent(event);
 }
 
 // This function is reimplemented from QGraphicsView to work around the problem
@@ -283,6 +485,11 @@ void DesktopView::wheelEvent(QWheelEvent* event)
 // CompositionMode_Source.
 void DesktopView::drawBackground(QPainter *painter, const QRectF &rect)
 {
+    Plasma::ZoomLevel zoomLevel = PlasmaApp::self()->desktopZoomLevel();
+    if (zoomLevel == Plasma::DesktopZoom) {
+        return;
+    }
+
     const QPainter::CompositionMode savedMode = painter->compositionMode();
     const QBrush brush = backgroundBrush();
 
@@ -299,34 +506,35 @@ void DesktopView::drawBackground(QPainter *painter, const QRectF &rect)
         painter->setCompositionMode(QPainter::CompositionMode_Source);
         painter->drawTiledPixmap(r, texture);
         painter->setCompositionMode(savedMode);
-        return;
+        break;
     }
 
     case Qt::SolidPattern:
         painter->setCompositionMode(QPainter::CompositionMode_Source);
         painter->fillRect(rect.toAlignedRect(), brush.color());
         painter->setCompositionMode(savedMode);
-        return;
+        break;
 
     default:
         QGraphicsView::drawBackground(painter, rect);
+        break;
     }
 }
 
-void DesktopView::screenOwnerChanged(int wasScreen, int isScreen, Plasma::Containment* containment)
+void DesktopView::screenOwnerChanged(int wasScreen, int isScreen, Plasma::Containment* newContainment)
 {
-    kDebug() << "was:" << wasScreen << "is:" << isScreen << "my screen:" << screen() << "containment:" << (QObject*)containment << "myself:" << (QObject*)this;
-    if (PlasmaApp::isPanelContainment(containment)) {
+    //kDebug() << "was:" << wasScreen << "is:" << isScreen << "my screen:" << screen() << "containment:" << (QObject*)newContainment << newContainment->activity() << "myself:" << (QObject*)this <<"containment desktop:"<<newContainment->desktop() << "my desktop:"<<containment()->desktop();
+    if (PlasmaApp::isPanelContainment(newContainment)) {
         // we don't care about panel containments changing screens on us
         return;
     }
 
-    if (wasScreen == screen() && this->containment() == containment) {
-            setContainment(0);
+    if (wasScreen == screen() && this->containment() == newContainment) {
+        setContainment(0);
     }
 
-    if (isScreen == screen()) {
-        setContainment(containment);
+    if (isScreen > -1 && isScreen == screen() && (!AppSettings::perVirtualDesktopViews() || newContainment->desktop() == m_desktop - 1) ) {
+        setContainment(newContainment);
     }
 }
 
